/*  Copyright (C) 2010-2024 Institute of Aerospace Systems, RWTH Aachen University - All rights reserved.
    This file is part of UNICADO.

    UNICADO is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    UNICADO is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with UNICADO.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "bwbCalculatePolarConfig.h"

#include <runtimeInfo/runtimeInfo.h>
#include <moduleBasics/module.h>

#include <algorithm>
#include <exception>
#include <iomanip>
#include <string>
#include <vector>
#include <utility>  // make_pair

#ifdef _WIN32
    #include <fileapi.h>
    #include <windows.h>
#endif  // _WIN32

using std::endl;

bwbCalculatePolarConfig::bwbCalculatePolarConfig(const node& config)
    :
    // lifting line settings
    CLModeAirfoilSelection(EndnodeReadOnly<std::string>("program_settings/CLModeAirfoilSelection")),
    pathToLiftingLine(EndnodeReadOnly<std::string>("program_settings/LiftingLine/FolderPath")),
    stepWidthCL(EndnodeReadOnly<double>("LiftingLineForTAW/stepWidthCL")),
    // other settings
    setFlightConditionsMode(EndnodeReadOnly<std::string>("program_settings/FlightConditions/AdaptMachNumbersToCruiseReq")),
    extrapolationMargin(EndnodeReadOnly<double>("program_settings/FlightConditions/PolarAttributes/extrapolation_margin")),
    allowGrideChange(EndnodeReadOnly<bool>("program_settings/FlightConditions/PolarAttributes/allow_grid_change")),
    numberFlightConditions(EndnodeReadOnly<int>("program_settings/FlightConditions/numberFlightConditions")),
    // lifting line for TAW pitching moment corrections
    delta_CM0(EndnodeReadOnly<double>("LiftingLineForTAW/PitchingMoment/delta_CM0")),
    delta_dCMdCL(EndnodeReadOnly<double>("LiftingLineForTAW/PitchingMoment/delta_dCMdCL")),
    // lifiting line for TAW induced drag corrections
    indDragCtCorrForCalibration(EndnodeReadOnly<double>("LiftingLineForTAW/InducedDragCorrection/indDragCtCorrForCalibration")),
    factorIndDragCleanPolar(EndnodeReadOnly<double>("LiftingLineForTAW/InducedDragCorrection/factorIndDragCleanPolar")),
    // visc drag raymer transition parameters
    manualTransition(EndnodeReadOnly<bool>("program_settings/ViscDragRaymer/ManualTransition")),
    manualTransitionWing(EndnodeReadOnly<double>("program_settings/ViscDragRaymer/TransitionLocationWing")),
    manualTransitionFin(EndnodeReadOnly<double>("program_settings/ViscDragRaymer/TransitionLocationFin")),
    // visc drag raymer calibration parameters
    doViscDragCalibration(EndnodeReadOnly<bool>("program_settings/ViscDragRaymer/UseCalibration")),
    CDvisc_CDsum(EndnodeReadOnly<double>("program_settings/ViscDragRaymer/CalibrationHighMa/CDSum")),
    CDvisc_CLfact(EndnodeReadOnly<double>("program_settings/ViscDragRaymer/CalibrationHighMa/CLFact")),
    CDvisc_CLexp(EndnodeReadOnly<double>("program_settings/ViscDragRaymer/CalibrationHighMa/CLExp")),
    CDvisc_CDsum_lowMa(EndnodeReadOnly<double>("program_settings/ViscDragRaymer/CalibrationLowMa/CDSum")),
    CDvisc_CLfact_lowMa(EndnodeReadOnly<double>("program_settings/ViscDragRaymer/CalibrationLowMa/CLFact")),
    CDvisc_CLexp_lowMa(EndnodeReadOnly<double>("program_settings/ViscDragRaymer/CalibrationLowMa/CLExp")),
    // wave drag mason
    numberWingStrips(EndnodeReadOnly<int>("program_settings/WaveDragMason/NumberWingStrips")),
    useCostomReferenceAngleForSweep(EndnodeReadOnly<bool>("program_settings/WaveDragMason/UseCostomReferenceAngleForSweep")),
    doWaveDragMasonCalibration(EndnodeReadOnly<bool>("program_settings/WaveDragMason/UseCalibration")),
    doWaveDragMasonDragCountCorrection(EndnodeReadOnly<bool>("program_settings/WaveDragMason/DoDragCountCorrection")),
    Ka(EndnodeReadOnly<double>("program_settings/WaveDragMason/Ka")),
    maximumSegmentSweep(EndnodeReadOnly<double>("program_settings/WaveDragMason/MaximumSegmentSweep")),
    customSweepPosition(EndnodeReadOnly<double>("program_settings/WaveDragMason/CustomSweepAngle")),
    CDwave_CLFact(EndnodeReadOnly<double>("program_settings/WaveDragMason/CLFact")),
    CDwave_CLExp(EndnodeReadOnly<double>("program_settings/WaveDragMason/CLExp")),
    deltaWaveDragCountWing(EndnodeReadOnly<double>("program_settings/WaveDragMason/deltaWaveDragWing")),
    deltaTotalDragHL(EndnodeReadOnly<double>("program_settings/SemiEmpiricalHighLiftAdaptions/deltaTotalDrag")),
    factorDragHL(EndnodeReadOnly<double>("program_settings/SemiEmpiricalHighLiftAdaptions/factorDrag")) {
    CLModeAirfoilSelection.read(config);
    stepWidthCL.read(config);
    setFlightConditionsMode.read(config);
    extrapolationMargin.read(config);
    allowGrideChange.read(config);
    numberFlightConditions.read(config);
    pathToLiftingLine.read(config);
    manualTransition.read(config);
    manualTransitionWing.read(config);
    manualTransitionFin.read(config);
    doViscDragCalibration.read(config);
    CDvisc_CDsum.read(config);
    CDvisc_CLfact.read(config);
    CDvisc_CLexp.read(config);
    CDvisc_CDsum_lowMa.read(config);
    CDvisc_CLfact_lowMa.read(config);
    CDvisc_CLexp_lowMa.read(config);
    useCostomReferenceAngleForSweep.read(config);
    doWaveDragMasonCalibration.read(config);
    doWaveDragMasonDragCountCorrection.read(config);
    Ka.read(config);
    numberWingStrips.read(config);
    maximumSegmentSweep.read(config);
    customSweepPosition.read(config);
    CDwave_CLFact.read(config);
    CDwave_CLExp.read(config);
    deltaWaveDragCountWing.read(config);
    delta_CM0.read(config);
    delta_dCMdCL.read(config);
    indDragCtCorrForCalibration.read(config);
    factorIndDragCleanPolar.read(config);
    liftingLineFolderPath = pathToLiftingLine.value();
    deltaTotalDragHL.read(config);
    factorDragHL.read(config);
    myRuntimeInfo->out << "config: " << liftingLineFolderPath << endl;
    if (setFlightConditionsMode.value() == "mode_0") {
        myRuntimeInfo->info << "set flight conditions from config xml" << endl;
        for (size_t i(0); i < numberFlightConditions.value(); i++) {
            readflightConditions(config, i);
        }
    } else if (setFlightConditionsMode.value() == "mode_1") {
        myRuntimeInfo->info << "Hab vergessen was hier rein sollte" << endl;
    }
    CGPositions = {"design", "forward", "aft"};
}

void bwbCalculatePolarConfig::readflightConditions(const node& config, size_t ID) {
    EndnodeReadOnly flightConditionExtrapolationMargin(EndnodeReadOnly<double>("program_settings/FlightConditions/FlightCondition@" + num2Str(ID) + "/extrapolation_margin"));
    EndnodeReadOnly flightConditionAltitude(EndnodeReadOnly<double>("program_settings/FlightConditions/FlightCondition@" + num2Str(ID) + "/Altitude"));
    EndnodeReadOnly flightConditionMach(EndnodeReadOnly<double>("program_settings/FlightConditions/FlightCondition@" + num2Str(ID) + "/Mach"));
    EndnodeReadOnly flightConditionAllowGridChange(EndnodeReadOnly<bool>("program_settings/FlightConditions/FlightCondition@" + num2Str(ID) + "/allow_grid_change"));
    flightConditionAllowGridChange.read(config);
    flightConditionAltitude.read(config);
    flightConditionMach.read(config);
    flightConditionExtrapolationMargin.read(config);
    flightConditions.push_back(flightCondition());
    flightConditions.back().Mach = flightConditionMach.value();
    flightConditions.back().Altitude = flightConditionAltitude.value();
    flightConditions.back().extrapolationMargin = flightConditionExtrapolationMargin.value();
    flightConditions.back().allowGridChange = flightConditionAllowGridChange.value();
    machNumbers.push_back(flightConditions.back().Mach);
}
