/**
 * @file cruise_step.h
 * @author Gerrit Pauls (gerrit.pauls@tuhh.de)
 * @brief 
 * @version 0.1
 * @date 2024-08-30
 * 
 * @copyright Copyright (c) 2024
 * 
 */

/* List of references
    [1] Airbus customer services, Flight Operations Support & Line Assistance: Getting to grips with the cost index, Issue II, May 1998, p.7
*/

#ifndef CREATE_MISSION_XML_SRC_LIBS_MISSION_STEPS_CRUISE_STEP_H_
#define CREATE_MISSION_XML_SRC_LIBS_MISSION_STEPS_CRUISE_STEP_H_

#include <string>
#include <aixml/endnode.h>

/** \brief Class for flight steps on cruising flight
 */
class cruise_step {
 public:
    /* Constructor and destructor */
    /** \brief Constructor for class cruise_step
     * \param a_mode: reference to a flight mode of the cruise step
     * \param a_rating: reference to the rating of the cruise step
     * \param a_derate: derate of the cruise step [-]
     * \param a_config: reference to the configuration of the cruise step
     * \param an_end_altitude: altitude at the end of the cruise step [m]
     * \param auto_select_FL: switch for automatic select of the cruise altitude of the cruise step
     * \param round_to_regular_flight_level: switch for rounding to regular flight level of the cruise step
     * \param a_rate_of_climb_limit: rate of climb of the cruise step [m/s]
     * \param an_end_CAS: calibrated airspeed at the end of the cruise step [m/s]
     * \param an_end_Mach: Mach number at the end of the cruise step [-]
     * \param a_relative_segment_length: relative segment length of the cruise step [-]
     */
    cruise_step(const std::string &a_mode, const std::string &a_rating, double a_derate, const std::string &a_config, double an_end_altitude,
               bool auto_select_FL, bool round_to_regular_flight_level,
               double auto_climb_step_altitude_change, bool a_flight_management_system,
               double a_rate_of_climb_limit, double an_end_CAS, double an_end_Mach, double a_relative_segment_length);
    /** \brief Destructor for class cruise_step
     */
    virtual ~cruise_step();

    /** \brief function to write cruise steps into mission file
     * \details It checks the given values and updates the endnodes within mission_file
     * \param subpath: path where the endnode shall write data
     * \param mission_file: xml node of the given mission file
     */
    void write_cruise_steps(std::string subpath, node* mission_file);

 private:
    /* Attributes */
    Endnode<double> derate; /**< Derate of the cruise step [-] */
    Endnode<std::string> mode; /**< Flight mode of the cruise step */
    Endnode<std::string> rating; /**< Rating of the cruise step */
    Endnode<std::string> configuration; /**< Configuration of the cruise step */
    Endnode<double> rate_of_climb_limit; /**< Rate of climb of the cruise step [m/s] */
    Endnode<double> end_CAS; /**< Calibrated air speed at the end of the cruise step [m/s] */
    Endnode<double> end_Mach; /**< Mach number at the end of the cruise step [-] */
    Endnode<double> end_altitude; /**< FLight altitude at the end of the cruise step [m] */
    Endnode<bool> flight_management_system; /**< Flight management system of the cruise step */
    Endnode<double> cost_index; /**< Cost index [kg/min], scaled 0 to 999 according to Sperry/Honeywell [1] */
    Endnode<double> relative_segment_length; /**< Relative segment length of the cruise step [-] */
    Endnode<bool> round_to_regular_flight_level; /**< Switch for rounding the optimum flight level of the cruise step */
    Endnode<bool> auto_select_optimum_flight_level; /**< Switch for automatic selection of the optimum flight level of the cruise step */
    Endnode<double> auto_climb_step_height; /**< Altitude step at cruising flight if automatic altitude adaption [m] */
};

#endif // CREATE_MISSION_XML_SRC_LIBS_MISSION_STEPS_CRUISE_STEP_H_
