/**
 * @file steps_setter.cpp
 * @author Gerrit Pauls (gerrit.pauls@tuhh.de)
 * @brief sets mission steps for create mission XML's standard mission
 * @version 0.1
 * @date 2024-08-30
 * 
 * @copyright Copyright (c) 2024
 * 
 */

#include <aixml/endnode.h>
#include <string>
#include <moduleBasics/module.h>
#include <unordered_map>
#include "standard_mission/standard_mission.h"

void standard_mission::set_departure_profile(standard_mission::mission_parameters* mission) {
    /* Create objects for take off */
    if (mission->takeoff_procedure == 0) { // This is for a standard take-off procedure, others can be added later
        /* Transfer parameters: Mode, rating, derate, config., endAlt., rateOfclimb, endCAS*/
        mission->departure_steps.push_back(departure_step("takeoff", "takeoff", 1., "takeoff", NAN, NAN, NAN));
        mission->departure_steps.push_back(departure_step("climb", mission->climb_thrust_setting, 1.,
                                        "takeoff_landing_gear_retracted", convertUnit(FOOT, METER, 1500.), mission->maximum_rate_of_climb, NAN));
        mission->departure_steps.push_back(departure_step("climb", mission->climb_thrust_setting, 1.,
                                        "takeoff_landing_gear_retracted", convertUnit(FOOT, METER, 3000.), mission->maximum_rate_of_climb, NAN));
        mission->departure_steps.push_back(departure_step("accelerate", mission->climb_thrust_setting, 1.,
                                        "climb", NAN, convertUnit(FOOTPERMINUTE, METERPERSECOND, 1000.),  convertUnit(KNOTS, METERPERSECOND, 210.)));
        mission->departure_steps.push_back(departure_step("accelerate", mission->climb_thrust_setting, 1.,
                                        "clean", NAN, convertUnit(FOOTPERMINUTE, METERPERSECOND, 1200.), mission->CAS_ATC_limit_climb));
        mission->departure_steps.push_back(departure_step("climb", mission->climb_thrust_setting, 1., "clean",
                                        convertUnit(FOOT, METER, 10000.), mission->maximum_rate_of_climb, NAN));
    } else if (mission->takeoff_procedure == 1) { // The ICAO-A take-off procedure has not yet been implemented, error message
        throwError(__FILE__, __func__, __LINE__, "ICAO-A Take-Off Procedure not yet implemented.");
    } else if (mission->takeoff_procedure == 2) { // The ICAO-B take-off procedure has not yet been implemented, error message
        throwError(__FILE__, __func__, __LINE__, "ICAO-B Take-Off Procedure not yet implemented.");
    } else if (mission->takeoff_procedure == 3) { // Standard_19-Seat_Commuter take-off procedure
        /* Parameters:                                 #Step, Mode,         rating,     derate, config.,              endAlt., rateOfclimb, endCAS */
        mission->departure_steps.push_back(departure_step("takeoff", "takeoff", 1., "takeoff", NAN, NAN, NAN));
        mission->departure_steps.push_back(departure_step("climb", mission->climb_thrust_setting, 1.,
                                        "takeoff_landing_gear_retracted", convertUnit(FOOT, METER, 200.), mission->maximum_rate_of_climb, NAN));
        mission->departure_steps.push_back(departure_step("accelerate", mission->climb_thrust_setting, 1., "clean", NAN, 1200.,
                                        mission->CAS_ATC_limit_climb));
        mission->departure_steps.push_back(departure_step("climb",
                                        mission->climb_thrust_setting, 1., "clean", convertUnit(FOOT, METER, 10000.), mission->maximum_rate_of_climb,  NAN));
    } else { // Unknown take-off procedure, error message
        myRuntimeInfo->err << "Unkown take-Off procedure in config file. Abort!" << std::endl;
        exit(1);
    }
}

void standard_mission::set_cruise_profile(standard_mission::mission_parameters* mission) {
    /* Create objects for cruise */
    if (mission->CAS_over_flight_level_100_climb >= mission->CAS_ATC_limit_climb) {
        /* If 2nd CAS (e.g. 300KTAS) larger than 1st CAS (e.g. 250KTAS) -> create segment for acceleration to second CAS ("change_speed_to_CAS") */
        mission->cruise_steps.push_back(cruise_step("change_speed_to_CAS",
                                            mission->climb_thrust_setting, 1., "clean", NAN, mission->auto_select_flight_level,
                                            mission->round_to_regular_flight_level, mission->auto_climb_altitude_steps, false,
                                            convertUnit(FOOTPERMINUTE, METERPERSECOND, 300.), mission->CAS_over_flight_level_100_climb, NAN, NAN));
    } else {
        /* If 2nd CAS (e.g. 300KTAS) smaller than 1st CAS -> create segment for deceleration to second CAS ("change_speed_to_CAS") */
        mission->cruise_steps.push_back(cruise_step("change_speed_to_CAS",
                                            mission->climb_thrust_setting, 1., "clean", NAN, mission->auto_select_flight_level,
                                            mission->round_to_regular_flight_level, mission->auto_climb_altitude_steps, false,
                                            mission->maximum_rate_of_climb, mission->CAS_over_flight_level_100_climb, NAN, NAN));
    }
    if (mission->mission_type == "requirements_mission") { // Mission type: requirement mission
        /* For requirement mission only one additional cruise step --> climb to ceiling requirements! After that, the requirement mission can be aborted */
        mission->cruise_steps.push_back(cruise_step("climb_to_ceiling",
                                            mission->climb_thrust_setting, 1., "clean",
                                            mission->cruise_step_segments.front().altitude,
                                            mission->auto_select_initial_cruise_altitude, mission->round_to_regular_flight_level,
                                            mission->auto_climb_altitude_steps, false, NAN,
                                            NAN, mission->initial_cruise_mach.value(), NAN));
    } else { // Mission type: design/study mission
        /* For design and study mission, additional cruise segments have to be created. climb to ICA and accelerate to crossover Mach number */
        mission->cruise_steps.push_back(cruise_step("climb_to_cruise", mission->climb_thrust_setting, 1., "clean",
                                            mission->cruise_step_segments.front().altitude,
                                            mission->auto_select_initial_cruise_altitude, mission->round_to_regular_flight_level,
                                            mission->auto_climb_altitude_steps, false, mission->maximum_rate_of_climb, NAN,
                                            mission->mach_climb, NAN));
        /* accelerate to initial cruise Mach number */
        mission->cruise_steps.push_back(cruise_step("change_speed_to_Mach", mission->climb_thrust_setting, 1., "clean", NAN,
                                            mission->auto_select_flight_level, mission->round_to_regular_flight_level,
                                            mission->auto_climb_altitude_steps, false, NAN, NAN, mission->initial_cruise_mach.value(), NAN));
        std::string mode;
        std::string rating;
        double rate_of_climb_limit;
        double relative_segment_length;
        double end_altitude;
        /* Create additional steps (according to definition in XML file) */
        int cruise_step_blocks(3 + (2 * mission->cruise_step_segments.size() - 1) + 1);

        for (int i = 4; i <= cruise_step_blocks - 1; i++) {
            if (i % 2 == 0) { // Mode: cruise
                mode = "cruise";
                rating = "cruise";
                if (mission->auto_select_flight_level) {
                    rate_of_climb_limit = mission->auto_rate_of_climb_steps;
                } else {
                    rate_of_climb_limit = NAN;
                }
                relative_segment_length = mission->cruise_step_segments[i / 2 - 2].relative_segment_length;
                end_altitude = NAN;
            } else { // Mode: change flight level...
                if (mission->auto_rate_of_climb_steps > 0.) { // ... at a constant rate of climb
                    rate_of_climb_limit = mission->auto_rate_of_climb_steps;
                    mode = "change_flight_level_constant_ROC";
                } else { // ... at an undefined rate of climb
                    rate_of_climb_limit = NAN;
                    mode = "change_flight_level";
                }
                rating = mission->climb_thrust_setting;
                relative_segment_length = NAN;
                end_altitude =  mission->cruise_step_segments[(i - 1) / 2 - 1].altitude;
            }
            mission->cruise_steps.push_back(cruise_step(mode, rating, 1., "clean", end_altitude,
                                            mission->auto_select_flight_level, mission->round_to_regular_flight_level,
                                            mission->auto_climb_altitude_steps, false, rate_of_climb_limit, NAN, NAN, relative_segment_length));
        }
        mission->cruise_steps.push_back(cruise_step("descend_to_approach", "idle", 1., "clean", convertUnit(FOOT, METER, 10000.),
                                            mission->auto_select_flight_level, mission->round_to_regular_flight_level,
                                            mission->auto_climb_altitude_steps, false, NAN,
                                            mission->CAS_over_flight_level_100_descent, NAN, NAN));
    }
}

void standard_mission::set_approach_profile(standard_mission::mission_parameters* mission) {
    /* Create objects for approach - by now, no rate of climb value used in those procedures, but are kept in for possible adaption */
    if (mission->approach_procedure == 0) { // Standard approach procedure
        /* Get speeds */
        double v_1_temp(mission->CAS_ATC_limit_descent);
        double v_2_temp(std::max(convertUnit(KNOTS, METERPERSECOND, 170.), mission->approach_speed + convertUnit(KNOTS, METERPERSECOND, 5.)));
        double v_3_temp(mission->approach_speed + convertUnit(KNOTS, METERPERSECOND, 5.));
        double v_4_temp(mission->approach_speed);
        /* Creation of the approach steps: mode, rating, derate, config., endAlt, rate_of_climb_limit, endCAS, GD, glidePath */
        mission->approach_steps.push_back(approach_step("change_speed", "idle", 1., "clean", NAN, NAN, v_1_temp, convertUnit(FOOT, METER, 5000.), 0.));
        mission->approach_steps.push_back(approach_step("descend", "cruise", 1., "clean", convertUnit(FOOT, METER, 3000.), NAN, NAN, NAN, convertUnit(DEGREE, RADIAN, -3.)));
        mission->approach_steps.push_back(approach_step("change_speed", "idle", 1., "approach", NAN, NAN, v_2_temp, convertUnit(FOOT, METER, 10000.), 0.));
        mission->approach_steps.push_back(approach_step("level_glide_slope_interception", "cruise", 1., "approach_landing_gear_out", NAN, NAN, NAN, NAN, NAN));
        mission->approach_steps.push_back(approach_step("change_speed", "idle", 1., "approach_landing_gear_out",
                                            NAN, NAN, v_3_temp,  convertUnit(FOOT, METER, 6000.), convertUnit(DEGREE, RADIAN, -3.)));
        mission->approach_steps.push_back(approach_step("descend", "cruise", 1., "approach_landing_gear_out",
                                            convertUnit(FOOT, METER, 1500.), NAN, NAN, NAN, convertUnit(DEGREE, RADIAN, -3.)));
        mission->approach_steps.push_back(approach_step("change_speed", "idle", 1., "landing",
                                            NAN, NAN, v_4_temp, convertUnit(FOOT, METER, 5000.), convertUnit(DEGREE, RADIAN, -3.)));
        mission->approach_steps.push_back(approach_step("descend", "cruise", 1., "landing", convertUnit(FOOT, METER, 1000.), NAN, NAN, NAN, convertUnit(DEGREE, RADIAN, -3.)));
        mission->approach_steps.push_back(approach_step("descend", "cruise", 1., "landing", convertUnit(FOOT, METER, 50.), NAN, NAN, NAN, convertUnit(DEGREE, RADIAN, -3.)));
        mission->approach_steps.push_back(approach_step("landing", "takeoff", 1., "landing", 0., NAN, NAN, NAN, convertUnit(DEGREE, RADIAN, -3.)));
    } else if (mission->approach_procedure == 1) { // The CDA approach procedure has not yet been implemented, error message
        myRuntimeInfo->err << "Continuous descent approach procedure not yet implemented. Abort!" << std::endl;
        exit(1);
    } else if (mission->approach_procedure == 2) { // The CDA-Steep approach procedure has not yet been implemented, error message
        myRuntimeInfo->err << "Steep continuous descent approach procedure not yet implemented. Abort!" << std::endl;
        exit(1);
    } else if (mission->approach_procedure == 3) { // Standard_19-Seat-Commuter approach procedure
        /* Get speeds */
        double v_1_temp(mission->CAS_ATC_limit_descent);
        double v_2_temp(mission->approach_speed);
        if (fabs(mission->CAS_ATC_limit_descent - mission->CAS_over_flight_level_100_descent) > ACCURACY_LOW) {
            /* Creation of the approach steps: mode , rating, derate, config., endAlt, rate_of_climb_limit, endCAS, GD, glidePath */
            mission->approach_steps.push_back(approach_step("change_speed", "idle", 1., "clean", NAN, NAN, v_1_temp, convertUnit(FOOT, METER, 5000), 0.));
            mission->approach_steps.push_back(approach_step("descend", "cruise", 1., "clean", convertUnit(FOOT, METER, 2000.),  NAN, NAN, NAN, convertUnit(DEGREE, RADIAN, -3.)));
            mission->approach_steps.push_back(approach_step("change_speed", "idle", 1., "approach",
                                                            NAN, NAN, v_2_temp, convertUnit(FOOT, METER, 6200.), convertUnit(DEGREE, RADIAN, -3.)));
            mission->approach_steps.push_back(approach_step("descend", "cruise", 1., "landing", convertUnit(FOOT, METER, 50.), NAN, NAN, NAN, convertUnit(DEGREE, RADIAN, -3.)));
            mission->approach_steps.push_back(approach_step("landing", "takeoff", 1., "landing", 0., NAN, NAN, NAN, convertUnit(DEGREE, RADIAN, -3.)));
        } else {
            /* Creation of the approach steps: mode, rating, derate, config., endAlt, rate_of_climb_limit, endCAS, GD, glidePath */
            mission->approach_steps.push_back(approach_step("descend", "cruise", 1., "clean", convertUnit(FOOT, METER, 2000.),  NAN, NAN, NAN, convertUnit(DEGREE, RADIAN, -3.)));
            mission->approach_steps.push_back(approach_step("change_speed", "idle",   1., "approach",
                                                            NAN, NAN, v_2_temp, convertUnit(FOOT, METER, 6200.), convertUnit(DEGREE, RADIAN, -3.)));
            mission->approach_steps.push_back(approach_step("descend", "cruise", 1., "landing", convertUnit(FOOT, METER, 50.), NAN, NAN, NAN, convertUnit(DEGREE, RADIAN, -3.)));
            mission->approach_steps.push_back(approach_step("landing", "takeoff",     1., "landing", 0., NAN, NAN, NAN, convertUnit(DEGREE, RADIAN, -3.)));
        }
    } else { // Unknown approach procedure, error message
        myRuntimeInfo->err << "Unknown approach procedure in config file. Abort!" << std::endl;
        exit(1);
    }
}
