/*  Copyright (C) 2012-2022 Institute of Aerospace Systems, RWTH Aachen University - All rights reserved.
    This file is part of UNICADO.

    UNICADO is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    UNICADO is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with UNICADO.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "ecoDatabase.h"
#include <cmath>
#include <string>
#include <unitConversion/unitConversion.h>

ecoDatabase::ecoDatabase() {
    //ctor
/*TODO(franz#1#): T134 better manageability with regard to data change: store data in csv format and read in here
 (flexible programming, depending on the amount of data in csv format)*/
//    /* Ecological database of an aircraft according to MA Arzdorf S.96 Abb.B.3 */
//    //Emissionen durch Treibstoffverbrauch im Flug
//    //Angaben in [kg/kg]
//    aircraftEmData.specification = "ACFlightEmissions";
//    aircraftEmData.CO2 = 3.15;
//    aircraftEmData.H2O = 1.24;
//    aircraftEmData.NOx = 1.63E-02;
//    aircraftEmData.SO2 = 7.99E-04;
//    aircraftEmData.CO = 2.61E-03;
//    aircraftEmData.CH4 = 4.00E-05;
//    aircraftEmData.PM10 = 1.75E-05;
//    aircraftEmData.GWP100 = 3.2;
//    aircraftEmData.AP = 9.08E-03;
//    aircraftEmData.ADP = 1.09E-07;
//    aircraftEmData.POCP = 9.12E-04;
//    aircraftEmData.OLDP = 1.00E-11;
//    aircraftEmData.PMFP = 3.74E-03;
    /* Ecological database of raw material extraction according to MA Arzdorf P.95 */
    //emissions per mass of raw material [kg/kg]
    //Aluminium
    matEmData.push_back(EmissionsData(std::string("Al")));
    matEmData.at(0).CO2 = 8.07;
    matEmData.at(0).H2O = 1.1 * pow(10, 2);
    matEmData.at(0).NOx = 1.69 * pow(10, -2);
    matEmData.at(0).SO2 = 3.5 * pow(10, -2);
    matEmData.at(0).CO = 3.27 * pow(10, -3);
    matEmData.at(0).CH4 = 1.65 * pow(10, -2);
    matEmData.at(0).PM10 = 1.93 * pow(10, -7);
    matEmData.at(0).GWP100 = 9.42;
    matEmData.at(0).AP = 5.15 * pow(10, -2);
    matEmData.at(0).ADP = 3.5 * pow(10, -6);
    matEmData.at(0).POCP = 2.82 * pow(10, -3);
    matEmData.at(0).OLDP = 2.46 * pow(10, -9);
    matEmData.at(0).PMFP = 1.88 * pow(10, -2);
    matSpecificEnergy.push_back(45.56 * convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.));
    //Carbon fiber reinforced plastic
// TODO(franz#1#): T133 CF and epoxy resin should be listed separately or determined by config CF content of CFRP user?
    double rateCF = 0.65;
    /*Consisting of 65 % carbon fiber and 35 % epoxy resin (mass related);
      see Thermosets and Composites: Material Selection, Applications, Manufacturing ... by Michel Biron p. 60:*/
    matEmData.push_back(EmissionsData(std::string("CFRP")));
    matEmData.at(1).CO2 = rateCF * 4.3 * pow(10, 1) + (1 - rateCF) * 0.548 * pow(10, 1);
    matEmData.at(1).H2O = rateCF * 2.0 * pow(10, 2) + (1 - rateCF) * 2.73 * pow(10, 1);
    matEmData.at(1).NOx = rateCF * 6.27 * pow(10, -2) + (1 - rateCF) * 7.23 * pow(10, -3);
    matEmData.at(1).SO2 = rateCF * 4.27 * pow(10, -2) + (1 - rateCF) * 5.3 * pow(10, -3);
    matEmData.at(1).CO  = rateCF * 2.34 * pow(10, -2) + (1 - rateCF) * 2.92 * pow(10, -3);
    matEmData.at(1).CH4 = rateCF * 8.33 * pow(10, -2) + (1 - rateCF) * 1.62 * pow(10, -2);
    matEmData.at(1).PM10 = rateCF * 4.73 * pow(10, -8) + (1 - rateCF) * 4.89 * pow(10, -7);
    matEmData.at(1).GWP100 = rateCF * 4.53 * pow(10, 1) + (1 - rateCF) * 0.606 * pow(10, 1);
    matEmData.at(1).AP = rateCF * 9.14 * pow(10, -2) + (1 - rateCF) * 1.07 * pow(10, -2);
    matEmData.at(1).ADP = 2 * 5.46 * pow(10, -6); //ATTENTION: wrong value, still from Achim; not found in GaBi! Assumption, due to change of the others: times 2
    matEmData.at(1).POCP = rateCF * 8.08 * pow(10, -3) + (1 - rateCF) * 1.75 * pow(10, -3);
    matEmData.at(1).OLDP = rateCF * 3.04 * pow(10, -9) + (1 - rateCF) * 1.56 * pow(10, -10);
    matEmData.at(1).PMFP = rateCF * 2.48 * pow(10, -2) + (1 - rateCF) * 3.12 * pow(10, -3); //GaBi but not CML2001, rather ReCiPe 1.08
    matSpecificEnergy.push_back((rateCF * 2.36 * pow(10, 2) + (1 - rateCF) * 3.55 * pow(10, 1))*convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.));
    //Steel
    matEmData.push_back(EmissionsData(std::string("St")));
    matEmData.at(2).CO2 = 2.01;
    matEmData.at(2).H2O = 2.45;
    matEmData.at(2).NOx = 4.94 * pow(10, -3);
    matEmData.at(2).SO2 = 5.64 * pow(10, -3);
    matEmData.at(2).CO = 2.24 * pow(10, -2);
    matEmData.at(2).CH4 = 5.33 * pow(10, -3);
    matEmData.at(2).PM10 = 5.78 * pow(10, -6);
    matEmData.at(2).GWP100 = 2.2;
    matEmData.at(2).AP = 9.28 * pow(10, -3);
    matEmData.at(2).ADP = 2.27 * pow(10, -7);
    matEmData.at(2).POCP = 1.3 * pow(10, -3);
    matEmData.at(2).OLDP = 6.51 * pow(10, -12);
    matEmData.at(2).PMFP = 3.41 * pow(10, -3);
    matSpecificEnergy.push_back(6.14 * convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.));
    //Titan
    matEmData.push_back(EmissionsData(std::string("Ti")));
    matEmData.at(3).CO2 = 1.48 * pow(10, 1);
    matEmData.at(3).H2O = 1.51 * pow(10, 2);
    matEmData.at(3).NOx = 6.35 * pow(10, -2);
    matEmData.at(3).SO2 = 4.37 * pow(10, -2);
    matEmData.at(3).CO = 2.93 * pow(10, -1);
    matEmData.at(3).CH4 = 3.64 * pow(10, -2);
    matEmData.at(3).PM10 = 4.69 * pow(10, -6);
    matEmData.at(3).GWP100 = 1.58 * pow(10, 1);
    matEmData.at(3).AP = 8.79 * pow(10, -2);
    matEmData.at(3).ADP = 1.52 * pow(10, -6);
    matEmData.at(3).POCP = 1.4 * pow(10, -2);
    matEmData.at(3).OLDP = 1.18 * pow(10, -9);
    matEmData.at(3).PMFP = 3.45 * pow(10, -2);
    matSpecificEnergy.push_back(69.72 * convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.));
//    //Chrom
//    matEmData.push_back(std::string("Cr"));
//    matEmData.at(4).CO2 = 6.68;
//    matEmData.at(4).H2O = 2.03*pow(10,1);
//    matEmData.at(4).NOx = 2.41*pow(10,-2);
//    matEmData.at(4).SO2 = 3.73*pow(10,-2);
//    matEmData.at(4).CO = 3.05*pow(10,-3);
//    matEmData.at(4).CH4 = 1.61*pow(10,-2);
//    matEmData.at(4).PM10 = 3.69*pow(10,-5);
//    matEmData.at(4).GWP100 = 7.11;
//    matEmData.at(4).AP = 5.77*pow(10,-2);
//    matEmData.at(4).ADP = 5.36*pow(10,-4);
//    matEmData.at(4).POCP = 2.95*pow(10,-3);
//    matEmData.at(4).OLDP = 7.57*pow(10,-11);
//    matEmData.at(4).PMFP = 1.88*pow(10,-2);
//    matSpecificEnergy.at(4) = 23.47*convertUnit(KILO,WATTHOUR,NOPREFIX,JOULE, 1.);
//    //Kupfer
//    matEmData.push_back(std::string("Cu"));
//    matEmData.at(5).CO2 = 3.47;
//    matEmData.at(5).H2O = 5.27*pow(10,1);
//    matEmData.at(5).NOx = 8.23*pow(10,-3);
//    matEmData.at(5).SO2 = 1.43*pow(10,-2);
//    matEmData.at(5).CO = 2.84*pow(10,-3);
//    matEmData.at(5).CH4 = 4.73*pow(10,-3);
//    matEmData.at(5).PM10 = 2.39*pow(10,-5);
//    matEmData.at(5).GWP100 = 3.65;
//    matEmData.at(5).AP = 2.16*pow(10,-2);
//    matEmData.at(5).ADP = 7.01*pow(10,-3);
//    matEmData.at(5).POCP = 1.52*pow(10,-3);
//    matEmData.at(5).OLDP = 4.44*pow(10,-10);
//    matEmData.at(5).PMFP = 1.1*pow(10,-2);
//    matSpecificEnergy.at(5) = 17.11*convertUnit(KILO,WATTHOUR,NOPREFIX,JOULE, 1.);
//    //Nickel
    matEmData.push_back(EmissionsData(std::string("Ni")));
    matEmData.at(4).CO2 = 8.67;
    matEmData.at(4).H2O = 2.13 * pow(10, 2);
    matEmData.at(4).NOx = 3.29 * pow(10, -2);
    matEmData.at(4).SO2 = 2.91 * pow(10, -1);
    matEmData.at(4).CO = 1.26 * pow(10, -2);
    matEmData.at(4).CH4 = 1.63 * pow(10, -2);
    matEmData.at(4).PM10 = 2.12 * pow(10, -5);
    matEmData.at(4).GWP100 = 9.18;
    matEmData.at(4).AP = 3.74 * pow(10, -1);
    matEmData.at(4).ADP = 5.09 * pow(10, -5);
    matEmData.at(4).POCP = 1.61 * pow(10, -2);
    matEmData.at(4).OLDP = 4.6 * pow(10, -10);
    matEmData.at(4).PMFP = 7.26 * pow(10, -2);
    matSpecificEnergy.push_back(43.89 * convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.));
//    //Plastic
//    matEmData.push_back(std::string("Pl"));
//    matEmData.at(7).CO2 = 7.91;
//    matEmData.at(7).H2O = 1.56*pow(10,1);
//    matEmData.at(7).NOx = 7.45*pow(10,-3);
//    matEmData.at(7).SO2 = 8.74*pow(10,-3);
//    matEmData.at(7).CO = 2.36*pow(10,-3);
//    matEmData.at(7).CH4 = 1.65*pow(10,-2);
//    matEmData.at(7).PM10 = 4*pow(10,-7);
//    matEmData.at(7).GWP100 = 8.58;
//    matEmData.at(7).AP = 1.45*pow(10,-2);
//    matEmData.at(7).ADP = 2.47*pow(10,-6);
//    matEmData.at(7).POCP = 2.3*pow(10,-3);
//    matEmData.at(7).OLDP = 1.35*pow(10,-10);
//    matEmData.at(7).PMFP = 3.71*pow(10,-3);
//    matSpecificEnergy.at(7) = 48.89*convertUnit(KILO,WATTHOUR,NOPREFIX,JOULE, 1.);
    /* Ecological database of energy supply according to MA Arzdorf p.98 */
    /* Factors, resource energy/usable energy */
    //electricity data in [kg/MJ]
    elecEmData.CO2 = 4.5 * pow(10, -1) / convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.);
    elecEmData.H2O = 4.01 * pow(10, 0) / convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.);
    elecEmData.NOx = 7.63 * pow(10, -4) / convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.);
    elecEmData.SO2 = 1.47 * pow(10, -3) / convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.);
    elecEmData.CO = 3.14 * pow(10, -4) / convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.);
    elecEmData.CH4 = 9.11 * pow(10, -4) / convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.);
    elecEmData.PM10 = 1.1 * pow(10, -5) / convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.);
    //potential [1/MJ]
    elecEmData.GWP100 = 4.75 * pow(10, -1) / convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.);
    elecEmData.AP = 2.24 * pow(10, -3) / convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.);
    elecEmData.ADP = 5.81 * pow(10, -8) / convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.);
    elecEmData.POCP = 1.33 * pow(10, -4) / convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.);
    elecEmData.OLDP = 3.25 * pow(10, -10) / convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.);
    elecEmData.PMFP = 5.36 * pow(10, -4) / convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.);
    elecEmData.specificEnergy = 2.92;//how much energy is needed to produce 1 kWh (for final consumers)?
    //heating data in [kg/MJ]
    heatEmData.CO2 = 2.28 * pow(10, -1) / convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.);
    heatEmData.H2O = 2.13 * pow(10, -1) / convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.);
    heatEmData.NOx = 1.84 * pow(10, -4) / convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.);
    heatEmData.SO2 = 9.79 * pow(10, -5) / convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.);
    heatEmData.CO = 7.45 * pow(10, -5) / convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.);
    heatEmData.CH4 = 6.16 * pow(10, -4) / convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.);
    heatEmData.PM10 = 5.54 * pow(10, -7) / convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.);
    //potential [1/MJ]
    heatEmData.GWP100 = 2.44 * pow(10, -1) / convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.);
    heatEmData.AP = 2.14 * pow(10, -4) / convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.);
    heatEmData.ADP = 8.86 * pow(10, -9) / convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.);
    heatEmData.POCP = 3.71 * pow(10, -5) / convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.);
    heatEmData.OLDP = 4.32 * pow(10, -12) / convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.);
    heatEmData.PMFP = 6.55 * pow(10, -5) / convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.);
    heatEmData.specificEnergy = 1.23;
    /* Ecological database of fuel production according to MA Arzdorf p.96 */
    /* Factor: Resource energy per kg of produced fuel */
    //Emission data of fuel production. Emissions per fuel mass [kg/kg];
    /*kerosene*/
    fuelProdEmData.push_back(EmissionsData(std::string("Kerosine")));
    fuelProdEmData.back().CO2 = 2.94 * pow(10, -1.);
    fuelProdEmData.back().H2O = 2.11 * pow(10, -1.);
    fuelProdEmData.back().NOx = 9.44 * pow(10, -4.);
    fuelProdEmData.back().SO2 = 1.98 * pow(10, -3.);
    fuelProdEmData.back().CO = 4.76 * pow(10, -4.);
    fuelProdEmData.back().CH4 = 3.97 * pow(10, -3.);
    fuelProdEmData.back().PM10 = 5.08 * pow(10, -5.);
    //Potential [1/kg]
    fuelProdEmData.back().GWP100 = 3.95 * pow(10, -1.);
    fuelProdEmData.back().AP = 2.87 * pow(10, -3.);
    fuelProdEmData.back().ADP = 1.03 * pow(10, -7);
    fuelProdEmData.back().POCP = 4.38 * pow(10, -4.);
    fuelProdEmData.back().OLDP = 9.48 * pow(10, -12.);
    fuelProdEmData.back().PMFP = 6.70 * pow(10, -4.);
    fuelSpecificEnergy.push_back(convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 14.89));
    /*Diesel*/
    fuelProdEmData.push_back(EmissionsData(std::string("Diesel")));
    fuelProdEmData.back().CO2 = 3.6E-01;
    fuelProdEmData.back().H2O = 3.72E-01;
    fuelProdEmData.back().NOx = 1.07E-03;
    fuelProdEmData.back().SO2 = 2.09E-03;
    fuelProdEmData.back().CO = 5.24E-04;
    fuelProdEmData.back().CH4 = 4.07E-03;
    fuelProdEmData.back().PM10 = 6.26E-05;
    //Potential [1/kg]
    fuelProdEmData.back().GWP100 = 3.32E-01;
    fuelProdEmData.back().AP = 3.73E-03;
    fuelProdEmData.back().ADP = 1.32E-07;
    fuelProdEmData.back().POCP = 4.63E-04;
    fuelProdEmData.back().OLDP = 1.68E-11;
    fuelProdEmData.back().PMFP = 8.68E-04;
    fuelSpecificEnergy.push_back(convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 15.06));
    /*Heating oil*/
    fuelProdEmData.push_back(EmissionsData(std::string("HeatOil")));
    fuelProdEmData.back().CO2 = 3.4E-01;
    fuelProdEmData.back().H2O = 2.65E-01;
    fuelProdEmData.back().NOx = 1.01E-03;
    fuelProdEmData.back().SO2 = 2.17E-03;
    fuelProdEmData.back().CO = 5.24E-04;
    fuelProdEmData.back().CH4 = 4.06E-03;
    fuelProdEmData.back().PM10 = 5.57E-05;
    //Potential [1/kg]
    fuelProdEmData.back().GWP100 = 4.43E-01;
    fuelProdEmData.back().AP = 3.14E-03;
    fuelProdEmData.back().ADP = 1.05E-07;
    fuelProdEmData.back().POCP = 4.72E-04;
    fuelProdEmData.back().OLDP = 1.37E-11;
    fuelProdEmData.back().PMFP = 7.29E-04;
    fuelSpecificEnergy.push_back(convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 15.14));
    /*Gasoline*/
    fuelProdEmData.push_back(EmissionsData(std::string("Gasoline")));
    fuelProdEmData.back().CO2 = 6.34E-01;
    fuelProdEmData.back().H2O = 5.12E-01;
    fuelProdEmData.back().NOx = 1.5E-03;
    fuelProdEmData.back().SO2 = 3.21E-03;
    fuelProdEmData.back().CO = 8.36E-04;
    fuelProdEmData.back().CH4 = 4.35E-03;
    fuelProdEmData.back().PM10 = 8.65E-05;
    //Potential [1/kg]
    fuelProdEmData.back().GWP100 = 6.78E-01;
    fuelProdEmData.back().AP = 4.72E-03;
    fuelProdEmData.back().ADP = 1.14E-07;
    fuelProdEmData.back().POCP = 6.64E-04;
    fuelProdEmData.back().OLDP = 2.58E-11;
    fuelProdEmData.back().PMFP = 1.57E-03;
    fuelSpecificEnergy.push_back(convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 15.86));
    /*NaturalGas*/
    fuelProdEmData.push_back(EmissionsData(std::string("NaturalGas")));
    fuelProdEmData.back().CO2 = 2.9E-01;
    fuelProdEmData.back().H2O = 6.71E-01;
    fuelProdEmData.back().NOx = 5.07E-04;
    fuelProdEmData.back().SO2 = 1.21E-03;
    fuelProdEmData.back().CO = 3.6E-04;
    fuelProdEmData.back().CH4 = 7.52E-03;
    fuelProdEmData.back().PM10 = 5.57E-06;
    //Potential [1/kg]
    fuelProdEmData.back().GWP100 = 4.79E-01;
    fuelProdEmData.back().AP = 1.75E-03;
    fuelProdEmData.back().ADP = 1.1E-03;
    fuelProdEmData.back().POCP = 3.71E-04;
    fuelProdEmData.back().OLDP = 5.37E-11;
    fuelProdEmData.back().PMFP = 3.75E-04;
    fuelSpecificEnergy.push_back(convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 15.44));
    /* Ecological database of transport logistics according to MA Arzdorf p.97 Fig.B.4*/
    //DATA in [kg/(1000kg*km)]
    //Aircraft
    transpEmData.push_back(EmissionsData(std::string("Aircraft")));
    transpEmData.at(0).CO2 = 5.81 * pow(10, -1);
    transpEmData.at(0).H2O = 2.45 * pow(10, -1);
    transpEmData.at(0).NOx = 2.9 * pow(10, -3);
    transpEmData.at(0).SO2 = 4.7 * pow(10, -4);
    transpEmData.at(0).CO = 5.22 * pow(10, -4);
    transpEmData.at(0).CH4 = 6.76 * pow(10, -4);
    transpEmData.at(0).PM10 = 8.57 * pow(10, -6);
    transpEmData.at(0).GWP100 = 6.06 * pow(10, -1);
    transpEmData.at(0).AP = 2.02 * pow(10, -3);
    transpEmData.at(0).ADP = 1.73 * pow(10, -8);
    transpEmData.at(0).POCP = 2.28 * pow(10, -4);
    transpEmData.at(0).OLDP = 1.6 * pow(10, -12);
    transpEmData.at(0).PMFP = 7.46 * pow(10, -4);
    transportSpecificEnergy.push_back(9.03);
    //Truck
    transpEmData.push_back(EmissionsData(std::string("Truck")));
    transpEmData.at(1).CO2 = 5.76 * pow(10, -2);
    transpEmData.at(1).H2O = 8.22 * pow(10, -1);
    transpEmData.at(1).NOx = 1.82 * pow(10, -5);
    transpEmData.at(1).SO2 = 3.59 * pow(10, -5);
    transpEmData.at(1).CO = 9.17 * pow(10, -5);
    transpEmData.at(1).CH4 = 6.97 * pow(10, -5);
    transpEmData.at(1).PM10 = 1.07 * pow(10, -6);
    transpEmData.at(1).GWP100 = 5.99 * pow(10, -2);
    transpEmData.at(1).AP = 3.88 * pow(10, -4);
    transpEmData.at(1).ADP = 2.25 * pow(10, -9);
    transpEmData.at(1).POCP = -1.56 * pow(10, -4);
    transpEmData.at(1).OLDP = 2.87 * pow(10, -13);
    transpEmData.at(1).PMFP = 3.09 * pow(10, -5);
    transportSpecificEnergy.push_back(0.922);
    //Ship
    transpEmData.push_back(EmissionsData(std::string("Ship")));
    transpEmData.at(2).CO2 = 3.55 * pow(10, -3);
    transpEmData.at(2).H2O = 2.44 * pow(10, -4);
    transpEmData.at(2).NOx = 8.61 * pow(10, -5);
    transpEmData.at(2).SO2 = 5.68 * pow(10, -5);
    transpEmData.at(2).CO = 9.63 * pow(10, -6);
    transpEmData.at(2).CH4 = 3.71 * pow(10, -6);
    transpEmData.at(2).PM10 = 5.37 * pow(10, -8);
    transpEmData.at(2).GWP100 = 3.67 * pow(10, -3);
    transpEmData.at(2).AP = 1.11 * pow(10, -4);
    transpEmData.at(2).ADP = 9.35 * pow(10, -11);
    transpEmData.at(2).POCP = 6.6 * pow(10, -6);
    transpEmData.at(2).OLDP = 1.29 * pow(10, -14);
    transpEmData.at(2).PMFP = 3.24 * pow(10, -5);
    transportSpecificEnergy.push_back(0.0485);
    //Train
    transpEmData.push_back(EmissionsData(std::string("Train")));
    transpEmData.at(3).CO2 = 1.45 * pow(10, -2);
    transpEmData.at(3).H2O = 1.29 * pow(10, -1);
    transpEmData.at(3).NOx = 2.46 * pow(10, -5);
    transpEmData.at(3).SO2 = 4.72 * pow(10, -5);
    transpEmData.at(3).CO = 1.01 * pow(10, -5);
    transpEmData.at(3).CH4 = 2.93 * pow(10, -5);
    transpEmData.at(3).PM10 = 3.45 * pow(10, -7);
    transpEmData.at(3).GWP100 = 1.53 * pow(10, -2);
    transpEmData.at(3).AP = 7.2 * pow(10, -5);
    transpEmData.at(3).ADP = 1.87 * pow(10, -9);
    transpEmData.at(3).POCP = 4.28 * pow(10, -6);
    transpEmData.at(3).OLDP = 1.05 * pow(10, -11);
    transpEmData.at(3).PMFP = 1.73 * pow(10, -5);
    transportSpecificEnergy.push_back(0.337);
    /* Ecological database of combustion according to MA Arzdorf p.100 Fig.B.7 */
//    /* Ecological database for recycling according to Master's Thesis Arzdorf S.99 Abb.B.6 */
//    //units in[kg/kg]
//    //Aluminium
//    recyclingEmData.push_back(std::string("Al"));
//    recyclingEmData.at(0).CO2 = 1.37;
//    recyclingEmData.at(0).H2O = 1.22E+01;
//    recyclingEmData.at(0).NOx = 2.33E-03;
//    recyclingEmData.at(0).SO2 = 4.48E-03;
//    recyclingEmData.at(0).CO = 9.57E-04;
//    recyclingEmData.at(0).CH4 = 2.78E-03;
//    recyclingEmData.at(0).PM10 = 3.35E-05;
//    recyclingEmData.at(0).GWP100 = 1.45;
//    recyclingEmData.at(0).AP = 6.83E-03;
//    recyclingEmData.at(0).ADP = 1.77E-07;
//    recyclingEmData.at(0).POCP = 4.05E-04;
//    recyclingEmData.at(0).OLDP = 9.91E-10;
//    recyclingEmData.at(0).PMFP = 1.63E-03;
//    //Kohlefaser
//    recyclingEmData.push_back(std::string("CFRP"));
//    recyclingEmData.at(1).CO2 = 0.;
//    recyclingEmData.at(1).H2O = 0.;
//    recyclingEmData.at(1).NOx = 0.;
//    recyclingEmData.at(1).SO2 = 0.;
//    recyclingEmData.at(1).CO = 0.;
//    recyclingEmData.at(1).CH4 = 0.;
//    recyclingEmData.at(1).PM10 = 0.;
//    recyclingEmData.at(1).GWP100 = 0.;
//    recyclingEmData.at(1).AP = 0.;
//    recyclingEmData.at(1).ADP = 0.;
//    recyclingEmData.at(1).POCP = 0.;
//    recyclingEmData.at(1).OLDP = 0.;
//    recyclingEmData.at(1).PMFP = 0.;
//    //Stahl
//    recyclingEmData.push_back(std::string("St"));
//    recyclingEmData.at(2).CO2 = 1.55;
//    recyclingEmData.at(2).H2O = 1.38E+01;
//    recyclingEmData.at(2).NOx = 2.63E-03;
//    recyclingEmData.at(2).SO2 = 5.06E-03;
//    recyclingEmData.at(2).CO = 1.08E-03;
//    recyclingEmData.at(2).CH4 = 3.14E-03;
//    recyclingEmData.at(2).PM10 = 3.79E-05;
//    recyclingEmData.at(2).GWP100 = 1.63;
//    recyclingEmData.at(2).AP = 7.71E-03;
//    recyclingEmData.at(2).ADP = 2.0E-07;
//    recyclingEmData.at(2).POCP = 4.58E-04;
//    recyclingEmData.at(2).OLDP = 1.12E-09;
//    recyclingEmData.at(2).PMFP = 1.84E-03;
//    //Titan
//    recyclingEmData.push_back(std::string("Ti"));
//    recyclingEmData.at(3).CO2 = recyclingEmData.at(2).CO2;
//    recyclingEmData.at(3).H2O = recyclingEmData.at(2).H2O;
//    recyclingEmData.at(3).NOx = recyclingEmData.at(2).NOx;
//    recyclingEmData.at(3).SO2 = recyclingEmData.at(2).SO2;
//    recyclingEmData.at(3).CO = recyclingEmData.at(2).CO;
//    recyclingEmData.at(3).CH4 = recyclingEmData.at(2).CH4;
//    recyclingEmData.at(3).PM10 = recyclingEmData.at(2).PM10;
//    recyclingEmData.at(3).GWP100 = recyclingEmData.at(2).GWP100;
//    recyclingEmData.at(3).AP = recyclingEmData.at(2).AP;
//    recyclingEmData.at(3).ADP = recyclingEmData.at(2).ADP;
//    recyclingEmData.at(3).POCP = recyclingEmData.at(2).POCP;
//    recyclingEmData.at(3).OLDP = recyclingEmData.at(2).OLDP;
//    recyclingEmData.at(3).PMFP = recyclingEmData.at(2).PMFP;
//    //Chrom
//    recyclingEmData.push_back(std::string("Cr"));
//    recyclingEmData.at(4).CO2 = recyclingEmData.at(2).CO2;
//    recyclingEmData.at(4).H2O = recyclingEmData.at(2).H2O;
//    recyclingEmData.at(4).NOx = recyclingEmData.at(2).NOx;
//    recyclingEmData.at(4).SO2 = recyclingEmData.at(2).SO2;
//    recyclingEmData.at(4).CO = recyclingEmData.at(2).CO;
//    recyclingEmData.at(4).CH4 = recyclingEmData.at(2).CH4;
//    recyclingEmData.at(4).PM10 = recyclingEmData.at(2).PM10;
//    recyclingEmData.at(4).GWP100 = recyclingEmData.at(2).GWP100;
//    recyclingEmData.at(4).AP = recyclingEmData.at(2).AP;
//    recyclingEmData.at(4).ADP = recyclingEmData.at(2).ADP;
//    recyclingEmData.at(4).POCP = recyclingEmData.at(2).POCP;
//    recyclingEmData.at(4).OLDP = recyclingEmData.at(2).OLDP;
//    recyclingEmData.at(4).PMFP = recyclingEmData.at(2).PMFP;
//    //Kupfer
//    recyclingEmData.push_back(std::string("Cu"));
//    recyclingEmData.at(5).CO2 = 8.19;
//    recyclingEmData.at(5).H2O = 7.3E+01;
//    recyclingEmData.at(5).NOx = 1.39E-02;
//    recyclingEmData.at(5).SO2 = 2.68E-02;
//    recyclingEmData.at(5).CO = 5.71E-03;
//    recyclingEmData.at(5).CH4 = 1.66E-02;
//    recyclingEmData.at(5).PM10 = 2.00E-04;
//    recyclingEmData.at(5).GWP100 = 8.65;
//    recyclingEmData.at(5).AP = 4.08E-02;
//    recyclingEmData.at(5).ADP = 1.06E-06;
//    recyclingEmData.at(5).POCP = 2.42E-03;
//    recyclingEmData.at(5).OLDP = 5.92E-09;
//    recyclingEmData.at(5).PMFP = 9.76E-03;
//    //Nickel
//    recyclingEmData.push_back(std::string("Ni"));
//    recyclingEmData.at(6).CO2 = 6.08E-01;
//    recyclingEmData.at(6).H2O = 5.41;
//    recyclingEmData.at(6).NOx = 1.03E-03;
//    recyclingEmData.at(6).SO2 = 1.98E-03;
//    recyclingEmData.at(6).CO = 4.24E-04;
//    recyclingEmData.at(6).CH4 = 1.23E-03;
//    recyclingEmData.at(6).PM10 = 1.49E-05;
//    recyclingEmData.at(6).GWP100 = 6.41E-01;
//    recyclingEmData.at(6).AP = 3.02E-03;
//    recyclingEmData.at(6).ADP = 7.84E-08;
//    recyclingEmData.at(6).POCP = 1.8E-04;
//    recyclingEmData.at(6).OLDP = 4.39E-10;
//    recyclingEmData.at(6).PMFP = 7.42E-04;
//    //Kunststoff
//    recyclingEmData.push_back(std::string("Pl"));
//    recyclingEmData.at(7).CO2 = 2.57E-02;
//    recyclingEmData.at(7).H2O = 2.29E-01;
//    recyclingEmData.at(7).NOx = 4.36E-05;
//    recyclingEmData.at(7).SO2 = 8.41E-05;
//    recyclingEmData.at(7).CO = 1.8E-05;
//    recyclingEmData.at(7).CH4 = 5.21E-05;
//    recyclingEmData.at(7).PM10 = 6.29E-07;
//    recyclingEmData.at(7).GWP100 = 2.72E-02;
//    recyclingEmData.at(7).AP = 1.28E-04;
//    recyclingEmData.at(7).ADP = 3.32E-09;
//    recyclingEmData.at(7).POCP = 7.61E-06;
//    recyclingEmData.at(7).OLDP = 1.86E-11;
//    recyclingEmData.at(7).PMFP = 3.07E-05;
    /* Ecological database for inceineration according to MA Arzdorf S.100 Abb.B.7 */
/*DATA in [kg/kg]; aluminium, steel and titanium and nickel will not be burned! Values there may also be wrong
  (positive, instead of negative? are not used in the current scenario, would otherwise have to be checked!)*/
    //Aluminium
    incinerationEmData.push_back(EmissionsData(std::string("Al")));
//    incinerationEmData.at(0).CO2 = -7.72*pow(10.,-1.);
//    incinerationEmData.at(0).H2O = 6.63*pow(10.,-1.);
//    incinerationEmData.at(0).NOx = -1.71*pow(10.,-3.);
//    incinerationEmData.at(0).SO2 = -1.57*pow(10.,-3.);
//    incinerationEmData.at(0).CO = -1.06*pow(10.,-2.);
//    incinerationEmData.at(0).CH4 = -2.5*pow(10.,-4.);
//    incinerationEmData.at(0).PM10 = -9.9*pow(10.,-6.);
//    incinerationEmData.at(0).GWP100 = -7.78*pow(10.,-1.);
//    incinerationEmData.at(0).AP = -2.68*pow(10.,-3.);
//    incinerationEmData.at(0).ADP = -5.5*pow(10.,5.);
//    incinerationEmData.at(0).POCP = -4.6*pow(10.,-4.);
//    incinerationEmData.at(0).OLDP = 3.7*pow(10.,-8.);
//    incinerationEmData.at(0).PMFP = -7.4*pow(10.,-4.);
    incinerationSpecificEnergy.push_back(0.0 * convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.)); //no value available, but anyway aluminium will not be burned
    //carbon fiber
    // positive values given for Achim, but probably error (see Hedlund), therefore used negatively!
    incinerationEmData.push_back(EmissionsData(std::string("CFRP")));
//    incinerationEmData.at(1).CO2 = -1.93;
//    incinerationEmData.at(1).H2O = -1.72*pow(10,1);
//    incinerationEmData.at(1).NOx = -3.27*pow(10,-3);
//    incinerationEmData.at(1).SO2 = -6.31*pow(10,-3);
//    incinerationEmData.at(1).CO = -1.35*pow(10,-3);
//    incinerationEmData.at(1).CH4 = -3.91*pow(10,-3);
//    incinerationEmData.at(1).PM10 = -4.72*pow(10,-5);
//    incinerationEmData.at(1).GWP100 = -2.04;
//    incinerationEmData.at(1).AP = -9.61*pow(10,-3);
//    incinerationEmData.at(1).ADP = -2.49*pow(10,-7);
//    incinerationEmData.at(1).POCP = -5.71*pow(10,-4);
//    incinerationEmData.at(1).OLDP = -1.39*pow(10,-9);
//    incinerationEmData.at(1).PMFP = -2.3*pow(10,-3);
    incinerationSpecificEnergy.push_back(-6.84 * convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.));
    //Steel
    incinerationEmData.push_back(EmissionsData(std::string("St")));
//    incinerationEmData.at(2).CO2 = incinerationEmData.at(0).CO2;
//    incinerationEmData.at(2).H2O = incinerationEmData.at(0).H2O;
//    incinerationEmData.at(2).NOx = incinerationEmData.at(0).NOx;
//    incinerationEmData.at(2).SO2 = incinerationEmData.at(0).SO2;
//    incinerationEmData.at(2).CO = incinerationEmData.at(0).CO;
//    incinerationEmData.at(2).CH4 = incinerationEmData.at(0).CH4;
//    incinerationEmData.at(2).PM10 = incinerationEmData.at(0).PM10;
//    incinerationEmData.at(2).GWP100 = incinerationEmData.at(0).GWP100;
//    incinerationEmData.at(2).AP = incinerationEmData.at(0).AP;
//    incinerationEmData.at(2).ADP = incinerationEmData.at(0).ADP;
//    incinerationEmData.at(2).POCP = incinerationEmData.at(0).POCP;
//    incinerationEmData.at(2).OLDP = incinerationEmData.at(0).OLDP;
//    incinerationEmData.at(2).PMFP = incinerationEmData.at(0).PMFP;
    incinerationSpecificEnergy.push_back(0.0 * convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.));
    //Titan
    incinerationEmData.push_back(EmissionsData(std::string("Ti")));
//    incinerationEmData.at(3).CO2 = incinerationEmData.at(0).CO2;
//    incinerationEmData.at(3).H2O = incinerationEmData.at(0).H2O;
//    incinerationEmData.at(3).NOx = incinerationEmData.at(0).NOx;
//    incinerationEmData.at(3).SO2 = incinerationEmData.at(0).SO2;
//    incinerationEmData.at(3).CO = incinerationEmData.at(0).CO;
//    incinerationEmData.at(3).CH4 = incinerationEmData.at(0).CH4;
//    incinerationEmData.at(3).PM10 = incinerationEmData.at(0).PM10;
//    incinerationEmData.at(3).GWP100 = incinerationEmData.at(0).GWP100;
//    incinerationEmData.at(3).AP = incinerationEmData.at(0).AP;
//    incinerationEmData.at(3).ADP = incinerationEmData.at(0).ADP;
//    incinerationEmData.at(3).POCP = incinerationEmData.at(0).POCP;
//    incinerationEmData.at(3).OLDP = incinerationEmData.at(0).OLDP;
//    incinerationEmData.at(3).PMFP = incinerationEmData.at(0).PMFP;
    incinerationSpecificEnergy.push_back(0.0 * convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.));
//    //Chrom
//    incinerationEmData.push_back(std::string("Cr"));
//    incinerationEmData.at(4).CO2 = incinerationEmData.at(0).CO2;
//    incinerationEmData.at(4).H2O = incinerationEmData.at(0).H2O;
//    incinerationEmData.at(4).NOx = incinerationEmData.at(0).NOx;
//    incinerationEmData.at(4).SO2 = incinerationEmData.at(0).SO2;
//    incinerationEmData.at(4).CO = incinerationEmData.at(0).CO;
//    incinerationEmData.at(4).CH4 = incinerationEmData.at(0).CH4;
//    incinerationEmData.at(4).PM10 = incinerationEmData.at(0).PM10;
//    incinerationEmData.at(4).GWP100 = incinerationEmData.at(0).GWP100;
//    incinerationEmData.at(4).AP = incinerationEmData.at(0).AP;
//    incinerationEmData.at(4).ADP = incinerationEmData.at(0).ADP;
//    incinerationEmData.at(4).POCP = incinerationEmData.at(0).POCP;
//    incinerationEmData.at(4).OLDP = incinerationEmData.at(0).OLDP;
//    incinerationEmData.at(4).PMFP = incinerationEmData.at(0).PMFP;
//    //copper
//    incinerationEmData.push_back(std::string("Cu"));
//    incinerationEmData.at(5).CO2 = incinerationEmData.at(0).CO2;
//    incinerationEmData.at(5).H2O = incinerationEmData.at(0).H2O;
//    incinerationEmData.at(5).NOx = incinerationEmData.at(0).NOx;
//    incinerationEmData.at(5).SO2 = incinerationEmData.at(0).SO2;
//    incinerationEmData.at(5).CO = incinerationEmData.at(0).CO;
//    incinerationEmData.at(5).CH4 = incinerationEmData.at(0).CH4;
//    incinerationEmData.at(5).PM10 = incinerationEmData.at(0).PM10;
//    incinerationEmData.at(5).GWP100 = incinerationEmData.at(0).GWP100;
//    incinerationEmData.at(5).AP = incinerationEmData.at(0).AP;
//    incinerationEmData.at(5).ADP = incinerationEmData.at(0).ADP;
//    incinerationEmData.at(5).POCP = incinerationEmData.at(0).POCP;
//    incinerationEmData.at(5).OLDP = incinerationEmData.at(0).OLDP;
//    incinerationEmData.at(5).PMFP = incinerationEmData.at(0).PMFP;
//    //Nickel
    incinerationEmData.push_back(EmissionsData(std::string("Ni")));
//    incinerationEmData.at(6).CO2 = incinerationEmData.at(0).CO2;
//    incinerationEmData.at(6).H2O = incinerationEmData.at(0).H2O;
//    incinerationEmData.at(6).NOx = incinerationEmData.at(0).NOx;
//    incinerationEmData.at(6).SO2 = incinerationEmData.at(0).SO2;
//    incinerationEmData.at(6).CO = incinerationEmData.at(0).CO;
//    incinerationEmData.at(6).CH4 = incinerationEmData.at(0).CH4;
//    incinerationEmData.at(6).PM10 = incinerationEmData.at(0).PM10;
//    incinerationEmData.at(6).GWP100 = incinerationEmData.at(0).GWP100;
//    incinerationEmData.at(6).AP = incinerationEmData.at(0).AP;
//    incinerationEmData.at(6).ADP = incinerationEmData.at(0).ADP;
//    incinerationEmData.at(6).POCP = incinerationEmData.at(0).POCP;
//    incinerationEmData.at(6).OLDP = incinerationEmData.at(0).OLDP;
//    incinerationEmData.at(6).PMFP = incinerationEmData.at(0).PMFP;
    incinerationSpecificEnergy.push_back(0.0 * convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.));
//    //plastic
//    incinerationEmData.push_back(std::string("Pl"));
//    incinerationEmData.at(7).CO2 = 2.38;
//    incinerationEmData.at(7).H2O = 2.8;
//    incinerationEmData.at(7).NOx = 5.5*pow(10,-3);
//    incinerationEmData.at(7).SO2 = 6.44*pow(10,-5);
//    incinerationEmData.at(7).CO = 3.85*pow(10,-4);
//    incinerationEmData.at(7).CH4 = 1.84*pow(10,-4);
//    incinerationEmData.at(7).PM10 = 1.45*pow(10,-10);
//    incinerationEmData.at(7).GWP100 = 3.48;
//    incinerationEmData.at(7).AP = 2.93*pow(10,-3);
//    incinerationEmData.at(7).ADP = 1.98*pow(10,-8);
//    incinerationEmData.at(7).POCP = 1.81*pow(10,-4);
//    incinerationEmData.at(7).OLDP = 6.87*pow(10,-12);
//    incinerationEmData.at(7).PMFP = 1.25*pow(10,-3);
    /* Ecological database of landfill according to MA Arzdorf p.100 Fig.B.8 */
    //DATA in[kg/kg]
    //Aluminium
    disposalEmData.push_back(EmissionsData(std::string("Al")));
    disposalEmData.at(0).CO2 = 1.28 * pow(10, -2);
    disposalEmData.at(0).H2O = 3.33 * pow(10, -2);
    disposalEmData.at(0).NOx = 7.37 * pow(10, -5);
    disposalEmData.at(0).SO2 = 3.88 * pow(10, -5);
    disposalEmData.at(0).CO = 4.94 * pow(10, -5);
    disposalEmData.at(0).CH4 = 2.28 * pow(10, -5);
    disposalEmData.at(0).PM10 = 4.09 * pow(10, -9);
    disposalEmData.at(0).GWP100 = 1.35 * pow(10, -2);
    disposalEmData.at(0).AP = 8.62 * pow(10, -5);
    disposalEmData.at(0).ADP = 5.08 * pow(10, -9);
    disposalEmData.at(0).POCP = 8.09 * pow(10, -6);
    disposalEmData.at(0).OLDP = 1.87 * pow(10, -13);
    disposalEmData.at(0).PMFP = 2.23 * pow(10, -4);
    disposalSpecificEnergy.push_back(0.06 * convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.));
    //Carbon fiber (no data available at Arzdorf), assumption: like plastic-> data of plastic
    disposalEmData.push_back(EmissionsData(std::string("CFRP")));
    disposalEmData.at(1).CO2 = 6.26 * pow(10, -2);
    disposalEmData.at(1).H2O = 1.44 * pow(10, -1);
    disposalEmData.at(1).NOx = 1.39 * pow(10, -4);
    disposalEmData.at(1).SO2 = 1.18 * pow(10, -4);
    disposalEmData.at(1).CO = 1.1 * pow(10, -4);
    disposalEmData.at(1).CH4 = 3.5 * pow(10, -4);
    disposalEmData.at(1).PM10 = 9.56 * pow(10, -9);
    disposalEmData.at(1).GWP100 = 7.16 * pow(10, -2);
    disposalEmData.at(1).AP = 2.22 * pow(10, -4);
    disposalEmData.at(1).ADP = 1.42 * pow(10, -8);
    disposalEmData.at(1).POCP = 2.77 * pow(10, -5);
    disposalEmData.at(1).OLDP = 2.79 * pow(10, -12);
    disposalEmData.at(1).PMFP = 4.27 * pow(10, -4);
    disposalSpecificEnergy.push_back(0.34 * convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.));
    //Steel
    disposalEmData.push_back(EmissionsData(std::string("St")));
    disposalEmData.at(2).CO2 = 9.16 * pow(10, -3);
    disposalEmData.at(2).H2O = 1.5 * pow(10, -2);
    disposalEmData.at(2).NOx = 5.69 * pow(10, -5);
    disposalEmData.at(2).SO2 = 1.44 * pow(10, -5);
    disposalEmData.at(2).CO = 2.47 * pow(10, -5);
    disposalEmData.at(2).CH4 = 3.92 * pow(10, -5);
    disposalEmData.at(2).PM10 = 9.8 * pow(10, -10);
    disposalEmData.at(2).GWP100 = 1.02 * pow(10, -2);
    disposalEmData.at(2).AP = 4.7 * pow(10, -5);
    disposalEmData.at(2).ADP = 1.56 * pow(10, -9);
    disposalEmData.at(2).POCP = 5.69 * pow(10, -6);
    disposalEmData.at(2).OLDP = 2.99 * pow(10, -13);
    disposalEmData.at(2).PMFP = 5.66 * pow(10, -5);
    disposalSpecificEnergy.push_back(0.05 * convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.));
    //Titan
    disposalEmData.push_back(EmissionsData(std::string("Ti")));
    disposalEmData.at(3).CO2 = disposalEmData.at(2).CO2;
    disposalEmData.at(3).H2O = disposalEmData.at(2).H2O;
    disposalEmData.at(3).NOx = disposalEmData.at(2).NOx;
    disposalEmData.at(3).SO2 = disposalEmData.at(2).SO2;
    disposalEmData.at(3).CO = disposalEmData.at(2).CO;
    disposalEmData.at(3).CH4 = disposalEmData.at(2).CH4;
    disposalEmData.at(3).PM10 = disposalEmData.at(2).PM10;
    disposalEmData.at(3).GWP100 = disposalEmData.at(2).GWP100;
    disposalEmData.at(3).AP = disposalEmData.at(2).AP;
    disposalEmData.at(3).ADP = disposalEmData.at(2).ADP;
    disposalEmData.at(3).POCP = disposalEmData.at(2).POCP;
    disposalEmData.at(3).OLDP = disposalEmData.at(2).OLDP;
    disposalEmData.at(3).PMFP = disposalEmData.at(2).PMFP;
    disposalSpecificEnergy.push_back(0.05 * convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.));
//    //Chrom
//    disposalEmData.push_back(std::string("Cr"));
//    disposalEmData.at(4).CO2 = disposalEmData.at(2).CO2;
//    disposalEmData.at(4).H2O = disposalEmData.at(2).H2O;
//    disposalEmData.at(4).NOx = disposalEmData.at(2).NOx;
//    disposalEmData.at(4).SO2 = disposalEmData.at(2).SO2;
//    disposalEmData.at(4).CO = disposalEmData.at(2).CO;
//    disposalEmData.at(4).CH4 = disposalEmData.at(2).CH4;
//    disposalEmData.at(4).PM10 = disposalEmData.at(2).PM10;
//    disposalEmData.at(4).GWP100 = disposalEmData.at(2).GWP100;
//    disposalEmData.at(4).AP = disposalEmData.at(2).AP;
//    disposalEmData.at(4).ADP = disposalEmData.at(2).ADP;
//    disposalEmData.at(4).POCP = disposalEmData.at(2).POCP;
//    disposalEmData.at(4).OLDP = disposalEmData.at(2).OLDP;
//    disposalEmData.at(4).PMFP = disposalEmData.at(2).PMFP;
//    disposalSpecificEnergy.at(4) = 0.05*convertUnit(KILO,WATTHOUR,NOPREFIX,JOULE, 1.);
//    //Kupfer
//    disposalEmData.push_back(std::string("Cu"));
//    disposalEmData.at(5).CO2 = disposalEmData.at(2).CO2;
//    disposalEmData.at(5).H2O = disposalEmData.at(2).H2O;
//    disposalEmData.at(5).NOx = disposalEmData.at(2).NOx;
//    disposalEmData.at(5).SO2 = disposalEmData.at(2).SO2;
//    disposalEmData.at(5).CO = disposalEmData.at(2).CO;
//    disposalEmData.at(5).CH4 = disposalEmData.at(2).CH4;
//    disposalEmData.at(5).PM10 = disposalEmData.at(2).PM10;
//    disposalEmData.at(5).GWP100 = disposalEmData.at(2).GWP100;
//    disposalEmData.at(5).AP = disposalEmData.at(2).AP;
//    disposalEmData.at(5).ADP = disposalEmData.at(2).ADP;
//    disposalEmData.at(5).POCP = disposalEmData.at(2).POCP;
//    disposalEmData.at(5).OLDP = disposalEmData.at(2).OLDP;
//    disposalEmData.at(5).PMFP = disposalEmData.at(2).PMFP;
//    disposalSpecificEnergy.at(5) = 0.05*convertUnit(KILO,WATTHOUR,NOPREFIX,JOULE, 1.);
    //Nickel
    disposalEmData.push_back(EmissionsData(std::string("Ni")));
    disposalEmData.at(4).CO2 = disposalEmData.at(2).CO2;
    disposalEmData.at(4).H2O = disposalEmData.at(2).H2O;
    disposalEmData.at(4).NOx = disposalEmData.at(2).NOx;
    disposalEmData.at(4).SO2 = disposalEmData.at(2).SO2;
    disposalEmData.at(4).CO = disposalEmData.at(2).CO;
    disposalEmData.at(4).CH4 = disposalEmData.at(2).CH4;
    disposalEmData.at(4).PM10 = disposalEmData.at(2).PM10;
    disposalEmData.at(4).GWP100 = disposalEmData.at(2).GWP100;
    disposalEmData.at(4).AP = disposalEmData.at(2).AP;
    disposalEmData.at(4).ADP = disposalEmData.at(2).ADP;
    disposalEmData.at(4).POCP = disposalEmData.at(2).POCP;
    disposalEmData.at(4).OLDP = disposalEmData.at(2).OLDP;
    disposalEmData.at(4).PMFP = disposalEmData.at(2).PMFP;
    disposalSpecificEnergy.push_back(0.05 * convertUnit(KILO, WATTHOUR, NOPREFIX, JOULE, 1.));
    //Plastic
//    disposalEmData.push_back(std::string("Pl"));
//    disposalEmData.at(7).CO2 = 6.26*pow(10,-2);
//    disposalEmData.at(7).H2O = 1.44*pow(10,-1);
//    disposalEmData.at(7).NOx = 1.39*pow(10,-4);
//    disposalEmData.at(7).SO2 = 1.18*pow(10,-4);
//    disposalEmData.at(7).CO = 1.1*pow(10,-4);
//    disposalEmData.at(7).CH4 = 3.5*pow(10,-4);
//    disposalEmData.at(7).PM10 = 9.56*pow(10,-9);
//    disposalEmData.at(7).GWP100 = 7.16*pow(10,-2);
//    disposalEmData.at(7).AP = 2.22*pow(10,-4);
//    disposalEmData.at(7).ADP = 1.42*pow(10,-8);
//    disposalEmData.at(7).POCP = 2.77*pow(10,-5);
//    disposalEmData.at(7).OLDP = 2.79*pow(10,-12);
//    disposalEmData.at(7).PMFP = 4.27*pow(10,-4);
//    disposalSpecificEnergy.at(7) = 0.34*convertUnit(KILO,WATTHOUR,NOPREFIX,JOULE, 1.);
}

ecoDatabase::EmissionsData::EmissionsData()
    :
    specification("notSpecified"),
    CO2(0.),
    H2O(0.),
    NOx(0.),
    SO2(0.),
    CO(0.),
    CH4(0.),
    PM10(0.),
    GWP100(0.),
    AP(0.),
    ADP(0.),
    POCP(0.),
    OLDP(0.),
    PMFP(0.),
    specificEnergy(0.) {
}

ecoDatabase::EmissionsData::EmissionsData(const std::string& theSpecification)
    :
    specification(theSpecification),
    CO2(0.),
    H2O(0.),
    NOx(0.),
    SO2(0.),
    CO(0.),
    CH4(0.),
    PM10(0.),
    GWP100(0.),
    AP(0.),
    ADP(0.),
    POCP(0.),
    OLDP(0.),
    PMFP(0.),
    specificEnergy(0.) {
}
ecoDatabase::EmissionsData::~EmissionsData() {
}

ecoDatabase::~ecoDatabase() {
//dtor
}
