/*  Copyright (C) 2012-2022 Institute of Aerospace Systems, RWTH Aachen University - All rights reserved.
    This file is part of UNICADO.

    UNICADO is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    UNICADO is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with UNICADO.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "dallaraIOData.h"
#include "dallaraConfig.h"
#include <aixml/node.h>
#include <aixml/endnode.h>
#include <standardFiles/functions.h>
#include <unitConversion/unitConversion.h>
#include <vector>


DallaraIOData::DallaraIOData(const node& acXML, const node& config, std::shared_ptr<node> ecoAssXML)
    :
    configuration(DallaraConfig()),
    rangeStudy(0.),
    topOfClimbRange(0.),
    topOfDescentRange(0.),
    flightsPerYear(0),
    endOfCruiseStep(),
    flightLevel(),
    ATR(0.) {
        configuration.readAllConfigData(config);
        readAircraft(acXML);
        readEcologicalAssessmentXML(*ecoAssXML);
    }

/* Input data methods*/
void DallaraIOData::readAircraft(const node& acXML) {
    rangeStudy = EndnodeReadOnly<double>("requirements_and_specifications/requirements/top_level_aircraft_requirements/study_mission/range").read(acXML).value();
    topOfClimbRange = EndnodeReadOnly<double>("analysis/mission/study_mission/cruise/top_of_climb_range").read(acXML).value();
    topOfDescentRange = EndnodeReadOnly<double>("analysis/mission/study_mission/cruise/top_of_descent_range").read(acXML).value();
    flightsPerYear = EndnodeReadOnly<int>("assessment/cost_estimation/operating_cost/direct_operating_cost/flights_per_year_study_mission").read(acXML).value();
    durationOperation = EndnodeReadOnly<int>("requirements_and_specifications/assessment_scenario/duration_operation").read(acXML).value_xml();
    emissionMass[7] = convertUnit(METER, NAUTICALMILE, rangeStudy); //AIC; Climateimpact over flown length [nm] not over emitted emissions
    int numberOfCruiseSteps = acXML.getVector("aircraft_exchange_file/analysis/mission/study_mission/cruise/cruise_steps/cruise_step").size();
    for (int i = 0; i < numberOfCruiseSteps; i++) {
        EndnodeReadOnly<double> cruiseStepEnd("analysis/mission/study_mission/cruise/cruise_steps/cruise_step@" + num2Str(i) + "/relative_end_of_cruise_step");
        endOfCruiseStep.push_back(cruiseStepEnd.read(acXML).value());
        EndnodeReadOnly<double> cruiseStepAlt("analysis/mission/study_mission/cruise/cruise_steps/cruise_step@" + num2Str(i) + "/altitude");
        flightLevel.push_back(convertUnit(METER, FOOT, cruiseStepAlt.read(acXML).value())/100.);
    }
}

void DallaraIOData::readEcologicalAssessmentXML(const node& ecoAssXML) {
    /* Fill std::vector emissionMass: emitted emissions per mission or for AIC only route length*/ //
    std::string emissionsPath("ecological_data_file/mission_emissions/study_mission/emissions/");
    emissionMass[0] = EndnodeReadOnly<double>("LTO_cycle/CO2").read(ecoAssXML.at(emissionsPath)).value() //CO2
                        + EndnodeReadOnly<double>("cruise/CO2").read(ecoAssXML.at(emissionsPath)).value();
    emissionMass[1] = EndnodeReadOnly<double>("LTO_cycle/NOx").read(ecoAssXML.at(emissionsPath)).value() //CH4 (via NOx)
                        + EndnodeReadOnly<double>("cruise/NOx").read(ecoAssXML.at(emissionsPath)).value();
    emissionMass[2] = EndnodeReadOnly<double>("LTO_cycle/NOx").read(ecoAssXML.at(emissionsPath)).value() //O3L (via NOx)
                        + EndnodeReadOnly<double>("cruise/NOx").read(ecoAssXML.at(emissionsPath)).value();
    emissionMass[3] = EndnodeReadOnly<double>("LTO_cycle/NOx").read(ecoAssXML.at(emissionsPath)).value() //O3S (via NOx)
                        + EndnodeReadOnly<double>("cruise/NOx").read(ecoAssXML.at(emissionsPath)).value();
    emissionMass[4] = EndnodeReadOnly<double>("LTO_cycle/H2O").read(ecoAssXML.at(emissionsPath)).value() //H2O
                        + EndnodeReadOnly<double>("cruise/H2O").read(ecoAssXML.at(emissionsPath)).value();
    emissionMass[5] = EndnodeReadOnly<double>("LTO_cycle/soot").read(ecoAssXML.at(emissionsPath)).value() //Soot
                        + EndnodeReadOnly<double>("cruise/soot").read(ecoAssXML.at(emissionsPath)).value();
    emissionMass[6] = EndnodeReadOnly<double>("LTO_cycle/SO4").read(ecoAssXML.at(emissionsPath)).value() //SO4
                        + EndnodeReadOnly<double>("cruise/SO4").read(ecoAssXML.at(emissionsPath)).value();
}

/* Output data methods*/
void DallaraIOData::updateAircraftXML(std::shared_ptr<RuntimeIO> rtIO) {
    Endnode<double> ATRnode("average_temperature_response",
                            "Integrated temperature change per year caused by aircraft operation divided by operating lifetime",
                            ATR, "K", 0, 1e-5);
    std::string subpath("assessment/ecological_assessment/");
    if (!(rtIO->acxml.find(subpath))) { // if ecological_assessment node doesn't exist, add one
        rtIO->acxml[subpath];
        rtIO->acxml.at(subpath).addAttrib("description", "Ecological assessment data");
    }
    ATRnode.update(rtIO->acxml.at("aircraft_exchange_file/assessment/ecological_assessment/"));
    rtIO->saveAcXML();
}

