# Import standard modules.
import math

# import own modules.
from src.wing_mounted.empirical.landing_gear_design_tu_berlin.general.aicraft_classification_number.flexible_pavement.cvrg import cvrg  # noPep8 e501
from src.wing_mounted.empirical.landing_gear_design_tu_berlin.general.aicraft_classification_number.flexible_pavement.alpha_factor_from_curve import alpha_factor_from_curve  # noPep8 e501
from src.wing_mounted.empirical.landing_gear_design_tu_berlin.general.aicraft_classification_number.flexible_pavement.alpha_factor_from_curve import new_alpha_factor_from_curve  # noPep8 e501


def acn_flex_comp(dict_acn_flex_parameter, dict_constants, runtime_output):
    """ function to estimate the acn of an aircraft for flexible pavements in accordance to ICAO COMFAA tool.

    :param dict dict_acn_flex_parameter: Dict containing initial parameter and values for acn estimation
    :param dict dict_constants: Dict containing constants for acn estimation
    :param logging.Logger runtime_output: Logging object used for capturing log messages in the module
    :return dict dict_acn_flex_parameter: Dict containing parameter and values of current acn estimation
    """
    ''' initialize local parameter '''
    number_of_cbr_subgrades = 1  # NCBRS
    number_of_integrate = 10  # NIntegrate
    subgrade_index_start = 2  # KZStart
    i = dict_acn_flex_parameter['XK']
    if dict_acn_flex_parameter['YK'] > i:
        i = dict_acn_flex_parameter['YK']
    dict_acn_flex_parameter['XG'] = [None] * i
    dict_acn_flex_parameter['YG'] = [None] * i

    # try to estimate the acn of specific aircraft project for flexible pavements
    try:
        # estimate total aircraft weight in [kilo newton]
        dict_acn_flex_parameter['WT'] = dict_acn_flex_parameter['AMASS'] * dict_constants['g0'] / 1000  # WT
        # estimate total aircraft weight on main landing gear group in [kilo newton]
        dict_acn_flex_parameter['TLMG'] = dict_acn_flex_parameter['WT'] * dict_acn_flex_parameter['PMMG'] / 100  # TLMG
        # estimate total aircraft weight on one main landing gear strut in [kilo newton]
        dict_acn_flex_parameter['TLSMG'] = dict_acn_flex_parameter['TLMG'] / dict_acn_flex_parameter['AMLG']  # TLSMG
        # estimate total aircraft weight on one single wheel of oen main landing gear strut in [kilo newton]
        dict_acn_flex_parameter['TLSW'] = dict_acn_flex_parameter['TLSMG'] / dict_acn_flex_parameter['NW']  # TLSW
        # estimate the contact of one single wheel of one main landing gear strut in [cm * cm]  # ARESW
        dict_acn_flex_parameter['ARESW'] = dict_acn_flex_parameter['TLSW'] * 10000 / dict_acn_flex_parameter['PRSW']
        # estimate the radius of contact area of one single wheel of one main landing gear strut in [cm]
        dict_acn_flex_parameter['RESW'] = math.sqrt(dict_acn_flex_parameter['ARESW'] / dict_constants['pi'])  # RESW
        # estimate the single tire pressure in [MPa]
        dict_acn_flex_parameter['PESW'] = dict_acn_flex_parameter['PRSW'] / 1000  # PESW
        # estimate the contact area of one single wheel of one main landing gear strut in [inch * inch]
        dict_acn_flex_parameter['CAREA'] = ((dict_acn_flex_parameter['RESW'] ** 2) * dict_constants['pi']) / \
                                           (dict_constants['inch_to_cm'] * dict_constants['inch_to_cm'])
        # estimate the square of the single wheel contact area in [inch * inch]
        dict_acn_flex_parameter['RESW2'] = (dict_acn_flex_parameter['RESW'] ** 2) / \
                                           (dict_constants['inch_to_cm'] * dict_constants['inch_to_cm'])  # RESW2
        # initialize array of all wheels of one main landing gear strut with tire pressure in [kPA]
        dict_acn_flex_parameter['PRS'] = [dict_acn_flex_parameter['PRSW']] * dict_acn_flex_parameter['NW']  # PRSW
        # initialize array of all wheels of one main landing gear strut with tire pressure in [Psi]
        dict_acn_flex_parameter['PR'] = [dict_acn_flex_parameter['PESW']] * dict_acn_flex_parameter['NW']  # PR
        # initialize array of all wheels of one main landing gear strut with tire contact radii in [cm]
        dict_acn_flex_parameter['Rad'] = [dict_acn_flex_parameter['RESW']] * dict_acn_flex_parameter['NW']  # Rad

        # check if the current aircraft is not the first in estimation
        #  -> Attention: this is only implemented in accordance to ICAO
        #    -> in normal preliminary aircraft design not necessary because only one single aircraft will be developed
        if not dict_acn_flex_parameter['IPRS'] == 1:
            # convert the input x and y coordinates of each wheel of current aircraft from inch to meter
            for x in range(0, dict_acn_flex_parameter['NW']):
                dict_acn_flex_parameter['X'][x] = dict_acn_flex_parameter['X'][x] * dict_constants['inch_to_cm'] / 100
                dict_acn_flex_parameter['Y'][x] = dict_acn_flex_parameter['Y'][x] * dict_constants['inch_to_cm'] / 100

            # convert the input grid x and y coordinates of current aircraft from inch to meter
            dict_acn_flex_parameter['GX'] = dict_acn_flex_parameter['GX'] * dict_constants['inch_to_cm'] / 100
            dict_acn_flex_parameter['DGX'] = dict_acn_flex_parameter['DGX'] * dict_constants['inch_to_cm'] / 100
            dict_acn_flex_parameter['GY'] = dict_acn_flex_parameter['GY'] * dict_constants['inch_to_cm'] / 100
            dict_acn_flex_parameter['DGY'] = dict_acn_flex_parameter['DGY'] * dict_constants['inch_to_cm'] / 100
            dict_acn_flex_parameter['GZ'] = dict_acn_flex_parameter['GZ'] * dict_constants['inch_to_cm'] / 100

        # else condition: current aircraft is the first in what is under consideration
        #  -> in aircraft design process that is the only condition to use -> because only one aircraft is developed
        # 120
        else:
            dict_acn_flex_parameter['NAL'] = 1
            # check if a new alpha factor should be estimated from alpha curves
            #  -> if true: -> call function to estimate alpha from curve
            if dict_acn_flex_parameter['select_new_alpha_from_curve']:
                # call function to estimate new alpha factor from curve for current aircraft project
                dict_acn_flex_parameter['B'][0] = new_alpha_factor_from_curve(dict_acn_flex_parameter, runtime_output)

            # else condition: use default alpha factors from ICAO for ACN estimation
            else:
                # check if the entered user input for the number of coverages is equal
                # to the number of standard coverages (10.000) given by the ICAO -> if true:
                #  -> use hard coded default values for the alpha factor depending on the number of wheels
                #   of one main landing gear strut given by the ICAO
                if dict_acn_flex_parameter['coverages'] == dict_acn_flex_parameter['StandardCoverages']:
                    # check the number of wheels on one main landing gear strut and select the specific alpha factor
                    if dict_acn_flex_parameter['NW'] == 1:
                        dict_acn_flex_parameter['B'][0] = 0.995
                    elif dict_acn_flex_parameter['NW'] == 2:
                        dict_acn_flex_parameter['B'][0] = 0.9
                    elif dict_acn_flex_parameter['NW'] == 4:
                        dict_acn_flex_parameter['B'][0] = 0.825
                    elif dict_acn_flex_parameter['NW'] == 6:
                        dict_acn_flex_parameter['B'][0] = 0.72
                    elif dict_acn_flex_parameter['NW'] == 8:
                        dict_acn_flex_parameter['B'][0] = 0.755
                    elif dict_acn_flex_parameter['NW'] == 12:
                        dict_acn_flex_parameter['B'][0] = 0.755
                    elif dict_acn_flex_parameter['NW'] == 16:
                        dict_acn_flex_parameter['B'][0] = 0.705
                    elif dict_acn_flex_parameter['NW'] == 18:
                        dict_acn_flex_parameter['B'][0] = 0.7
                    elif dict_acn_flex_parameter['NW'] == 24:
                        dict_acn_flex_parameter['B'][0] = 0.689
                    else:
                        # call function to estimate alpha factor from curve
                        dict_acn_flex_parameter['B'][0] = alpha_factor_from_curve(dict_acn_flex_parameter,
                                                                                  runtime_output)
                else:
                    # call function to estimate alpha factor from curve
                    dict_acn_flex_parameter['B'][0] = alpha_factor_from_curve(dict_acn_flex_parameter, runtime_output)

            # check if the own input alpha factor is greater then zero -> if true: -> set the input alpha factor
            if dict_acn_flex_parameter['InputAlpha'] > 0:
                dict_acn_flex_parameter['B'][0] = dict_acn_flex_parameter['InputAlpha']

        # 150
        # convert tire pressure and contact radii from meter to inch
        for i in range(0, dict_acn_flex_parameter['NW']):
            dict_acn_flex_parameter['PR'][i] = dict_acn_flex_parameter['PR'][i] * dict_constants['mpa_to_psi']
            dict_acn_flex_parameter['Rad'][i] = dict_acn_flex_parameter['Rad'][i] / dict_constants['inch_to_cm']

        # check if it is possible to initialise the following statements in imperial units
        dict_acn_flex_parameter['RESW'] = dict_acn_flex_parameter['RESW'] / dict_constants['inch_to_cm']
        dict_acn_flex_parameter['PESW'] = dict_acn_flex_parameter['PESW'] * dict_constants['mpa_to_psi']
        dict_acn_flex_parameter['DZ'] = dict_acn_flex_parameter['DZ'] / dict_constants['inch_to_cm']
        dict_acn_flex_parameter['DPE'] = 3.0 / (2.0 * dict_constants['pi'] *
                                                dict_acn_flex_parameter['RESW'] * dict_acn_flex_parameter['PESW'])

        if not dict_acn_flex_parameter['InputCBR'] == 0.0:
            dict_acn_flex_parameter['SCBR'] = dict_acn_flex_parameter['InputCBR']
        else:
            number_of_cbr_subgrades = len(dict_acn_flex_parameter['SCBR'])

        # 180
        # loop across all given subgrades to estimate acn flex depending on this
        for i_cbr in range(0, number_of_cbr_subgrades):
            dict_acn_flex_parameter['TRGT'] = dict_acn_flex_parameter['SCBR'][i_cbr]

            # reinitialize ITCT in each loop with zero for use in sub function cvrg
            dict_acn_flex_parameter['ITCT'] = 0

            # estimate of design depth from abbreviated CBR (line 322 ICAO Code)
            dict_acn_flex_parameter['DZ'] = math.sqrt(dict_acn_flex_parameter['CAREA'] *
                                                      dict_acn_flex_parameter['PESW'] / 10.0 /
                                                      dict_acn_flex_parameter['SCBR'][i_cbr])

            # estimate grid depth
            for i in range(subgrade_index_start - 1, dict_acn_flex_parameter['KZ']):
                dict_acn_flex_parameter['Z'][i] = dict_acn_flex_parameter['Z'][i-1] + dict_acn_flex_parameter['DZ']
                dict_acn_flex_parameter['Z2'][i] = dict_acn_flex_parameter['Z'][i] * dict_acn_flex_parameter['Z'][i]

            # loop to achieve the convergence of cbr curves
            number_of_iterations = 0  # NIter
            # 200
            while dict_acn_flex_parameter['ITER']:
                curve_point_found = False
                dict_acn_flex_parameter['Z2'][dict_acn_flex_parameter['KZ'] - 1] = \
                    dict_acn_flex_parameter['Z'][dict_acn_flex_parameter['KZ'] - 1] * \
                    dict_acn_flex_parameter['Z'][dict_acn_flex_parameter['KZ'] - 1]

            # 210
                # SET UP SIN(PHI) SQ AND COS(PHI)
                dict_acn_flex_parameter['SN2'] = []
                dict_acn_flex_parameter['CS'] = []
                dict_acn_flex_parameter['RAD2'] = []
                dict_acn_flex_parameter['PID'] = dict_constants['pi'] / 2 / number_of_integrate  # PID
                dict_acn_flex_parameter['PHI'] = -dict_acn_flex_parameter['PID']/2  # PHI
                for i in range(0, number_of_integrate):
                    dict_acn_flex_parameter['PHI'] = dict_acn_flex_parameter['PHI'] + dict_acn_flex_parameter['PID']
                    # angles over one half of a quadrant (PI / 2).
                    dict_acn_flex_parameter['SINE'] = math.sin(dict_acn_flex_parameter['PHI'])  # SINE
                    # array of sine square
                    dict_acn_flex_parameter['SN2'].append(dict_acn_flex_parameter['SINE']
                                                          * dict_acn_flex_parameter['SINE'])  # SN2
                    # array of cos values of phi
                    dict_acn_flex_parameter['CS'].append(math.cos(dict_acn_flex_parameter['PHI']))  # CS

                # list of tire contact radii squared
                for i in range(0, dict_acn_flex_parameter['NW']):
                    dict_acn_flex_parameter['RAD2'].append(dict_acn_flex_parameter['Rad'][i] ** 2)

                # set-up x-coordinates for grid
                dict_acn_flex_parameter['XG'][0] = dict_acn_flex_parameter['GX']
                for i in range(1, dict_acn_flex_parameter['KX']):
                    dict_acn_flex_parameter['XG'][i] = dict_acn_flex_parameter['XG'][i-1] + \
                                                       dict_acn_flex_parameter['DGX']

                # set-up y-coordinates for grid
                dict_acn_flex_parameter['YG'][0] = dict_acn_flex_parameter['GY']
                for i in range(1, dict_acn_flex_parameter['KY']):
                    dict_acn_flex_parameter['YG'][i] = dict_acn_flex_parameter['YG'][i - 1] + \
                                                       dict_acn_flex_parameter['DGY']

                dict_acn_flex_parameter['S'] = []
                # loop across all y-grid points to estimate tire specific parameter depending on the y-grid point
                for i_it in range(0, dict_acn_flex_parameter['KX']):
                    dict_acn_flex_parameter['XJ'] = dict_acn_flex_parameter['XG'][i_it]

                    # loop across all x-grid points to estimate tire specific parameter depending on the x-grid point
                    #  for current y-grid point
                    array_of_current_j_values = []
                    for j_it in range(0, dict_acn_flex_parameter['KY']):
                        dict_acn_flex_parameter['YI'] = dict_acn_flex_parameter['YG'][j_it]

                        # loop across the depth
                        for k_it in range(subgrade_index_start - 1, dict_acn_flex_parameter['KZ']):
                            dict_acn_flex_parameter['Z2K'] = dict_acn_flex_parameter['Z2'][k_it]
                            dict_acn_flex_parameter['SS'] = 0

                            # loop over wheels on one gear
                            for l_it in range(0, dict_acn_flex_parameter['NW']):
                                # tire contact radius squared.
                                dict_acn_flex_parameter['RAD2L'] = dict_acn_flex_parameter['RAD2'][l_it]
                                # tire contact radius
                                dict_acn_flex_parameter['RADL'] = dict_acn_flex_parameter['Rad'][l_it]
                                # tire contact pressure
                                dict_acn_flex_parameter['PRL'] = dict_acn_flex_parameter['PR'][l_it]
                                # reset variable 'SJIK' to zero for each loop
                                dict_acn_flex_parameter['SJIK'] = 0
                                # x distance from wheel to grid point
                                dict_acn_flex_parameter['XLG'] = \
                                    dict_acn_flex_parameter['X'][l_it] - dict_acn_flex_parameter['XJ']
                                # square of x distance from wheel to grid point
                                dict_acn_flex_parameter['R2'] = dict_acn_flex_parameter['XLG'] ** 2
                                # y distance from wheel to grid point
                                dict_acn_flex_parameter['YLG'] = \
                                    dict_acn_flex_parameter['Y'][l_it] - dict_acn_flex_parameter['YI']
                                # sum of the squares of x and y distance from wheel to grid point
                                dict_acn_flex_parameter['R2'] = \
                                    dict_acn_flex_parameter['YLG'] ** 2 + dict_acn_flex_parameter['R2']
                                # normalized distance from wheel to grid point.
                                dict_acn_flex_parameter['R'] = math.sqrt(dict_acn_flex_parameter['R2'])

                                # check if the difference between normalized distance and tire contact radius
                                #  is greater then zero -> if true: -> estimate parameter 'SJIK' for this case
                                #   (ICAO code line: 429)
                                if (dict_acn_flex_parameter['R'] - dict_acn_flex_parameter['RADL']) > 0:
                                    # 290
                                    for m_it in range(0, number_of_integrate):
                                        # estimate Bousinesq half-space
                                        dict_acn_flex_parameter['SAR'] = math.sqrt(dict_acn_flex_parameter['R2'] -
                                                                                   dict_acn_flex_parameter['RAD2L'] *
                                                                                   dict_acn_flex_parameter['SN2'][m_it])
                                        # estimate Bousinesq half-space for grid
                                        dict_acn_flex_parameter['AC'] = \
                                            dict_acn_flex_parameter['RADL'] * dict_acn_flex_parameter['CS'][m_it]
                                        # estimate point outside the tire
                                        dict_acn_flex_parameter['SPA'] = \
                                            dict_acn_flex_parameter['AC'] + dict_acn_flex_parameter['SAR']
                                        # square of estimated point outside the tire (contact area)
                                        dict_acn_flex_parameter['SPA2'] = dict_acn_flex_parameter['SPA'] ** 2
                                        # estimate variables for 'SJIK' calculation
                                        dict_acn_flex_parameter['SSA'] = \
                                            dict_acn_flex_parameter['SAR'] - dict_acn_flex_parameter['AC']
                                        dict_acn_flex_parameter['SSA2'] = dict_acn_flex_parameter['SSA'] ** 2
                                        # estimate 'SJIK' of current tire on current y- and x-grid point
                                        dict_acn_flex_parameter['SJIK'] = \
                                            dict_acn_flex_parameter['SJIK'] + dict_acn_flex_parameter['AC'] / \
                                            dict_acn_flex_parameter['SAR'] * \
                                            (dict_acn_flex_parameter['SPA2'] /
                                             math.sqrt(dict_acn_flex_parameter['SPA2'] +
                                                       dict_acn_flex_parameter['Z2K']) -
                                             dict_acn_flex_parameter['SSA2'] /
                                             math.sqrt(dict_acn_flex_parameter['SSA2'] +
                                                       dict_acn_flex_parameter['Z2K'])) * dict_acn_flex_parameter['PID']

                                # else condition:
                                #  -> sum of displacement due to one wheel is greater or equal to normalized distance
                                else:
                                    # 260
                                    for m_it in range(0, number_of_integrate):
                                        # estimate Bousinesq half-space
                                        dict_acn_flex_parameter['SAR'] = math.sqrt(dict_acn_flex_parameter['RAD2L'] -
                                                                                   dict_acn_flex_parameter['R2'] *
                                                                                   dict_acn_flex_parameter['SN2'][m_it])

                                        # estimate deflection times -> constant at an arbitrary point in a
                                        #  Bousinesq half space with circular load
                                        dict_acn_flex_parameter['RC'] = \
                                            dict_acn_flex_parameter['R'] * dict_acn_flex_parameter['CS'][m_it]
                                        dict_acn_flex_parameter['ERP'] = \
                                            dict_acn_flex_parameter['RC'] + dict_acn_flex_parameter['SAR']
                                        dict_acn_flex_parameter['ERP2'] = dict_acn_flex_parameter['ERP'] ** 2
                                        dict_acn_flex_parameter['ERM'] = \
                                            dict_acn_flex_parameter['SAR'] - dict_acn_flex_parameter['RC']
                                        dict_acn_flex_parameter['ERM2'] = dict_acn_flex_parameter['ERM'] ** 2

                                        # reset value of DW2 to zero
                                        dict_acn_flex_parameter['DW2'] = 0
                                        # estimate delta of wheel point
                                        dict_acn_flex_parameter['DW1'] = math.sqrt(dict_acn_flex_parameter['ERP2']
                                                                                   + dict_acn_flex_parameter['Z2K'])

                                        # check if the estimated wheel point is not equal to zero
                                        #  -> if true: -> estimate new value for second wheel point
                                        if not dict_acn_flex_parameter['DW1'] == 0:
                                            dict_acn_flex_parameter['DW2'] = \
                                                dict_acn_flex_parameter['ERP2'] / dict_acn_flex_parameter['DW1']

                                        # reset value of DW2 to zero
                                        dict_acn_flex_parameter['DW3'] = 0
                                        # estimate delta of wheel point
                                        dict_acn_flex_parameter['DW1'] = math.sqrt(dict_acn_flex_parameter['ERM2']
                                                                                   + dict_acn_flex_parameter['Z2K'])
                                        # check if the estimated wheel point is not equal to zero
                                        #  -> if true: -> estimate new value for third wheel point
                                        if not dict_acn_flex_parameter['DW1'] == 0:
                                            dict_acn_flex_parameter['DW3'] = \
                                                dict_acn_flex_parameter['ERM2'] / dict_acn_flex_parameter['DW1']

                                        # estimate 'SJIK' of current tire on current y- and x-grid point
                                        dict_acn_flex_parameter['SJIK'] = \
                                            dict_acn_flex_parameter['SJIK'] + (dict_acn_flex_parameter['DW2']
                                                                               + dict_acn_flex_parameter['DW3']) \
                                            * dict_acn_flex_parameter['PID']

                                # estimate total 'SJIK' and SS in accordance to ICAO line 443 (ACNflexICAO.bas)
                                dict_acn_flex_parameter['SJIK'] = \
                                    dict_acn_flex_parameter['SJIK'] * dict_acn_flex_parameter['PRL']
                                dict_acn_flex_parameter['SS'] = \
                                    dict_acn_flex_parameter['SS'] + dict_acn_flex_parameter['SJIK']

                        # store sum of current loop in temporary array for j values
                        array_of_current_j_values.append(dict_acn_flex_parameter['SS'])

                    # store list of j values in list of list 'S' on each index of i_it
                    dict_acn_flex_parameter['S'].append(array_of_current_j_values)

                # find maximum of all points from list 'S'
                dict_acn_flex_parameter['SD'] = max(max(item) for item in dict_acn_flex_parameter['S'])

                # calculate equivalent double wheel load
                #  -> This ESWL is not the ACN DSWL. Only used to find pavement thickness.
                dict_acn_flex_parameter['ESWL'] = []
                for i in range(subgrade_index_start - 1, dict_acn_flex_parameter['KZ']):
                    equivalent_single_wheel_load = dict_acn_flex_parameter['SD'] * \
                                                   math.sqrt(dict_acn_flex_parameter['RESW2'] +
                                                             dict_acn_flex_parameter['Z2'][i])
                    dict_acn_flex_parameter['ESWL'].append(equivalent_single_wheel_load)

                dict_acn_flex_parameter['SRAREA'] = math.sqrt(dict_acn_flex_parameter['CAREA'])

                dict_acn_flex_parameter['TA'] = []
                for j in range(subgrade_index_start - 1, dict_acn_flex_parameter['KZ']):
                    dict_acn_flex_parameter['TA'].append(dict_acn_flex_parameter['Z'][j] / 
                                                         dict_acn_flex_parameter['B'][0])
                    dict_acn_flex_parameter['V1'] = \
                        dict_acn_flex_parameter['TA'][j-1] / dict_acn_flex_parameter['SRAREA']

                    # check if the estimated value for v1 is not equal to zero
                    #  -> if true: -> estimate california bearing ratio from curve in accordance to ICAO code line 485
                    if not dict_acn_flex_parameter['V1'] == 0:
                        # loop across all entries from 'V' array to find operating point on curve
                        for k in range(dict_acn_flex_parameter['KS'], len(dict_acn_flex_parameter['V'])):
                            # check if the value from curve is lower or equal to estimated value v1
                            #  -> if true: -> thickness value considered exceeds limits of curve
                            #   -> estimate california bearing ratio with last value of 'V' array
                            if dict_acn_flex_parameter['V'][k] <= dict_acn_flex_parameter['V1']:
                                if k == len(dict_acn_flex_parameter['V'])-1:
                                    california_bearing_ratio = \
                                        dict_acn_flex_parameter['U'][179] * (dict_acn_flex_parameter['ESWL'][j - 1] /
                                                                             dict_acn_flex_parameter['CAREA'])
                                    dict_acn_flex_parameter['CBR'] = california_bearing_ratio
                                    dict_acn_flex_parameter['ITER'] = False

                            # else condition: thickness value considered is in the limits of curve
                            #  in accordance to icao code line 493
                            else:
                                dict_acn_flex_parameter['UL'] = \
                                    math.log10(dict_acn_flex_parameter['U'][k-1]) - \
                                    ((dict_acn_flex_parameter['V1'] - dict_acn_flex_parameter['V'][k-1]) *
                                     (math.log10(dict_acn_flex_parameter['U'][k-1]) -
                                     math.log10(dict_acn_flex_parameter['U'][k]))) / (dict_acn_flex_parameter['V'][k] -
                                                                                      dict_acn_flex_parameter['V'][k-1])

                                # check if the estimated value of 'UL' is greater than 300 (no point on curve)
                                #  -> if true: -> set 'UL' to maximum point on curve (300)
                                if dict_acn_flex_parameter['UL'] > 300:
                                    dict_acn_flex_parameter['UL'] = 300

                                # estimate california bearing ration depending on the estimated point on curve
                                dict_acn_flex_parameter['U1'] = 10 ** dict_acn_flex_parameter['UL']
                                dict_acn_flex_parameter['CBR'] = \
                                    dict_acn_flex_parameter['U1'] * (dict_acn_flex_parameter['ESWL'][j-1] /
                                                                     dict_acn_flex_parameter['CAREA'])

                                # save estimated value vor california bearing ratio to compare in sub-function
                                dict_acn_flex_parameter['FCTN'] = dict_acn_flex_parameter['CBR']

                                # call function to check the coverage of estimate california bearing ratio and thickness
                                dict_acn_flex_parameter = cvrg(dict_acn_flex_parameter, runtime_output)

                                # break the loop of finding point on curve -> point was found
                                curve_point_found = True
                                break

                        if curve_point_found:
                            break

                    # else condition: estimated value for v1 is equal to zero
                    #  -> estimate california bearing ratio by using first entry from 'U' list
                    #   in accordance to ICAO code line 480
                    else:
                        if curve_point_found:
                            break
                        california_bearing_ratio = dict_acn_flex_parameter['U'][0] * \
                            (dict_acn_flex_parameter['ESWL'][j-1] / dict_acn_flex_parameter['CAREA'])
                        dict_acn_flex_parameter['CBR'] = california_bearing_ratio
                        dict_acn_flex_parameter['ITER'] = False

                # check if iteration of current cbr value is successfully
                #  -> if true: -> estimate ACN of current cbr (ICAO code lin 509ff.)
                if not dict_acn_flex_parameter['ITER']:
                    # estimate current acn
                    dict_acn_flex_parameter['ACN'] = \
                        (dict_acn_flex_parameter['Z'][dict_acn_flex_parameter['KZ'] - 1]
                         * dict_constants['inch_to_cm']) \
                        * (dict_acn_flex_parameter['Z'][dict_acn_flex_parameter['KZ'] - 1]
                           * dict_constants['inch_to_cm']) / 1000 / (0.878 / dict_acn_flex_parameter['FCTN'] - 0.01249)

                    # estimate current equivalent single wheel load
                    dict_acn_flex_parameter['ESWL'] = dict_acn_flex_parameter['ESWL'][0] / dict_constants['kg_to_lbs']

                    # estimate necessary runway thickness of current california bearing ration
                    for i in range(0, dict_acn_flex_parameter['KZ']):
                        dict_acn_flex_parameter['Z'][i] = dict_acn_flex_parameter['Z'][i] * dict_constants['inch_to_cm']

                    # store all values to output lists for print handling
                    dict_acn_flex_parameter['PESWL'].append(round(dict_acn_flex_parameter['ESWL'], 2))
                    dict_acn_flex_parameter['PCBR'].append(int(round(dict_acn_flex_parameter['CBR'], 0)))
                    dict_acn_flex_parameter['PZ'].append(round(dict_acn_flex_parameter['Z'][i] * 10, 2))
                    dict_acn_flex_parameter['PACN'].append(round(dict_acn_flex_parameter['ACN'], 1))
                    dict_acn_flex_parameter['AlphaFactor'].append(round(dict_acn_flex_parameter['B'][0], 4))
                    dict_acn_flex_parameter['CBRThickness'].append(round(dict_acn_flex_parameter['Z'][i]
                                                                   / dict_constants['inch_to_cm'], 2))

                else:
                    number_of_iterations += 1

            # reinitialize iteration flag for next cbr value in loop
            dict_acn_flex_parameter['ITER'] = True

    # exception handling if the code to estimate the acn for flexible pavements occurs an error
    except ValueError as e:
        # Error handling code here
        runtime_output.error("Error:", str(e))

    runtime_output.debug('acn_flex_comp successfully executed!')

    return dict_acn_flex_parameter
