# Import standard modules.
import math


def estimate_clearances(dict_ac_exchange, dict_mod_config, landing_gear_description, runtime_output):
    """ Hier beschreibung hin: Estimation of ground clearances in accordance to Torenbeek and CS-25
            [ref. E. Torenbeek; 1982]
            [ref. CS-25 Amendment 27; 2022]

    :param dict dict_ac_exchange: Dict containing parameter and according values from aircraft exchange file
    :param dict dict_mod_config: Dict containing parameter and according values from module configuration file
    :param dict landing_gear_description: Dict containing parameter and values of current landing gear estimation
    :param logging.Logger runtime_output: Logging object used for capturing log messages in the module
    :return dict landing_gear_description: Dict containing parameter and values of current landing gear estimation
    """
    ''' initialize local parameter '''
    specific_flag = False
    required_flag = False
    nacelle_clearance_angle = []
    wing_tip_clearance_angle = 0
    minimum_required_clearance_angle = 5  # according to CS25.149 -> minimum 5 degree bank angle
    propulsor_mounting_position = list(dict_ac_exchange['propulsor_mounting_position'].values())
    nacelle_y_position = {key: value for key, value in dict_ac_exchange['engine_y_position'].items() if
                          value is not None}
    nacelle_y_position = list(nacelle_y_position.values())
    nacelle_z_position = {key: value for key, value in dict_ac_exchange['engine_z_position'].items() if
                          value is not None}
    nacelle_z_position = list(nacelle_z_position.values())
    nacelle_section_height = {key: value for key, value in dict_ac_exchange['nacelle_section_height'].items() if
                              value is not None}
    nacelle_section_height = list(nacelle_section_height.values())
    main_gear_outer_strut_y_position = landing_gear_description['landing_gear_positions'][
        'main_gear_outer_strut_y_position']

    ''' check nacelle clearance '''
    try:
        # check if at least one engine exist in the aircraft exchange file
        if len(nacelle_y_position) > 0:
            index = 0
            # loop across all existing engines
            for position in nacelle_y_position:
                # check if the current engine is mounted to the wing -> if true: -> estimate clearance angle
                if propulsor_mounting_position[index] == 'wing':
                    # check if the y-position of current engine is not within the outer landing gear leg position
                    #  -> if true: -> estimate clearance angle
                    if abs(position) > main_gear_outer_strut_y_position:
                        # estimate clearance angle between outer main landing gear strut and lowest point of nacelle
                        distances = abs(position) - main_gear_outer_strut_y_position
                        nacelle_ground_clearance = round(landing_gear_description['lever_arms'][
                            'vertical_distance_between_ground_and_fuselage_center_line']
                                             - abs(nacelle_z_position[index])
                                             - abs(nacelle_section_height[index]/2), 4)
                        nacelle_clearance_angle.append(
                            round(math.degrees(math.atan(nacelle_ground_clearance/distances)), 4))
                    # Else condition: current engine is within the outer landing gear leg position
                    #  -> raise an error and skip nacelle ground clearance estimation
                    else:
                        raise ValueError('At least one wing mounted engine is within the outer landing gear leg '
                                         'position. No valid configuration!')
                    index += 1

            # loop across all existing nacelle clearance angles to check angle requirements
            for angle in nacelle_clearance_angle:
                if angle < dict_mod_config['nacelle_clearance'] and not angle < minimum_required_clearance_angle:
                    specific_flag = True
                elif angle < minimum_required_clearance_angle:
                    required_flag = True

            if specific_flag and not required_flag:
                raise ValueError('At least one wing mounted engine falls below the specified clearance angle of: '
                                 + str(dict_mod_config['nacelle_clearance']) + ' degrees!')
            elif required_flag:
                raise ValueError('At least one wing mounted engine falls below the required clearance angle of the '
                                 'CS25.149 of: '
                                 + str(minimum_required_clearance_angle) + ' degrees!')
            else:
                runtime_output.print('All engines mounted to the wing conforms the specified clearance angle of: '
                                     + str(dict_mod_config['nacelle_clearance']) + ' degrees!')

        # Else condition: no engine exists in the aircraft exchange file -> print a warning
        else:
            runtime_output.warning('Attention: No engines available in the aircraft exchange file, '
                                   'nacelle ground clearance could not be checked!')

    # exception handling for nacelle ground clearance check
    except ValueError as e:
        runtime_output.error('Error: ' + str(e))

    ''' check wing tip clearance '''
    try:
        _, wing_reference_point_on_tip_in_y_value = \
            next(reversed(list(dict_ac_exchange['wing_section_position_in_y'].items())), (None, None))
        _, wing_reference_point_on_tip_in_z_value = \
            next(reversed(list(dict_ac_exchange['wing_section_position_in_z'].items())), (None, None))

        if wing_reference_point_on_tip_in_y_value is not None and wing_reference_point_on_tip_in_z_value is not None:
            distance_between_wing_tip_and_outer_strut = \
                round(wing_reference_point_on_tip_in_y_value
                      - landing_gear_description['landing_gear_positions']['main_gear_outer_strut_y_position'], 4)
            wing_tip_ground_clearance = \
                (landing_gear_description['lever_arms']['vertical_distance_between_ground_and_fuselage_center_line']
                 - wing_reference_point_on_tip_in_z_value)
            wing_tip_clearance_angle = \
                round(math.degrees(math.atan(wing_tip_ground_clearance/distance_between_wing_tip_and_outer_strut)), 4)

            if wing_tip_clearance_angle < dict_mod_config['wing_strike_limit'] \
                    and not wing_tip_clearance_angle < minimum_required_clearance_angle:
                raise ValueError('The resulting wing tip angle is below the specified clearance angle of: '
                                 + str(dict_mod_config['wing_strike_limit']) + ' degrees!')
            elif wing_tip_clearance_angle < minimum_required_clearance_angle:
                raise ValueError('The resulting wing tip angle is below the required clearance angle of the '
                                 'CS25.149 of: ' + str(minimum_required_clearance_angle) + ' degrees!')
            else:
                runtime_output.print('The resulting wing tip angle conforms the specified clearance angle of: '
                                     + str(dict_mod_config['wing_strike_limit']) + ' degrees!')

        else:
            raise ValueError('At least one wing parameter is missing in the aircraft exchange file, '
                             'wing tip clearance could not be checked!')

    # exception handling for wing tip clearance check
    except ValueError as e:
        runtime_output.error('Error: ' + str(e))

    landing_gear_description['landing_gear_limitations']['nacelle_ground_clearance_angles'] = nacelle_clearance_angle
    landing_gear_description['landing_gear_limitations']['wing_tip_clearance_angles'] = wing_tip_clearance_angle

    # debug print
    runtime_output.debug('estimate_clearances successfully executed!')

    return landing_gear_description
