/**
 * \file flight_conditions.h
 * \author Gerrit Pauls (gerrit.pauls@tuhh.de)
 * \brief
 * \version 0.1
 * \date 2024-08-30
 *
 * @copyright Copyright (c) 2024
 *
 */

#ifndef MISSION_ANALYSIS_SRC_LIBS_CONDITION_METHODS_Flight_conditions_H_
#define MISSION_ANALYSIS_SRC_LIBS_CONDITION_METHODS_Flight_conditions_H_

#include <map>
#include <memory>
#include <string>
#include <utility>
#include <vector>

#include <aerodynamics/aerodynamics.h>
#include <engine/engine.h>

#include "operating_conditions.h"
#include "../mission_file_data_handler/mission_file_data_handler.h"
#include "../multi_engine_propulsion/multi_engine_propulsion.h"

class Operating_conditions;
class centerOfGravityShift;

/** \brief Class for flight conditions
 */
class Flight_conditions {
 public:
    /* Typedefs and Enums */
    enum missionMode {TAKEOFF = 1, ACCELERATE, CLIMBCONSTCAS, CLIMBTOCRUISE, CRUISE, DESCENT, APPROACH, DECELERATE, LANDING,
                      GSINTERCEPTION, FLCHANGE, FLCHANGESIMULATION,
                      CLIMBVARCAS = 30, CLIMBTOCRUISECI, CRUISECI, DESCENTCI,
                      CLIMBTOSERVICECEILING = 50, AIRTOAIRFUELLING = 98, FREEFLIGHT
                     };

    /* Objects */
    std::vector<mission_file_data_handler::mission_data_set>* mission_data; /**< reference to the mission data from mission XML */
    const Operating_conditions* operating_condition; /**< Constant pointer to an operating condition */

    atmosphere* atm;    /* pointer to given atmosphere */
    Multi_engine_propulsion* engine;     /* pointer to given engine */
    aerodynamics* aero; /* pointer to given aerodynamics object */

    /* Variables */
    double thrust;               /**< Thrust at the beginning of the segment [N] */
    double end_thrust;           /**< Thrust at the end of the segment [N] */
    double aircraft_mass;        /**< Aircraft mass at the beginning of the segment [kg] */
    double end_aircraft_mass;    /**< Aircraft mass at the end of the segment [kg] */
    double fuel_mass_flow;       /**< Fuel mass flow [kg/s] */
    double mean_fuel_mass_flow; /**< Fuel mass flow at the beginning of the segment [kg/s] */
    double end_fuel_mass_flow;   /**< Fuel mass flow at the end of the segment [kg/s] */
    double step_fuel_mass;       /**< Used fuel mass in the current step [kg] */
    std::pair<int, double> energyflow;       /**< Fuel mass flow [kg/s] */
    std::pair<int, double> start_energyflow; /**< Fuel mass flow at the beginning of the segment [J/s] */
    std::pair<int, double> end_energyflow;   /**< Fuel mass flow at the end of the segment [J/s] */
    std::pair<int, double> step_energy;      /**< Used energy amount in the current step for each energy carrier id int [J] */
    double CL;                   /**< Lift coefficient at segment start [-] */
    double drag;                 /**< drag [N] */
    double end_drag;             /**< drag at the end of the segment [N] */
    double reynolds;             /**< Reynolds number at MAC [-] */
    double end_reynolds;         /**< Reynolds number at MAC at the end of the segment [-] */
    double spoiler_factor;       /**< Spoiler factor [-] */
    double end_spoiler_factor;   /**< Spoiler factor at the end of the segment [-] */
    double TAS;                  /**< True airspeed [m/s] */
    double end_TAS;              /**< True airspeed at the end of the segment [m/s] */

    double altitude;     /**< Altitude at the beginning of the segment [m] */
    double end_altitude; /**< Altitude at the end of the segment [m] */
    double delta_h;      /**< Height change in the segment [m] */

    double delta_x; /**< Route change in the segment [m] */

    double delta_t; /**< Time change in the segment [s] */

    double ROC; /**< Rate of climb [m/s] */

    double glidepath_angle;      /**< Path angle at the beginning of the segment [rad] */
    double mean_glidepath_angle; /**< Mean path angle of the segment [rad] */
    double end_glidepath_angle;  /**< Path angle at the end of the segment [rad] */

    double acceleration;      /**< Acceleration at the beginning of the segment [m/s^2] */
    double mean_acceleration; /**< Mean acceleration of the segment [m/s^2] */
    double end_acceleration;  /**< Acceleration at the end of the segment [m/s^2] */

    double end_angle_of_attack;    /**< Angle of attack AOA at the end of the segment [deg] */
    double end_CL;                 /**< Lift coefficient at the end of the segment [-] */
    double end_specific_air_range; /**< Specific air range at the end of the segment [m/kg] */

    double rel_engine_availability; /**< Percentage of available engines; default 1 (AEO), OEI: (numberOfEngines-1)/numberOfEngines [-] */
    double thrust_efficiency;       /**< Efficiency of available thrust; default 1 (100 percent of thrust available), Thrust reverser active: efficiency of thrust reverser [-] */
    double bleed;                   /**< Bleed air mass flow of the segment [kg/s] */
    double shaft_power;             /**< Shaft power extraction of the segment [W] */

    std::vector<std::vector<double>> origin_polar;     /** 1. pos in each vector contains wing segment ID [-], 2. pos in each vector contains deflection of the segment [deg] **/
    std::vector<std::vector<double>> end_origin_polar; /** 1. pos in each vector contains wing segment ID [-], 2. pos in each vector contains deflection of the segment [deg] **/

    double stabilizer_incidence;     /** incidence angle of stabilizer at the beginning of the segment [deg] **/
    double end_stabilizer_incidence; /** incidence angle of stabilizer at the end of the segment [deg] **/

    double new_fuel_mass;      /**< fuel_mass at the end of the segment [] */
    double new_fuel_cg;        /**< fuelCoG at the end of the segment [] */
    double prior_aircraft_cg;  /**< aircraft CoG at the end of the previous segment [] */
    double prior_fuel_cg;      /**< fuel CoG at the end of the previous segment [] */
    double prior_fuel_mass;    /**< fuel_mass at the end of the previous segment [] */
    double new_aircraft_cg;    /**< aircraft CoG at the end of the segment [] */
    bool first_run_switch;     /**< true, if setSegmentCoG is ran for the first time [] */
    bool start_cg;             /**< start CoG calculation [] */
    bool engine_limit_reached; /**< Switch if the engine upper or lower operation limit is reached [-] */

    /* Constructor and destructor */
    /** \brief Constructor for class Flight_conditions
     * \param mission_data: reference to the mission object
     * \param oc: const reference to a certain operating condition
     * \param aero: reference to an aerodnymic object
     * \param atm: reference to an atmosphere object
     * \param eng: reference to an engine object
     * \param variable_camber_segment_quantity: const reference to the number of segments with variable camber [-]
     */
    Flight_conditions(std::vector<mission_file_data_handler::mission_data_set>* mission_data, Operating_conditions* oc,
                        aerodynamics* aero, atmosphere* atm, Multi_engine_propulsion* eng, const size_t& variable_camber_segment_quantity);
    /** \brief Destructor for class Flight_conditions
     */
    virtual ~Flight_conditions();

    /* Methods */
    /** \brief Function allows to adapt the engine rating from outside the flightCondition class
     *  \param oc Operating_conditions: const reference to a certain operating condition
     */
    void update_operating_condition(const Operating_conditions& oc);
    /** \brief Function calculates delta_x and delta_t
     *  \details the equations of constant and accelerated motion are used to get way and time values for the segment
     */
    void set_way_and_time();
    /** \brief Function sets initial segment values for glidepath_angle and ROC
     */
    void set_initial_flight_track();
    /** \brief Function sets initial segment values for the acceleration factor
     *  \param braking_coefficient: const reference to the aircraft's braking coefficient [-]
     *  \param friction_coefficient: const reference to the aircraft's friction coefficient [-]
     *  \param wing_surface_area: const reference to the aircraft's wing area [m^2]
     */
    void set_initial_acceleration_factors(const double& braking_coefficient, const double& friction_coefficient, const double& wing_surface_area);
    /** \brief Function calculates the current acceleration factor for the ground run
     *  \details Returns the acceleration factor in dependence of thrust, drag, lift, weight, and the friction coefficient between tyres and pavement given in the aircraftXML
     *           according to Raymer_1992_AircraftDesign-A Conceptual Approach, p. 486 (PDF-p. 501), eq. 17.97
     *  \param thrust: const reference to current thrust [N]
     *  \param drag: const reference to current drag [N]
     *  \param lift: const reference to current lift [N]
     *  \param weight: const reference to current weight (aircraft mass * gravity) [N]
     *  \param minimum_acceleration: const reference to minimum acceleration/deceleration [m/s^2]
     *  \param friction_coefficient: const reference to the aircraft's friction coefficient [-]
     *  \return acceleration factor according to Raymer for a takeOff run [m/s^2]
     */
    double get_ground_run_acceleration(const double& thrust, const double& drag, const double& lift, const double& weight,
                                        const double& minimum_acceleration, const double& friction_coefficient);
    /** \brief Function sets values for end_acceleration and mean_acceleration
     *  \param braking_coefficient: const reference to the aircraft's braking coefficient [-]
     *  \param friction_coefficient: const reference to the aircraft's friction coefficient [-]
     *  \param wing_surface_area: const reference to the aircraft's wing area [m^2]
     */
    void set_acceleration_factors(const double& braking_coefficient, const double& friction_coefficient, const double& wing_surface_area);
    /** \brief Function sets initial segment value for thrust
     *  \details the engine is set to a rating/spool speed
     */
    void set_initial_thrust();
    /** \brief Function sets values for end_glidepath_angle, mean_glidepath_angle, and ROC
     */
    void set_flight_track();
    /** \brief Function sets a value for end_thrust
     *  \details the engine is set to a rating/spool speed
     */
    void set_thrust();
	/** \brief Function activates air spoilers in Cruise/Approach and additionally ground spoilers after touchdown
     *  \details Spoilers are activated during landing or if stationary flight is requested and delivered idle thrust exceeds requested thrust
     */
    void activate_spoilers();
    /** \brief Function sets values for fuel_mass_flow, mean_fuel_mass_flow, deltaM_Fuel, and finally updates end_aircraft_mass
     *  \param use_breguet: switch if Breguet equation is used for fuel mass calculation (true) or current engine fuel flow (false - default) [-]
     */
    void update_end_mass(const bool& use_breguet = false);
};

#endif // MISSION_ANALYSIS_SRC_LIBS_CONDITION_METHODS_Flight_conditions_H_
