/*  Copyright (C) 2012-2022 Institute of Aerospace Systems, RWTH Aachen University - All rights reserved.
    This file is part of UNICADO.

    UNICADO is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    UNICADO is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with UNICADO.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "SARPerformance.h"

using std::endl;

namespace low {

SARPerformance::SARPerformance(std::shared_ptr<aircraft> theAircraft)
    :
    theAcftPt(theAircraft) {
    //  ctor
}
SARPerformance::~SARPerformance() {
    //  dtor
}

void SARPerformance::doSARCalculation(std::shared_ptr<abstractFlightEnvelopes> theEnvelopes) {
    // mass range from MTOM -> OME
    for (int i(0); i <= static_cast<int>(theEnvelopes->myFlightEnvelope.size()) - 1; i++) {
        theSAR.push_back(dataSAR());
        theSAR.back().Mass = theEnvelopes->myFlightEnvelope.at(i).grossMass;
        double maxAlt = theEnvelopes->myFlightEnvelope.at(i).maxAlt;
        double minAlt = (0.3048 * 30000);
        for (double alt = minAlt; alt <= maxAlt; alt += 0.3048 * 1000.) {
            double minMach = findMinMach(alt, theEnvelopes, i);
            double maxMach = findMaxMach(alt, theEnvelopes, i);
            theSAR.back().tabSAR.push_back(dataSARMach());
            theSAR.back().tabSAR.back().Altitude = alt;
            double bestSARatAlt(0.0);
            double bestSARatMach(0.0);
            int stepSize(20);
            for (int j(0); j <= stepSize; j++) {
                double mach = minMach + static_cast<double>(j) / stepSize * (maxMach - minMach);
                try {
                    double tempSAR = calculateSAR(theSAR.back().Mass, alt, mach, "MaxCont", "Clean");
                    theSAR.back().tabSAR.back().Mach.push_back(mach);
                    theSAR.back().tabSAR.back().SAR.push_back(tempSAR);
                    if (tempSAR >= bestSARatAlt) {
                        bestSARatAlt = tempSAR;
                        bestSARatMach = mach;
                    }
                } catch (const char * text) {
                    myRuntimeInfo->info << "CL_max reached. Stop SAR calculation." << std::endl;
                    break;
                } catch (const int& engineStatus) {
                    myRuntimeInfo->info << "Thrust limited. Stop SAR calculation" << std::endl;
                    break;
                }
            }
            if (bestSARatAlt >= theSAR.back().bestSAR) {
                theSAR.back().bestSAR = bestSARatAlt;
                theSAR.back().bestSARAlt = alt;
                theSAR.back().bestSARMach = bestSARatMach;
            }
        }
    }
}

double SARPerformance::findMinMach(double theAlt, std::shared_ptr<abstractFlightEnvelopes> theEnvelopes, int ID) {
    int i(0);
    while (theEnvelopes->myFlightEnvelope.at(ID).vStall.at(i).height <= theAlt) {
        i++;
    }
    return (1.10 * theEnvelopes->myFlightEnvelope.at(ID).vStall.at(i).speed) / theAcftPt->atm.getSpeedOfSound(theAlt);
}

double SARPerformance::findMaxMach(double theAlt, std::shared_ptr<abstractFlightEnvelopes> theEnvelopes, int ID) {
    double tempSpeed;
    int i(0);
    if (theAlt <= theEnvelopes->myFlightEnvelope.at(ID).vMO.back().height) {
        while (theEnvelopes->myFlightEnvelope.at(ID).vMO.at(i).height <= theAlt) {
            i++;
        }
        tempSpeed = theEnvelopes->myFlightEnvelope.at(ID).vMO.at(i).speed;
    } else {
        while (theEnvelopes->myFlightEnvelope.at(ID).MMO.at(i).height <= theAlt) {
            i++;
        }
        tempSpeed = theEnvelopes->myFlightEnvelope.at(ID).MMO.at(i).speed;
    }
    return tempSpeed / theAcftPt->atm.getSpeedOfSound(theAlt);
}

double SARPerformance::calculateSAR(double mass, double alt, double mach, string rating, string configuration) {
    double SAR(0.0);
    double cruiseTAS = theAcftPt->atm.getSpeedOfSound(alt) * mach;
    double cruiseDrag = theAcftPt->aero.getCruiseDrag(mach, alt, 0., mass, configuration, theAcftPt->atm);
    theAcftPt->myEngines.calculate_N1_with_thrustlimit(theAcftPt->myEnergyProviders.energyCarrierID_cruise, alt, mach, theAcftPt->atm, 1., rating, theAcftPt->myMission.bleedOffCr,
        theAcftPt->myMission.shaftOffCr, cruiseDrag / 1000.);
    if (1000. * theAcftPt->myEngines.get_thrust_aircraft(theAcftPt->myEnergyProviders.energyCarrierID_cruise) >= 0.99 * cruiseDrag) {
        SAR = cruiseTAS / theAcftPt->myEngines.get_aircraft_fuelflow(theAcftPt->myEnergyProviders.energyCarrierID_cruise);
    } else {
        SAR = 0.0;
    }
    return SAR;
}
}  // namespace low
