/*  Copyright (C) 2012-2022 Institute of Aerospace Systems, RWTH Aachen University - All rights reserved.
    This file is part of UNICADO.

    UNICADO is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    UNICADO is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with UNICADO.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef performance_assessment_SRC_TAW_DEFAULTMETHODS_VNDIAGRAM_LOW_FIDELITY_VNDIAGRAM_H_
#define performance_assessment_SRC_TAW_DEFAULTMETHODS_VNDIAGRAM_LOW_FIDELITY_VNDIAGRAM_H_

#include <unitConversion/unitConversion.h>

#include <iostream>
#include <cmath>
#include <algorithm>
#include <vector>
#include <string>
#include <memory>

#ifdef _WIN32
    #include <windows.h>
#endif  // _WIN32
#include <utility>

#include "../../aircraft.h"
#include "../../performance_assessment_settings.h"
#include "../abstractVnDiagram.h"

class performance_assessment_settings;
class aircraft;

using std::vector;
using std::string;

namespace low {

/**\class VnDiagram class contains data of the flight envelope diagram (upper and lower part of the abscissa and for different flight phases)
*
*/
class VnDiagram : public abstractVnDiagram{
 public:


    /**\brief Constructor VnDiagram
    *\param theAircraft Address to the aircraft object
    *\param theSettings Address to the performance_assessment_settings object
    */
    VnDiagram(std::shared_ptr<aircraft> theAircraft, const std::shared_ptr<performance_assessment_settings> theSettings);

    /**\brief Function to generate the flight envelope diagram
    *
    */
    void doVnDiagram() override;

    configDiagram tempValues; /**< configDiagram object */
    // vector<configDiagram> myCleanDiagram; /**< List of data for the v-N diagram for the clean configuration */


    // To describe the leading edge from equation (Clmax*rho/2*S)/W * pow(v,2)
    double CleanCoefficientPos; /**< Coefficient describing the quadratic part of the diagram shape for the positive n values (-) */
    // To describe the negative part
    double CleanCoefficientNeg; /**< Coefficient describing the quadratic part of the diagram shape for the negative n values (-) */
    // see above
    double FlapsCoefficientTO; /**< Coefficient describing the quadratic part of the diagram shape for the take-off flaps configuration (-) */
    double FlapsCoefficientLDG; /** Coefficient describing the quadratic part of the diagram shape for the landing flaps configuration (-) */

    double upperBoundClean; /**< Maximum positive load factor value n for the clean configuration (-) */
    double lowerBoundClean; /**< Minimum negative load factor value n for the clean configuration (-) */
    double upperBoundFlaps; /**< Maximum value for the flaps configuration (-) */
    double lowerBoundFlaps; /**< Minimum value for the flaps configuration (-) */

 private:
    std::shared_ptr<aircraft> theAcftPt; /**< Pointer to the theAcft object */
    const std::shared_ptr<performance_assessment_settings> mySettingsPt; /**< Pointer to the mySettings object */

    /**\brief Function calculates the upper bound of the flight envelope for the clean configuration
    *\param Coefficient Quadratic reciprocal of the lift of speed (s^2/m^2)
    *\param v Aircraft speed (m/s)
    *\return Load factor (-)
    */
    double calc_upperBoundClean(double Coefficient, double v);

    /**\brief Function calculates the lower bound of the flight envelope for the clean configuration
    *\param Coefficient Quadratic reciprocal of the lift of speed(s^2/m^2)
    *\param v Aircraft speed (m/s)
    *\return Load factor (-)
    */
    double calc_lowerBoundClean(double Coefficient, double v);

    /**\brief Function calculates the upper bound for the flaps configuration
    *\param configuration Possible flap configuration (take-off, landing)
    *\param Coefficient Quadratic reciprocal of the lift of speed(s^2/m^2)
    *\param v Aircraft speed (m/s)
    *\return Load factor (-)
    */
    double calc_upperBoundFlaps(const string &configuration, double Coefficient, double v);

//    /**\brief Function calculates the lower bound for the flaps configuration
//    *\param onfiguration Possible flap configuration (take-off, landing)
//    *\param Coefficient Quadratic reciprocal of the lift of speed (s^2/m^2)
//    *\param v Aircraft speed (m/s)
//    *\return Load factor (-)
//    */
//    double calc_lowerBoundFlaps(const string &configuration, double Coefficient, double v);

    int step; /**< Speed increment (kts) */
};

}  // namespace low
#endif  // performance_assessment_SRC_TAW_DEFAULTMETHODS_VNDIAGRAM_LOW_FIDELITY_VNDIAGRAM_H_

