/*  Copyright (C) 2012-2022 Institute of Aerospace Systems, RWTH Aachen University - All rights reserved.
    This file is part of UNICADO.

    UNICADO is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    UNICADO is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with UNICADO.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef performance_assessment_SRC_TAW_DEFAULTMETHODS_FUELPLANNING_ABSTRACTFUELPLANNING_H_
#define performance_assessment_SRC_TAW_DEFAULTMETHODS_FUELPLANNING_ABSTRACTFUELPLANNING_H_

#include <vector>
#include <memory>

namespace taw {
class performance_assessment_factory;
}

/* Abstract class of fuel planning */
// The purpose of this class is to be inherited by all "fuelPlanning" (high/mid/.../low) methods.

class abstractFuelPlanning {
 public:
    virtual ~abstractFuelPlanning() { }
    virtual void doFuelPlanning() = 0;

    virtual void doPayloadRangeOLD() { }

    virtual void doFuelPayloadRangeOLD() { }
    virtual void payloadRangeFromAcftXML() { }

    class fuelData {
     public:
        double taxiFuelTakeoff; /**< Taxi fuel at take-off (kg) */
        double taxiFuelLanding; /**< Taxi fuel at landing (kg) */
        double missionFuel;  /**< Mission fuel (kg) */  // flight fuel: fuel consumed without taxiing!
        double contigencyFuel; /**< Contingency fuel (kg) */
        double alternateFuel; /**< Alternate fuel (kg) */
        double finalReserveFuel; /**< Final reserve fuel (kg) */
        double tripFuel; /**< Trip fuel (kg) */

        /**\brief Constructor fuelData
        *
        */
        fuelData() {
            taxiFuelTakeoff     = 0.;
            taxiFuelLanding     = 0.;
            missionFuel           = 0.;
            contigencyFuel      = 0.;
            alternateFuel       = 0.;
            finalReserveFuel    = 0.;
            tripFuel            = 0.;
        }
    };

    /**\brief Class missionFuelData contains aircraft parameters for each mission step
    *
    */
    class missionFuelData {
     public:
        double Time; /**< Time spent for the mission (min) */
        double Range; /**< Distance covered in the mission (km) */
        double Altitude; /**< Aircraft altitude during the mission (m) */
        double TAS; /**< True air speed during the mission (m/s) */
        double Mach; /**< Mach number during the mission (-) */
        double Mass; /**< Aircraft mass (kg) */
        double consumedFuel; /**< Fuel consumed during the mission (kg) */
        double Thrust; /**< Aircraft thrust (kN) */
        double fuelFlow; /**< Fuel flow (kg/s) */
        double ROC; /**< Aircraft rate of climb (m/s) */

        missionFuelData() {
            Time = 0.0;
            Range = 0.0;
            Altitude = 0.0;
            TAS = 0.0;
            Mach = 0.0;
            Mass = 0.0;
            consumedFuel = 0.0;
            Thrust = 0.0;
            fuelFlow = 0.0;
            ROC = 0.0;
        }
    };
    std::vector<missionFuelData> theMissionFuel; /**< List of the mission parameters */

    fuelData myFuel;

    /**\class payloadRangeData Class with parameters for the payload range diagram
    *
    */
    class payloadRangeData {
     public:
        double Range; /**< Distance covered in the mission based on the payload-range diagram (NM) */
        double Payload; /**< Payload carried by the aircraft (kg) */
        double Fuel; /**< Fuel carried by the aircraft (kg) */
        double TOM; /**< Aircraft take-off mass (kg) */
        payloadRangeData() {
            Range = 0.0;
            Payload = 0.0;
            Fuel = 0.0;
            TOM = 0.0;
        }
    };
    std::vector<payloadRangeData> myPayloadRange;  /**< List of values of the payload-range diagram */

    double tripTime;  /**< Trip time (min) */
    double blockTime; /**< Block time (min) */

    std::shared_ptr<taw::performance_assessment_factory> theFactory;
    void setFactory(std::shared_ptr<taw::performance_assessment_factory> aFactory) { this->theFactory = aFactory; }
};

#endif  // performance_assessment_SRC_TAW_DEFAULTMETHODS_FUELPLANNING_ABSTRACTFUELPLANNING_H_



