/*  Copyright (C) 2024 Chair of Aircraft Design, Technical University Munich
    This file is part of UNICADO.

    UNICADO is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    UNICADO is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with UNICADO.  If not, see <http://www.gnu.org/licenses/>.
*/
/**
 * @file default_integration.cpp
 * @author Sebastian Oberschwendtner (sebastian.oberschwendtner@tum.de)
 * @brief Common functionality of the default integration strategy for propulsion systems.
 * @version 3.0.0
 * @date 2024-06-28
 */

/* === Includes === */
#include "default_integration.h"
#include <algorithm>
#include <aircraftGeometry2/geometry/entity3d.h>

/* === Design implementations === */
namespace design
{
    namespace integration
    {
        auto Default::select_fuselage() -> geometry::Fuselage&
        {
            /** @todo Make it possible to select a fuselage when there are more than one. */
            if (this->aircraft()->fuselages.size() != 1)
            {
                /** @todo Should we throw here or just issue a warning and select the first available fuselage? */
                throw std::runtime_error("[design::integrator::Default::select_fuselage()] The aircraft does not contain exactly one fuselage.");
            }
            return this->aircraft()->fuselages[0];
        }

        auto Default::select_vertical_tail() -> geometry::Wing&
        {
            /* Find the empennage surface which is vertical */
            auto empennage = std::ranges::find_if(this->aircraft()->empennage, geometry::is_vertical);
            if (empennage == this->aircraft()->empennage.end())
            {
                throw std::runtime_error("[design::integrator::Default::select_vertical_tail()] The aircraft does not contain a vertical tail.");
            }

            /* Return the vertical tail */
            return *empennage;
        }

        auto Default::select_wing() -> geometry::Wing&
        {
            /** @todo Make it possible to select a wing when there are more than one. */
            if (this->aircraft()->wings.size() != 1)
            {
                throw std::runtime_error("[design::integrator::Default::select_wing()] The aircraft does not contain exactly one wing.");
            }
            return this->aircraft()->wings[0];
        }
    }; // namespace integration
}; // namespace design
