/*  Copyright (C) 2023 Chair of Aircraft Design, Technical University Munich
    This file is part of UNICADO.

    UNICADO is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    UNICADO is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with UNICADO.  If not, see <http://www.gnu.org/licenses/>.
*/
/**
 * @file test_default_turbofan.cpp
 * @author Sebastian Oberschwendtner (sebastian.oberschwendtner@tum.de)
 * @brief Test the default propulsion integrator for a turbofan engine.
 * @version 3.0.0
 * @date 2023-12-20
 */

/* === Includes === */
#include <filesystem>
#include <gtest/gtest.h>
#include <aircraftGeometry2/geometry/builder.h>

/* Unit Under Test */
#include "integration/default/default_integration.h"

/* === Fixtures === */
class IntegrationDefault : public ::testing::Test
{
  protected:
    void SetUp() override
    {
        /* Setup the aircraft geometry */
        geom2::Polygon_2 poly = geom2::build::ellipse(1.0, 1.0).get_contour(false);
        geom2::Polygon_2 airfoil;
        airfoil.push_back({0, 0});
        airfoil.push_back({0.5, 0.2});
        airfoil.push_back({1.0, 0});
        airfoil.push_back({0.0, 0.0});

        /* Nacelle */
        nacelle.sections.emplace_back(poly);
        nacelle.sections.emplace_back(poly);
        nacelle.sections.back().origin = geom2::Point_3(0, 0, 2.0);
        nacelle.normal = geom2::Direction_3(1, 0, 0);

        /* Main Wing */
        wing.sections.emplace_back(airfoil);
        wing.sections.emplace_back(airfoil);
        wing.sections.at(0).set_chord_length(2.0);
        wing.sections.at(1).set_chord_length(1.0);
        wing.sections.at(1).origin = geom2::Point_3(0, 0, -5.0);
        wing.origin = geom2::Point_3(3, 0, 2);
        wing.normal = geom2::Direction_3(0, -1, 0);

        /* Vertical Tail */
        vtp.sections.emplace_back(airfoil);
        vtp.sections.emplace_back(airfoil);
        vtp.sections.at(0).set_chord_length(1.0);
        vtp.sections.at(1).set_chord_length(0.5);
        vtp.sections.at(1).origin = geom2::Point_3(1.0, 0, -3.0);
        vtp.origin = geom2::Point_3(1.0, 0, 0.5);
        vtp.normal = geom2::Direction_3(0, 0, -1);
        vtp.is_symmetric = false;

        /* Assemble the aircraft */
        aircraft->wings.emplace_back(wing);
        aircraft->fuselages.emplace_back(nacelle);
        aircraft->empennage.emplace_back(vtp);
    }
    std::shared_ptr<node> config = aixml::openDocument(std::filesystem::path{CMAKE_TEST_CONFIG_FILE});
    geom2::MultisectionSurface<geom2::PolygonSection> nacelle;
    geom2::MultisectionSurface<geom2::AirfoilSection> wing;
    geom2::MultisectionSurface<geom2::AirfoilSection> vtp;
    std::shared_ptr<geometry::Aircraft> aircraft = std::make_shared<geometry::Aircraft>();
};

/* === Tests === */
/**
 * @brief Test integrating a turbofan engine into a wing with two engines under the wing.
 */
TEST_F(IntegrationDefault, IntegrateTurbofanKeroseneTwoEnginesUnderWing)
{
    /* Create the engine designer */
    design::integration::Default integrator{config, aircraft, {0, 2, 0}};

    /* Setup right engine */
    Turbofan<EnergyCarrier::Kerosene> engine_right{0, {Parent::Wing, Lateral::Right, Longitudinal::Front, Vertical::Under}};
    engine_right.set_dimension({0.8, 0.8, 1.5});
    engine_right.set_fan({0.9});
    engine_right.set_nacelle(nacelle);

    /* Setup left engine */
    Turbofan<EnergyCarrier::Kerosene> engine_left{1, {Parent::Wing, Lateral::Left, Longitudinal::Front, Vertical::Under}};
    engine_left.set_dimension({0.8, 0.8, 1.5});
    engine_left.set_fan({0.9});
    engine_left.set_nacelle(nacelle);

    /* Test invalid engine setups */
    Turbofan<EnergyCarrier::Kerosene> engine_invalid{2, {Parent::Wing, Lateral::Left, Longitudinal::Front, Vertical::Over}};
    EXPECT_THROW(integrator(engine_invalid), std::runtime_error);

    /* Design the first engine */
    integrator(engine_right);
    EXPECT_NEAR(engine_right.position().x(), wing.origin.x() - 1.19678, 1e-3);
    EXPECT_NEAR(engine_right.position().y(), wing.origin.y() + 2.33719, 1e-3);
    EXPECT_NEAR(engine_right.position().z(), wing.origin.z() - 0.7, 1e-3);

    /* Integrating the first engine again should throw */
    EXPECT_THROW(integrator(engine_right), std::runtime_error);

    /* Design the second engine */
    integrator(engine_left);
    EXPECT_NEAR(engine_left.position().x(), wing.origin.x() - 1.19678, 1e-3);
    EXPECT_NEAR(engine_left.position().y(), wing.origin.y() - 2.33719, 1e-3);
    EXPECT_NEAR(engine_left.position().z(), wing.origin.z() - 0.7, 1e-3);

    /* Integrating the second engine again should throw */
    EXPECT_THROW(integrator(engine_left), std::runtime_error);
}

/**
 * @brief Test integrating a turbofan engine into a wing with four engines under the wing.
 */
TEST_F(IntegrationDefault, IntegrateTurbofanKeroseneFourEnginesUnderWing)
{
    /* Create the engine designer */
    design::integration::Default integrator{config, aircraft, {0, 4, 0}};

    /* Setup right engine */
    Turbofan<EnergyCarrier::Kerosene> engine_inner_right{0, {Parent::Wing, Lateral::Right, Longitudinal::Front, Vertical::Under}};
    engine_inner_right.set_dimension({0.8, 0.8, 1.5});
    engine_inner_right.set_fan({0.9});
    engine_inner_right.set_nacelle(nacelle);

    /* Setup left engine */
    Turbofan<EnergyCarrier::Kerosene> engine_inner_left{1, {Parent::Wing, Lateral::Left, Longitudinal::Front, Vertical::Under}};
    engine_inner_left.set_dimension({0.8, 0.8, 1.5});
    engine_inner_left.set_fan({0.9});
    engine_inner_left.set_nacelle(nacelle);

    /* Test invalid engine setups */
    Turbofan<EnergyCarrier::Kerosene> engine_invalid{2, {Parent::Wing, Lateral::Left, Longitudinal::Front, Vertical::Over}};
    EXPECT_THROW(integrator(engine_invalid), std::runtime_error);

    /* Design the first right engine */
    integrator(engine_inner_right);
    EXPECT_NEAR(engine_inner_right.position().x(), wing.origin.x() - 1.089, 1e-3);
    EXPECT_NEAR(engine_inner_right.position().y(), wing.origin.y() + 0.699975, 1e-3);
    EXPECT_NEAR(engine_inner_right.position().z(), wing.origin.z() - 0.7, 1e-3);

    /* Design the first engine again which should result in the outer engine now */
    integrator(engine_inner_right);
    EXPECT_NEAR(engine_inner_right.position().x(), wing.origin.x() - 1.107, 1e-3);
    EXPECT_NEAR(engine_inner_right.position().y(), wing.origin.y() + 1.600, 1e-3);
    EXPECT_NEAR(engine_inner_right.position().z(), wing.origin.z() - 0.7, 1e-3);

    /* Designing another engine should throw */
    EXPECT_THROW(integrator(engine_inner_right), std::runtime_error);

    /* Design the first left engine */
    integrator(engine_inner_left);
    EXPECT_NEAR(engine_inner_left.position().x(), wing.origin.x() - 1.089, 1e-3);
    EXPECT_NEAR(engine_inner_left.position().y(), wing.origin.y() - 0.699975, 1e-3);
    EXPECT_NEAR(engine_inner_left.position().z(), wing.origin.z() - 0.7, 1e-3);

    /* Design the first engine again which should result in the outer engine now */
    integrator(engine_inner_left);
    EXPECT_NEAR(engine_inner_left.position().x(), wing.origin.x() - 1.107, 1e-3);
    EXPECT_NEAR(engine_inner_left.position().y(), wing.origin.y() - 1.600, 1e-3);
    EXPECT_NEAR(engine_inner_left.position().z(), wing.origin.z() - 0.7, 1e-3);

    /* Designing another engine should throw */
    EXPECT_THROW(integrator(engine_inner_left), std::runtime_error);
}

/**
 * @brief Test integrating a turbofan into a fuselage with two engines in the rear.
 */
TEST_F(IntegrationDefault, IntegrateTurbofanKeroseneTwoEnginesMidFuselage)
{
    /* Create the engine designer */
    design::integration::Default integrator{config, aircraft, {2, 0, 0}};

    /* Setup right engine */
    Turbofan<EnergyCarrier::Kerosene> engine_right{0, {Parent::Fuselage, Lateral::Right, Longitudinal::Rear, Vertical::Mid}};
    engine_right.set_dimension({0.8, 0.8, 1.5});
    engine_right.set_fan({0.9});
    engine_right.set_nacelle(nacelle);

    /* Setup left engine */
    Turbofan<EnergyCarrier::Kerosene> engine_left{1, {Parent::Fuselage, Lateral::Left, Longitudinal::Rear, Vertical::Mid}};
    engine_left.set_dimension({0.8, 0.8, 1.5});
    engine_left.set_fan({0.9});
    engine_left.set_nacelle(nacelle);

    /* Test invalid engine setups */
    Turbofan<EnergyCarrier::Kerosene> engine_invalid{2, {Parent::Fuselage, Lateral::Left, Longitudinal::Front, Vertical::In}};
    EXPECT_THROW(integrator(engine_invalid), std::runtime_error);

    /* Design the first engine */
    integrator(engine_right);
    EXPECT_NEAR(engine_right.position().x(), 1.600, 1e-3);
    EXPECT_NEAR(engine_right.position().y(), 1.125, 1e-3);
    EXPECT_NEAR(engine_right.position().z(), 0.250, 1e-3);

    /* Integrating the first engine again should throw */
    EXPECT_THROW(integrator(engine_right), std::runtime_error);

    /* Design the second engine */
    integrator(engine_left);
    EXPECT_NEAR(engine_left.position().x(), 1.600, 1e-3);
    EXPECT_NEAR(engine_left.position().y(), -1.125, 1e-3);
    EXPECT_NEAR(engine_left.position().z(), 0.250, 1e-3);

    /* Integrating the second engine again should throw */
    EXPECT_THROW(integrator(engine_left), std::runtime_error);
}

/**
 * @brief Test integrating a turbofan into the bottom of a vertical tail.
 */
TEST_F(IntegrationDefault, IntegrateTurbofanKeroseneOneEngineInVerticalTail)
{
    /* Create the engine designer */
    design::integration::Default integrator{config, aircraft, {0, 0, 1}};

    /* Setup engine */
    Turbofan<EnergyCarrier::Kerosene> engine{0, {Parent::Empennage, Lateral::Mid, Longitudinal::Front, Vertical::In}};
    engine.set_dimension({0.8, 0.8, 1.5});
    engine.set_fan({0.9});
    engine.set_nacelle(nacelle);

    /* Design the engine */
    integrator(engine);
    EXPECT_NEAR(engine.position().x(), 1.264, 1e-3);
    EXPECT_NEAR(engine.position().y(), 0.0, 1e-3);
    EXPECT_NEAR(engine.position().z(), 1.0, 1e-3);

    /* Integrating the engine again should throw */
    EXPECT_THROW(integrator(engine), std::runtime_error);
}

/**
 * @brief Test integrating a turbofan into an unsupported parent component.
 */
TEST_F(IntegrationDefault, IntegrateTurbofanKeroseneUnsupported)
{
    /* Create the engine designer */
    design::integration::Default integrator{config, aircraft, {0, 0, 0}};

    /* Setup engine */
    Turbofan<EnergyCarrier::Kerosene> engine{0, {Parent::Fuselage, Lateral::Mid, Longitudinal::Rear, Vertical::Over}};
    engine.set_dimension({0.8, 0.8, 1.5});
    engine.set_fan({0.9});
    engine.set_nacelle(nacelle);

    /* Test the unsupported parent component */
    EXPECT_THROW(integrator(engine), std::runtime_error);
    engine = Turbofan<EnergyCarrier::Kerosene>{0, {Parent::Fuselage, Lateral::Mid, Longitudinal::Rear, Vertical::Mid}};
    EXPECT_THROW(integrator(engine), std::runtime_error);
}

/**
 * @brief Test integrating engines when no aircraft geometry is provided.
 */
TEST_F(IntegrationDefault, IntegrateTurbofanKeroseneNoAircraftGeometry)
{
    /* Create the engine designer */
    design::integration::Default integrator{config, nullptr, {0, 0, 0}};

    /* Setup engine */
    Turbofan<EnergyCarrier::Kerosene> engine0{0, {Parent::Wing, Lateral::Right, Longitudinal::Front, Vertical::Under}};
    engine0.set_dimension({0.8, 0.8, 1.5});
    engine0.set_fan({0.9});
    engine0.set_nacelle(nacelle);
    Turbofan<EnergyCarrier::Kerosene> engine1{0, {Parent::Wing, Lateral::Left, Longitudinal::Front, Vertical::Under}};
    engine1.set_dimension({0.8, 0.8, 1.5});
    engine1.set_fan({0.9});
    engine1.set_nacelle(nacelle);

    /* Test integrating the engine at default positions */
    EXPECT_NO_THROW(integrator(engine0));
    EXPECT_NEAR(engine0.position().x(), 0.0, 1e-3);
    EXPECT_NEAR(engine0.position().y(), 0.0, 1e-3);
    EXPECT_NEAR(engine0.position().z(), 0.0, 1e-3);

    /* Test integrating the engine at default positions */
    EXPECT_NO_THROW(integrator(engine1));
    EXPECT_NEAR(engine1.position().x(), 0.0, 1e-3);
    EXPECT_NEAR(engine1.position().y(), 0.0, 1e-3);
    EXPECT_NEAR(engine1.position().z(), 0.0, 1e-3);

}

/**
 * @brief Test integrating engines with invalid aircraft geometry.
 */
TEST_F(IntegrationDefault, IntegrateTurbofanKeroseneInvalidAircraftGeometry)
{
    /* Create the engine designer */
    design::integration::Default integrator{config, aircraft, {2, 2, 1}};

    /* Setup engine */
    Turbofan<EnergyCarrier::Kerosene> engine{0, {Parent::Empennage, Lateral::Mid, Longitudinal::Front, Vertical::In}};
    engine.set_dimension({0.8, 0.8, 1.5});
    engine.set_fan({0.9});
    engine.set_nacelle(nacelle);

    /* Test integrating when there is no vertical tail */
    aircraft->empennage.at(0).normal = geom2::Direction_3{0, 1, 0};
    EXPECT_THROW(integrator(engine), std::runtime_error);
    
    /* Test integrating when there are multiple wings */
    engine.set_parent({Parent::Wing, Lateral::Right, Longitudinal::Front, Vertical::Under});
    aircraft->wings.emplace_back(aircraft->wings.at(0));
    EXPECT_THROW(integrator(engine), std::runtime_error);
    
    /* Test integrating when there are multiple fuselages */
    engine.set_parent({Parent::Fuselage, Lateral::Right, Longitudinal::Rear, Vertical::Mid});
    aircraft->fuselages.emplace_back(aircraft->fuselages.at(0));
    EXPECT_THROW(integrator(engine), std::runtime_error);
}
