"""Module providing functions for data postprocessing."""
# Import standard modules.

# Import own modules.
from datapostprocessingmodule import method_data_postprocessing
from datapostprocessingmodule import write_key_data_to_aircraft_exchange_file
from datapostprocessingmodule import prepare_element_tree_for_module_key_parameter


def data_postprocessing(paths_and_names, routing_dict, data_dict, runtime_output):
    """Perform data postprocessing and write data to an aircraft exchange file.

    This function is responsible for data postprocessing that involves the following steps:
        (1) Data preparation: The module manager prepares a list containing all paths to the key parameters that must
        be written to the aircraft exchange file.
        (2) Write data to the aircraft exchange file: The results of the module execution are passed on to the function
        responsible for properly writing the data to the aircraft exchange file.
        (3) Method-specific data postprocessing: User-defined method-specific postprocessing is conducted as needed.

    :param dict paths_and_names: Dictionary containing system paths and ElementTrees
    :param dict routing_dict: Dictionary containing routing parameters
    :param dict data_dict: Dictionary containing the result of the module execution (direct operating cost estimation)
    :param logging.Logger runtime_output: Logging object used for capturing log messages in the module
    :return: None
    """

    """Data preparation."""
    # Changes to this list are the sole responsibility of the module manager!
    paths_to_key_parameters_list = [
        './component_design/tank/position/x',
        './component_design/tank/position/y',
        './component_design/tank/position/z',
        './component_design/tank/mass_properties/mass',
        './component_design/tank/mass_properties/inertia/j_xx',
        './component_design/tank/mass_properties/inertia/j_yy',
        './component_design/tank/mass_properties/inertia/j_zz',
        './component_design/tank/mass_properties/inertia/j_xy',
        './component_design/tank/mass_properties/inertia/j_xz',
        './component_design/tank/mass_properties/inertia/j_yx',
        './component_design/tank/mass_properties/inertia/j_yz',
        './component_design/tank/mass_properties/inertia/j_zx',
        './component_design/tank/mass_properties/inertia/j_zy',
        './component_design/tank/mass_properties/center_of_gravity/x',
        './component_design/tank/mass_properties/center_of_gravity/y',
        './component_design/tank/mass_properties/center_of_gravity/z',
        './component_design/tank/specific/additional_fuselage_length',
        './component_design/tank/specific/tank[@ID="0"]/name',
        './component_design/tank/specific/tank[@ID="0"]/designator',
        './component_design/tank/specific/tank[@ID="0"]/position/x',
        './component_design/tank/specific/tank[@ID="0"]/position/y',
        './component_design/tank/specific/tank[@ID="0"]/position/z',
        './component_design/tank/specific/tank[@ID="0"]/direction/x',
        './component_design/tank/specific/tank[@ID="0"]/direction/y',
        './component_design/tank/specific/tank[@ID="0"]/direction/z',
        './component_design/tank/specific/tank[@ID="0"]/mass_properties/mass',
        './component_design/tank/specific/tank[@ID="0"]/mass_properties/inertia/j_xx',
        './component_design/tank/specific/tank[@ID="0"]/mass_properties/inertia/j_yy',
        './component_design/tank/specific/tank[@ID="0"]/mass_properties/inertia/j_zz',
        './component_design/tank/specific/tank[@ID="0"]/mass_properties/inertia/j_xy',
        './component_design/tank/specific/tank[@ID="0"]/mass_properties/inertia/j_xz',
        './component_design/tank/specific/tank[@ID="0"]/mass_properties/inertia/j_yx',
        './component_design/tank/specific/tank[@ID="0"]/mass_properties/inertia/j_yz',
        './component_design/tank/specific/tank[@ID="0"]/mass_properties/inertia/j_zx',
        './component_design/tank/specific/tank[@ID="0"]/mass_properties/inertia/j_zy',
        './component_design/tank/specific/tank[@ID="0"]/mass_properties/center_of_gravity/x',
        './component_design/tank/specific/tank[@ID="0"]/mass_properties/center_of_gravity/y',
        './component_design/tank/specific/tank[@ID="0"]/mass_properties/center_of_gravity/z',
        './component_design/tank/specific/tank[@ID="0"]/maximum_energy_capacity',
        './component_design/tank/specific/tank[@ID="0"]/energy_capacity_required_for_mission',
        './component_design/tank/specific/tank[@ID="0"]/mass_properties/centroid/x',
        './component_design/tank/specific/tank[@ID="0"]/mass_properties/centroid/y',
        './component_design/tank/specific/tank[@ID="0"]/mass_properties/centroid/z',
        './component_design/tank/specific/tank[@ID="0"]/geometry/cross_section[@ID="0"]/name',
        './component_design/tank/specific/tank[@ID="0"]/geometry/cross_section[@ID="0"]/position/x',
        './component_design/tank/specific/tank[@ID="0"]/geometry/cross_section[@ID="0"]/position/y',
        './component_design/tank/specific/tank[@ID="0"]/geometry/cross_section[@ID="0"]/position/z',
        './component_design/tank/specific/tank[@ID="0"]/geometry/cross_section[@ID="0"]/shape',
        './component_design/tank/specific/tank[@ID="0"]/geometry/cross_section[@ID="0"]/height',
        './component_design/tank/specific/tank[@ID="0"]/geometry/cross_section[@ID="0"]/width',
        './component_design/tank/specific/tank[@ID="0"]/geometry/cross_section[@ID="0"]/length'
    ]
    module_key_parameters_dict = {
        'component_design': {
            'tank': {
                'attributes': {
                    'description': 'Description of aircraft tanks.',
                    'tool_level': '0'},
                'position': {
                    'attributes': {'description':
                                   'Tank reference point, position of the tanks with regard to the global reference '
                                   'point.'},
                    'x': {
                        'attributes': {'description': 'Distance between the foremost tank end and the global '
                                       'reference point in x-direction.'},
                        'value': '0',
                        'unit': 'm',
                        'lower_boundary': '-80',
                        'upper_boundary': '80'},
                    'y': {
                        'attributes': {'description': 'Distance between the foremost tank end and the global '
                                       'reference point in y-direction.'},
                        'value': '0',
                        'unit': 'm',
                        'lower_boundary': '-40',
                        'upper_boundary': '40'},
                    'z': {
                        'attributes': {'description': 'Distance between the foremost middle tank end and the global '
                                       'reference point in z-direction.'},
                        'value': '0',
                        'unit': 'm',
                        'lower_boundary': '-5',
                        'upper_boundary': '5'},
                },
                'mass_properties': {
                    'attributes': {
                        'description': 'Mass properties of all tanks.'},
                    'mass': {
                        'attributes': {'description': 'Total tank mass.'},
                        'value': '0',
                        'unit': 'kg',
                        'lower_boundary': '0',
                        'upper_boundary': 'inf'},
                    'inertia': {
                        'attributes': {
                            'description': 'Inertia of all tanks with regard to the total center of gravity.'},
                        'j_xx': {
                            'attributes': {
                                'description': 'Inertia of all tanks in x.'},
                            'value': '0',
                            'unit': 'kgm^2',
                            'lower_boundary': '-inf',
                            'upper_boundary': 'inf'},
                        'j_yy': {
                            'attributes': {
                                'description': 'Inertia of all tanks in y.'},
                            'value': '0',
                            'unit': 'kgm^2',
                            'lower_boundary': '-inf',
                            'upper_boundary': 'inf'},
                        'j_zz': {
                            'attributes': {
                                'description': 'Inertia of all tanks in z.'},
                            'value': '0',
                            'unit': 'kgm^2',
                            'lower_boundary': '-inf',
                            'upper_boundary': 'inf'},
                        'j_xy': {
                            'attributes': {
                                'description': 'Inertia of all tanks in xy.'},
                            'value': '0',
                            'unit': 'kgm^2',
                            'lower_boundary': '-inf',
                            'upper_boundary': 'inf'},
                        'j_xz': {
                            'attributes': {
                                'description': 'Inertia of all tanks in xz.'},
                            'value': '0',
                            'unit': 'kgm^2',
                            'lower_boundary': '-inf',
                            'upper_boundary': 'inf'},
                        'j_yx': {
                            'attributes': {
                                'description': 'Inertia of all tanks in yx.'},
                            'value': '0',
                            'unit': 'kgm^2',
                            'lower_boundary': '-inf',
                            'upper_boundary': 'inf'},
                        'j_yz': {
                            'attributes': {
                                'description': 'Inertia of all tanks in yz.'},
                            'value': '0',
                            'unit': 'kgm^2',
                            'lower_boundary': '-inf',
                            'upper_boundary': 'inf'},
                        'j_zx': {
                            'attributes': {
                                'description': 'Inertia of all tanks in zx.'},
                            'value': '0',
                            'unit': 'kgm^2',
                            'lower_boundary': '-inf',
                            'upper_boundary': 'inf'},
                        'j_zy': {
                            'attributes': {
                                'description': 'Inertia of all tanks in zy.'},
                            'value': '0',
                            'unit': 'kgm^2',
                            'lower_boundary': '-inf',
                            'upper_boundary': 'inf'},
                    },
                    'center_of_gravity': {
                        'attributes': {
                            'description': 'Center of gravity of all tanks.'},
                        'x': {
                            'attributes': {
                                'description': 'Center of gravity in x-direction with regard to the global reference '
                                               'point.'},
                            'value': '0',
                            'unit': 'm',
                            'lower_boundary': '-80',
                            'upper_boundary': '80'},
                        'y': {
                            'attributes': {
                                'description': 'Center of gravity in y-direction with regard to the global reference '
                                               'point.'},
                            'value': '0',
                            'unit': 'm',
                            'lower_boundary': '-40',
                            'upper_boundary': '40'},
                        'z': {
                            'attributes': {
                                'description': 'Center of gravity in z-direction with regard to the global reference '
                                               'point.'},
                            'value': '0',
                            'unit': 'm',
                            'lower_boundary': '-5',
                            'upper_boundary': '5'}
                    }
                },
                'specific': {
                    'attributes': {
                        'description': 'Tank design specific output parameter.'},
                    'tank': {
                        'attributes': {
                            'ID': '0',
                            'description': 'Description of one tank.'},
                        'name': {
                            'attributes': {
                                'description': 'Name of the tank.'},
                                'value': 'tank_0'},
                        'designator': {
                            'attributes': {
                                'description': 'Designator of the tank.'},
                                'value': 'placeholder_designator'},
                        'position': {
                            'attributes': {
                                'description': 'Local reference point, position of one tank wrt. the tank reference point.'},
                            'x': {
                                'attributes': {
                                    'description': 'Distance between the foremost tank end of one tank and the tank '
                                    'reference point in x-direction.'},
                                'value': '0',
                                'unit': 'm',
                                'lower_boundary': '-80',
                                'upper_boundary': '80'},
                            'y': {
                                'attributes': {
                                    'description': 'Distance between the foremost tank end of one tank and the tank '
                                    'reference point in y-direction.'},
                                'value': '0',
                                'unit': 'm',
                                'lower_boundary': '-40',
                                'upper_boundary': '40'},
                            'z': {
                                'attributes': {
                                    'description': 'Distance between the foremost tank end of one tank and the tank '
                                    'reference point in z-direction.'},
                                'value': '0',
                                'unit': 'm',
                                'lower_boundary': '-5',
                                'upper_boundary': '5'}
                            },
                        'direction': {
                            'attributes': {
                                'description': 'Unit vector according to global coordinate system for direction '
                                'applied at position.'},
                            'x': {
                                'attributes': {
                                    'description': 'X component of the unit vector.'},
                                'value': '0',
                                'unit': 'm',
                                'lower_boundary': '0',
                                'upper_boundary': '1'},
                            'y': {
                                'attributes': {
                                    'description': 'Y component of the unit vector.'},
                                'value': '0',
                                'unit': 'm',
                                'lower_boundary': '0',
                                'upper_boundary': '1'},
                            'z': {
                                'attributes': {
                                    'description': 'Z component of the unit vector.'},
                                'value': '0',
                                'unit': 'm',
                                'lower_boundary': '0',
                                'upper_boundary': '1'}
                            },
                        'mass_properties': {
                            'attributes': {
                                'description': 'Mass properties of one tank.'},
                            'mass': {
                                'attributes': {'description': 'Total dry mass of one tank.'},
                                'value': '0',
                                'unit': 'kg',
                                'lower_boundary': '0',
                                'upper_boundary': 'inf'},
                            'inertia': {
                                'attributes': {
                                    'description': 'Inertia of one tank with regard to its center of gravity.'},
                                    'j_xx': {
                                        'attributes': {
                                            'description': 'Inertia of the of one tank in x.'},
                                        'value': '0',
                                        'unit': 'kgm^2',
                                        'lower_boundary': '-inf',
                                        'upper_boundary': 'inf'},
                                    'j_yy': {
                                        'attributes': {
                                            'description': 'Inertia of the of one tank in y.'},
                                        'value': '0',
                                        'unit': 'kgm^2',
                                        'lower_boundary': '-inf',
                                        'upper_boundary': 'inf'},
                                    'j_zz': {
                                        'attributes': {
                                            'description': 'Inertia of the of one tank in z.'},
                                        'value': '0',
                                        'unit': 'kgm^2',
                                        'lower_boundary': '-inf',
                                        'upper_boundary': 'inf'},
                                    'j_xy': {
                                        'attributes': {
                                            'description': 'Inertia of one tank in xy.'},
                                        'value': '0',
                                        'unit': 'kgm^2',
                                        'lower_boundary': '-inf',
                                        'upper_boundary': 'inf'},
                                    'j_xz': {
                                        'attributes': {
                                            'description': 'Inertia of one tank in xz.'},
                                        'value': '0',
                                        'unit': 'kgm^2',
                                        'lower_boundary': '-inf',
                                        'upper_boundary': 'inf'},
                                    'j_yx': {
                                        'attributes': {
                                            'description': 'Inertia of one tank in yx.'},
                                        'value': '0',
                                        'unit': 'kgm^2',
                                        'lower_boundary': '-inf',
                                        'upper_boundary': 'inf'},
                                    'j_yz': {
                                        'attributes': {
                                            'description': 'Inertia of one tank in yz.'},
                                        'value': '0',
                                        'unit': 'kgm^2',
                                        'lower_boundary': '-inf',
                                        'upper_boundary': 'inf'},
                                    'j_zx': {
                                        'attributes': {
                                            'description': 'Inertia of one tank in zx.'},
                                        'value': '0',
                                        'unit': 'kgm^2',
                                        'lower_boundary': '-inf',
                                        'upper_boundary': 'inf'},
                                    'j_zy': {
                                        'attributes': {
                                            'description': 'Inertia of one tank in zy.'},
                                        'value': '0',
                                        'unit': 'kgm^2',
                                        'lower_boundary': '-inf',
                                        'upper_boundary': 'inf'}},
                            'center_of_gravity': {
                                'attributes': {'description': 'Center of gravity of one tank.'},
                                'x': {
                                    'attributes': {
                                        'description': 'Center of gravity in x-direction with regard to the global '
                                        'reference point.'},
                                    'value': '0',
                                    'unit': 'm',
                                    'lower_boundary': '-80',
                                    'upper_boundary': '80'},
                                'y': {
                                    'attributes': {
                                        'description': 'Center of gravity in y-direction with regard to the global '
                                        'reference point.'},
                                    'value': '0',
                                    'unit': 'm',
                                    'lower_boundary': '-40',
                                    'upper_boundary': '40'},
                                'z': {
                                    'attributes': {
                                        'description': 'Center of gravity in z-direction with regard to the global '
                                        'reference point.'},
                                    'value': '0',
                                    'unit': 'm',
                                    'lower_boundary': '-5',
                                    'upper_boundary': '5'}
                            },
                            'centroid': {
                                'attributes': {'description': 'Centroid of one tank.'},
                                'x': {
                                    'attributes': {
                                        'description': 'Centroid in x-direction with regard to the local '
                                        'reference point.'},
                                    'value': '0',
                                    'unit': 'm',
                                    'lower_boundary': '-80',
                                    'upper_boundary': '80'},
                                'y': {
                                    'attributes': {
                                        'description': 'Centroid in y-direction with regard to the local '
                                        'reference point.'},
                                    'value': '0',
                                    'unit': 'm',
                                    'lower_boundary': '-40',
                                    'upper_boundary': '40'},
                                'z': {
                                    'attributes': {
                                        'description': 'Centroid in z-direction with regard to the local '
                                        'reference point.'},
                                    'value': '0',
                                    'unit': 'm',
                                    'lower_boundary': '-5',
                                    'upper_boundary': '5'}
                            }
                        },
                        'maximum_energy_capacity': {
                            'attributes': {'description': 'Maximum energy capacity of one tank.'},
                            'value': '0',
                            'unit': 'J',
                            'lower_boundary': '0',
                            'upper_boundary': 'inf'},
                        'energy_capacity_required_for_mission': {
                            'attributes': {'description': 'Energy amount required for mission.'},
                            'value': 'True'},
                        'geometry': {
                            'attributes': {'description': 'Geometrical description of one tank.'},
                            'cross_section': {
                                'attributes': {
                                    'ID': '0',
                                    'description': 'Geometrical description of one tank cross section.'},
                                'name': {
                                    'attributes': {
                                        'description': 'Name of tank cross section.'},
                                        'value': 'tank_section_0'},
                                'position': {
                                    'attributes': {
                                        'description': 'Position of tank cross section with regard to the local '
                                        'reference point.'},
                                    'x': {
                                        'attributes': {
                                            'description': 'Distance between the tank cross section and the local '
                                            'reference point in x-direction.'},
                                        'value': '0',
                                        'unit': 'm',
                                        'lower_boundary': '-80',
                                        'upper_boundary': '80'},
                                    'y': {
                                        'attributes': {
                                            'description': 'Distance between the tank cross section and the local '
                                            'reference point in y-direction.'},
                                        'value': '0',
                                        'unit': 'm',
                                        'lower_boundary': '-40',
                                        'upper_boundary': '40'},
                                    'z': {
                                        'attributes': {
                                            'description': 'Distance between the tank cross section and the local '
                                            'reference point in z-direction.'},
                                        'value': '0',
                                        'unit': 'm',
                                        'lower_boundary': '-5',
                                        'upper_boundary': '5'}},
                                'shape': {
                                    'attributes': {
                                        'description': 'Description of the shape of the cross section (circular, '
                                        'rectangular, elliptical).'},
                                    'value': 'misc'},
                                'height': {
                                    'attributes': {
                                        'description': 'Height of the cross section.'},
                                    'value': '0',
                                    'unit': 'm',
                                    'lower_boundary': '0',
                                    'upper_boundary': 'inf'},
                                'width': {
                                    'attributes': {
                                        'description': 'Width of the cross section.'},
                                    'value': '0',
                                    'unit': 'm',
                                    'lower_boundary': '0',
                                    'upper_boundary': 'inf'},
                                'length': {
                                    'attributes': {
                                        'description': 'Length of the cross section (if length greater than 0: curved '
                                        'cross section, e.g., dashed tank end cap).'},
                                    'value': '0',
                                    'unit': 'm',
                                    'lower_boundary': '0',
                                    'upper_boundary': 'inf'}
                            }
                        }
                    },
                    'additional_fuselage_length': {
                                    'attributes': {
                                        'description': 'Additional fuselage length (if smaller than 0: shrink '
                                        'fuselage, if greater than 0: stretch fuselage).'},
                                    'value': '0',
                                    'unit': 'm',
                                    'lower_boundary': '0',
                                    'upper_boundary': 'inf' 
                    }
                }
            }
        }
    }

    paths_and_names = prepare_element_tree_for_module_key_parameter(paths_and_names, module_key_parameters_dict)

    # Run 'user_method_data_output_preparation' from 'usermethoddatapreparation.py'.
    key_output_dict, method_specific_output_dict = routing_dict['func_user_method_data_output_preparation'](data_dict)
    # Extract tool level from routing dictionary.
    tool_level = routing_dict['tool_level']

    """Write data to aircraft exchange file."""
    # Extract root and path to aircraft exchange file.
    root_of_aircraft_exchange_tree = paths_and_names['root_of_aircraft_exchange_tree']
    path_to_aircraft_exchange_file = paths_and_names['path_to_aircraft_exchange_file']
    # Write key data to aircraft exchange file.
    write_key_data_to_aircraft_exchange_file(root_of_aircraft_exchange_tree, path_to_aircraft_exchange_file,
                                             paths_to_key_parameters_list, key_output_dict, tool_level, runtime_output)

    """Method-specific postprocessing."""
    # Run 'method_data_postprocessing' from 'datapostprocessingmodule'.
    data_dict['module_key_parameters_dict'] = module_key_parameters_dict
    data_dict['paths_to_key_parameters_list'] = paths_to_key_parameters_list
    method_data_postprocessing(paths_and_names, routing_dict, data_dict,
                               method_specific_output_dict, runtime_output)
