"""Module providing functions to read the energy carrier and tank configuration from the aircraft XML."""
# Import standard libraries.
import sys

# Import own libraries.
from pyunitconversion import constants


def read_energy_carrier_and_tank_configuration(paths_and_names, dict_ac_data, dict_tank_design, runtime_output):
    """Determine energy carrier dependent parameter.

    Add some more text here.

    This function adds the 'tank_configuration' dict to the 'dict_tank_design' that itself contains a dictionary per
    tank entity:
        tank_configuration = {'tank_0': {...}, 'tank_1': {...}, ...}
    
    These dictionaries (e.g., 'tank_0') contain the following parameter:
        - str energy_carrier_name: Name of energy carrier (e.g., 'kerosene')
        - str tank_location: Location of tank (e.g., 'wing')
        - str tank_position: Position of tank (related to given location) (e.g., 'inner_left')
        - float energy_share: Share of energy from this tank in total energy demand
        - float energy_required: Required energy amount per tank in J
        - float volume_available: Available volume in m^3
        - float energy_available: Energy available in J

    :param dict paths_and_names: Dictionary containing system paths and ElementTrees
    :param dict dict_ac_data: Dict containing parameter and values from aircraft exchange and module configuration file
    :param dict dict_tank_design: Dictionary containing tank design parameter
    :param logging.Logger runtime_output: Logging object used for capturing log messages in the module
    :return dict dict_tank_design: Dictionary containing tank design parameter
    """
    # Extract necessary values from 'dict_ac_data'.
    try:
        energy_carrier_name = dict_ac_data['energy_carrier_name']
        mission_energy_id = dict_ac_data['mission_energy_ID']
        mission_energy_amount = dict_ac_data['mission_energy_amount']
        tank_energy_id = dict_ac_data['tank_energy_ID']
        tank_location = dict_ac_data['tank_location']
        tank_position = dict_ac_data['tank_position']
        tank_energy_share = dict_ac_data['energy_share']
    except KeyError:
        runtime_output.critical('Error: Missing data in aircraft exchange file. Energy carrier and tank configuration '
                                +' preparation not possible. Program aborted.')
        sys.exit('Exit information: Missing data in aircraft exchange file. Energy carrier and tank configuration '
                 'preparation not possible!')

    # Extract energy carrier ID and according name.
    # Initial and normal configuration:
    # energy_carrier_lookup = {0: {'energy_carrier': 'kerosene'}, 1: {'energy_carrier': 'liquid_hydrogen'}}
    energy_carrier_lookup = {int(key.split('ID')[-1]): {'energy_carrier': value}
                             for key, value in energy_carrier_name.items()}
    
    gravimetric_energy_densities = {'kerosene': constants.KEROSENE_GRAVIMETRIC_ENERGY_DENSITY,
                                'liquid_hydrogen': constants.LIQUID_HYDROGEN_GRAVIMETRIC_ENERGY_DENSITY}
    volumetric_energy_densities = {'kerosene': constants.KEROSENE_VOLUMETRIC_ENERGY_DENSITY,
                                'liquid_hydrogen': constants.LIQUID_HYDROGEN_VOLUMETRIC_ENERGY_DENSITY}

    # Map mission energy ID to energy carrier ID and update the lookup dictionary.
    for key, val in mission_energy_id.items():
        mission_id = int(key.split('ID')[-1])
        if val is not None:
            energy_carrier_lookup[val]['energy_required'] = (
                mission_energy_amount[f'mission_energy_amount_ID{mission_id}'])
            # Add mass/volume required (using gravimetric/volumetric energy density).
            for entry in energy_carrier_lookup.values():
                name = entry.get('energy_carrier')
                # if name in gravimetric_energy_densities:
                entry['mass_required'] = entry['energy_required']/gravimetric_energy_densities[name]
                # if name in volumetric_energy_densities:
                entry['volume_required'] = entry['energy_required']/volumetric_energy_densities[name]

        # If information from mission analysis not available (None).
        else:
            number_of_energy_carrier = len(energy_carrier_lookup)
            if number_of_energy_carrier > 1:
                runtime_output.critical('Error: More than one energy carrier not accepted yet. '
                        + 'Program aborted.')
                sys.exit('Exit information: Too much energy carrier defined (hybrid not implemented yet)!')
            else:
                for i in range(number_of_energy_carrier):
                    energy_carrier_lookup[i]['energy_required'] = (
                        mission_energy_amount[f'mission_energy_amount_ID{mission_id}'])

            # Set mass/volume required to None.
            for entry in energy_carrier_lookup.values():
                entry.update({'mass_required': None, 'volume_required': None})

    # Create tank dictionary.
    tank_dict = {
        f'tank_{identifier}': {
            'energy_carrier_id': tank_energy_id[f'tank_energy_ID_ID{identifier}'],
            'tank_location': tank_location[f'tank_location_ID{identifier}'],
            'tank_position': tank_position[f'tank_position_ID{identifier}'],
            'energy_share': tank_energy_share[f'energy_share_ID{identifier}']
        }
        for identifier in [key.split('ID')[-1] for key in tank_energy_id.keys()]
    }

    # List all tank energy carriers.
    tank_energy_carrier_ids = [value['energy_carrier_id'] for value in tank_dict.values()]
    invalid_information = any(x is None for x in tank_energy_carrier_ids)
    if invalid_information:
        runtime_output.critical('Error: Invalid or incomplete tank information. '
                                + 'Program aborted.')
        sys.exit('Exit information: Invalid or incomplete tank information!')

    # Update 'tank_dict' with values from 'energy_carrier_lookup' dict.
    try:
        # tank = {'energy_carrier_id': 0, 'tank_location': 'wing', 'tank_position': 'inner_left', 'energy_share': 0.0}
        # tank = {'energy_carrier_id': 0, 'tank_location': 'wing', 'tank_position': 'inner_left', 'energy_share': 0.0}
        for key, tank in tank_dict.items():
            energy_carrier_id = tank['energy_carrier_id']
            energy_carrier_data = energy_carrier_lookup[energy_carrier_id]
            energy_share = tank['energy_share']

            if tank_energy_carrier_ids.count(energy_carrier_id) == 1:
                tank.update(energy_carrier_data)
            else:
                tank['energy_carrier_name'] = energy_carrier_data['energy_carrier']
                if energy_carrier_data['energy_required'] is not None:
                    tank['energy_required'] = energy_carrier_data['energy_required'] * energy_share
                else:
                    tank['energy_required'] = None

    except KeyError:
        runtime_output.critical('Error: Invalid energy carrier information. '
                                + 'Program aborted.')
        sys.exit('Exit information: Invalid energy carrier information!')

    # Create tank_configuration_dict with specified keys in the desired order.
    tank_configuration_dict = {
        key: {
            'energy_carrier_name': tank.get('energy_carrier_name',
                                            energy_carrier_lookup[tank['energy_carrier_id']]['energy_carrier']),
            'tank_location': tank['tank_location'],
            'tank_position': tank['tank_position'],
            'energy_share': tank['energy_share'],
            'energy_required': tank['energy_required'],
            'volume_available': 0.0,
            'energy_available': 0.0
        }
        for key, tank in tank_dict.items()
    }

    # Prepare output data.
    dict_tank_design = tank_configuration_dict

    # Debug print.
    runtime_output.debug('Debug: Energy carrier extracted and tank configuration dictionaries prepared.')

    return dict_tank_design
