# Import standard libraries.
import numpy as np

# Import own modules.
from src.tube_and_wing.empirical.tank_design_tu_berlin.liquid_hydrogen.calculateparameterendcap import calculate_parameter_end_cap
from src.tube_and_wing.empirical.tank_design_tu_berlin.liquid_hydrogen.calculatethicknesswall import calculate_thickness_wall
from src.tube_and_wing.empirical.tank_design_tu_berlin.liquid_hydrogen.calculateparametertruncatedcone import calculate_parameter_truncated_cone
from src.tube_and_wing.empirical.tank_design_tu_berlin.general.calculatecenterofgravity import calculate_center_of_gravity


def calculate_conical_tank(tank_entity, dict_ac_data, dict_tank_design, runtime_output):
    """Calculate volume and masses of conical tank.

        [Add some text here.]

    :param str tank_entity: ...
    :param float volume_required: ...
    :param dict dict_ac_data: Dict containing parameter and values from aircraft exchange and module configuration file
    :param dict dict_tank_design: Dict containing results from tank design
    :param logging.Logger runtime_output: Logging object used for capturing log messages in the module
    :return float list: x coordinates, external fuselage widths in m, external fuselage heights in m
    """
    # Extract data from 'dict_ac_data'.
    mass_pumps = dict_ac_data['mass_pumps']
    mass_baffle = dict_ac_data['mass_baffle']
    type_end_cap = dict_ac_data['type_end_cap']
    density_wall = dict_ac_data['density_wall']
    material_wall = dict_ac_data['material_wall']
    internal_pressure = dict_ac_data['internal_pressure']
    density_insulation = dict_ac_data['density_insulation']
    factor_volume_allowance = dict_ac_data['factor_volume_allowance']
    thickness_wall_calculation_method = dict_ac_data['thickness_wall_calculation_method']

    mass_vapor_barrier = 24  # TODO: Extract from tank_design_xml?
    thickness_wall = 0.003
    thickness_insulation = 0.20
    thickness_wall_and_insulation = thickness_wall + thickness_insulation
    tensile_stress_allowable = 172  # in N/mm TODO: Read from aircraft exchange file/lib/...
    thickness_wall_minimum_aluminum = 0.003  # AD-2000: minimum wall thickness AL-2219 is 3 mmm.

    """ Calculation. """
    max_length = round(dict_tank_design[tank_entity]['geometry']['box_dimensions']['maximum_length'],2)
    # Note: The diameters are actual sizes. This means that the thicknesses for the tank wall and insulation have
    # already been taken into account.
    # Assumption: Maximum outer tank diameter equals maximum useable fuselage diameter minus twice the
    # thickness of insulation and wall. The calculated maximum outer diameter is assumed to be the actual tank
    # diameter.
    max_diameter_front = (dict_tank_design[tank_entity]['geometry']['box_dimensions']['maximum_diameter_front']
                          - 2*thickness_wall_and_insulation)
    max_diameter_rear = (dict_tank_design[tank_entity]['geometry']['box_dimensions']['maximum_diameter_rear']
                         - 2*thickness_wall_and_insulation)
    diameters = np.linspace(max_diameter_front, max_diameter_rear,int(max_length*100))

    if dict_tank_design[tank_entity]['tank_position'] == 'tailcone':
        # Special case since the geometry of the tail remains the same across the iteration loops.
        # Find position of end caps.
        # The position of the bulkhead is known. This is assumed to be the rearmost position of a tank component,
        # i.e. the tank must end there. Due to this, the rear end cap must start slightly before this position, so
        # that it does not extend beyond the position of the bulkhead. The beginning is determined iteratively.
        # Starting from the position of the bulkhead, the distance (dist_from_bulkhead) is increased until it
        # equals the minimum limit (boundary). The latter depends on the type of end cap. In the case of a
        # hemispherical design, the required distance to the end cap equals half the diameter. For a torispherical
        # end, it must be at least equal to the determined length of the end.

        # Set iteration values.
        idx_rear = 1
        idx_front = 0
        increment = 0.01
        boundary_rear = 100
        boundary_front = 100
        distance_to_rear = 0
        distance_to_front = 0

        # Determine foremost position of front tank end cap.
        while boundary_front > distance_to_front:
            boundary_front = calculate_parameter_end_cap(diameters[idx_front], type_end_cap, thickness_wall,
                                                         thickness_insulation, density_wall, density_insulation)[3]
            maximum_tank_diameter_front = diameters[idx_front]
            idx_front += 1
            distance_to_front += increment

        # Determine rearmost position of rear tank end cap.
        while boundary_rear > distance_to_rear:
            boundary_rear = calculate_parameter_end_cap(diameters[-idx_rear], type_end_cap, thickness_wall,
                                                        thickness_insulation, density_wall, density_insulation)[3]
            maximum_tank_diameter_rear = diameters[-idx_rear]
            idx_rear += 1
            distance_to_rear += increment

    else:
        print('Normal conical tank.')

    """ Calculate wall thickness."""
    # Assumption: Only dependent on diameter (if pressure etc. constant) -> calculation for biggest diameter.
    # TODO: Adjust diameter to actual tank outside diameter? or stay conservative?
    maximum_diameter = max(maximum_tank_diameter_front, maximum_tank_diameter_rear)
    thickness_wall = calculate_thickness_wall(maximum_diameter, internal_pressure, tensile_stress_allowable,
                                              thickness_wall_calculation_method)
    if (material_wall == 'aluminum') and (thickness_wall < thickness_wall_minimum_aluminum):
        thickness_wall = thickness_wall_minimum_aluminum

    """ Calculate volumes, masses, and lengths of tank parts."""
    # Calculate maximum length of conical tank part w/o end cap.
    length_conical_tank_excl_end_caps = max_length - distance_to_front - distance_to_rear

    # Note that the lengths of the end caps already take into account the tank wall and the insulation thickness.
    # 1) Front end cap.
    (volume_tank_front_end_cap, mass_wall_front_end_cap, mass_insulation_front_end_cap,
     length_front_end_cap) = calculate_parameter_end_cap(maximum_tank_diameter_front, type_end_cap, thickness_wall,
                                                         thickness_insulation, density_wall, density_insulation)
    # 2) Conical tank section.
    volume_tank_conical_section = 0
    (volume_tank_conical_section, mass_wall_conical_section,
     mass_insulation_conical_section) = calculate_parameter_truncated_cone(maximum_tank_diameter_front,
                                                                           maximum_tank_diameter_rear,
                                                                            length_conical_tank_excl_end_caps,
                                                                            thickness_wall, thickness_insulation,
                                                                            density_wall, density_insulation)
    # 3) Rear end cap.
    (volume_tank_rear_end_cap, mass_wall_rear_end_cap, mass_insulation_rear_end_cap,
     length_rear_end_cap) = calculate_parameter_end_cap(maximum_tank_diameter_rear, type_end_cap, thickness_wall,
                                                         thickness_insulation, density_wall, density_insulation)
    # 4) Total tank.
    volume_tank_available = volume_tank_front_end_cap + volume_tank_conical_section + volume_tank_rear_end_cap
    volume_tank_usable = volume_tank_available/(1 + factor_volume_allowance)
    energy_hydrogen_available = volume_tank_usable * dict_ac_data['liquid_hydrogen_volumetric_energy_density']
    mass_hydrogen_available = energy_hydrogen_available/dict_ac_data['liquid_hydrogen_gravimetric_energy_density']

    """ Calculate overall values."""
    # Calculate mass of tank wall.
    mass_wall = mass_wall_front_end_cap + mass_wall_conical_section + mass_wall_rear_end_cap
    # Calculate structural mass of tank.
    mass_tank_structure = mass_wall + mass_pumps + mass_baffle
    # Calculate mass of insulation.
    mass_insulation = ((mass_insulation_front_end_cap + mass_insulation_conical_section + mass_insulation_rear_end_cap)
                       + mass_vapor_barrier)
    # Calculate overall tank mass.
    mass_tank_overall = mass_tank_structure + mass_insulation
    # Calculate overall tank length.
    length_tank_overall = length_front_end_cap + length_conical_tank_excl_end_caps + length_rear_end_cap
    # Calculate gravimetric efficiency tank.
    gravimetric_efficiency = mass_hydrogen_available/(mass_hydrogen_available + mass_tank_overall)

    """ Calculate center of gravity."""
    (x_coordinate_center_of_gravity, y_coordinate_center_of_gravity, z_coordinate_center_of_gravity
     ) = calculate_center_of_gravity(dict_tank_design[tank_entity]['tank_shape'],
         dict_tank_design[tank_entity]['geometry']['box_dimensions']['maximum_diameter_front'],
         dict_tank_design[tank_entity]['geometry']['box_dimensions']['maximum_diameter_rear'],
         length_conical_tank_excl_end_caps)

    # Prepare data output.
    dict_tank_design[tank_entity]['energy_available'] = float(energy_hydrogen_available)
    dict_tank_design[tank_entity]['masses'] = {}
    dict_tank_design[tank_entity]['masses']['mass_insulation'] = float(mass_insulation)
    dict_tank_design[tank_entity]['masses']['mass_tank_overall'] = float(mass_tank_overall)
    dict_tank_design[tank_entity]['masses']['mass_tank_structure'] = float(mass_tank_structure)
    dict_tank_design[tank_entity]['geometry']['length_tank_overall'] = float(length_tank_overall)
    dict_tank_design[tank_entity]['geometry']['center_of_gravity'] = {}
    dict_tank_design[tank_entity]['geometry']['center_of_gravity']['x'] = float(x_coordinate_center_of_gravity)
    dict_tank_design[tank_entity]['geometry']['center_of_gravity']['y'] = float(y_coordinate_center_of_gravity)
    dict_tank_design[tank_entity]['geometry']['center_of_gravity']['z'] = float(z_coordinate_center_of_gravity)
    dict_tank_design[tank_entity]['gravimetric_efficiency'] = float(gravimetric_efficiency)

    return dict_tank_design
