"""File providing functions to read layer text from aircraft XML file."""
# Import standard libraries.
import sys


def read_energy_carrier(root_of_aircraft_exchange_tree, runtime_output):
    """Read energy carrier from aircraft exchange file.

    This function extracts information about the energy carrier used in an aircraft from the provided aircraft exchange
    file. It specifically looks for 'propulsor' nodes and their corresponding 'energy_carrier' sub-nodes.

    :param ElementTree root_of_aircraft_exchange_tree: Root of aircraft exchange XML
    :param logging.Logger runtime_output: Logging object used for capturing log messages in the module
    :raises ValueError: Raised if propulsor or energy carrier node does not exist
    :return string energy_carrier: Energy carrier string
    """

    # Initialize empty list.
    energy_carrier_list = []

    # Attempt to extract information on energy carrier from aircraft exchange file.
    try:
        # Find all 'propulsor' nodes in aircraft exchange file.
        propulsor_node_list = root_of_aircraft_exchange_tree.findall('.//propulsor')
        # Check, if propulsor nodes exist.
        if not propulsor_node_list:
            # Raise error, if no propulsor node exists.
            raise ValueError('No propulsor nodes found in the aircraft exchange file. Program aborted.')
        # Iterate over 'propulsor_node_list' and append values of energy carrier sub-nodes.
        for propulsor_node in propulsor_node_list:
            energy_carrier_node = propulsor_node.find('.//energy_carrier/value')
            if energy_carrier_node is not None:
                energy_carrier_list.append(energy_carrier_node.text)
            else:
                raise ValueError('No energy carrier nodes found in the aircraft exchange file. Program aborted.')

        # If 'energy_carrier_list' is not empty, compare all entries.
        if energy_carrier_list is not None:
            # If all entries are the same, set 'energy_carrier' to first list entry.
            if all(element == energy_carrier_list[0] for element in energy_carrier_list):
                energy_carrier = energy_carrier_list[0]
            # If list entries differ, set 'energy_carrier' to 'hybrid'.
            else:
                energy_carrier = 'hybrid'
        # Raise error, if 'energy_carrier_list' is empty.
        else:
            raise ValueError('No energy carrier node found in propulsor nodes. Program aborted.')

    # Exception handling for ValueError.
    except ValueError as e:
        runtime_output.critical('Error: ' + str(e))
        sys.exit(1)

    return energy_carrier


def read_engine_configuration(root_of_aircraft_exchange_tree, runtime_output):
    """Read engine configuration.

    Read engine configuration from aircraft exchange file.

    :param ElementTree root_of_aircraft_exchange_tree: Root of aircraft exchange XML
    :param logging.Logger runtime_output: Logging object used for capturing log messages in the module
    :return str engine_configuration: Information on engine configuration
    """

    engine_configuration = 'xyz'
    print(engine_configuration)

    return engine_configuration


def read_tank_configuration(root_of_aircraft_exchange_tree, runtime_output):
    """Read tank configuration.

    Read tank configuration information from aircraft exchange file.

    :param ElementTree root_of_aircraft_exchange_tree: Root of aircraft exchange XML
    :param logging.Logger runtime_output: Logging object used for capturing log messages in the module
    :return str tank_configuration: Information on tank configuration
    """

    tank_configuration = 'xyz'
    print(tank_configuration)

    return tank_configuration
