/**
 * \file icao_faa_codes.h
 * \author Christopher Ruwisch (christopher.ruwisch@tu-berlin.de)
 * \brief ICAO Aerodrome Reference Code and FAA aircraft design group
 * \version 0.1
 * \date 2023-12-20
 *
 * @copyright Copyright (c) 2023
 *
 */
#ifndef ICAO_FAA_CODES
#define ICAO_FAA_CODES

#include <runtimeInfo/runtimeInfo.h>

#include <cctype>
#include <limits>
#include <map>
#include <string>
#include <tuple>

std::map<std::string, std::tuple<double, double>> icao_easa_aerodrome_reference_code(const std::string& code);
std::tuple<double, double> icao_easa_reference_field_length(const std::string& code_number);
std::tuple<double, double> icao_easa_reference_wing_span(const std::string& code_letter);
std::tuple<double, double> icao_aircraft_approach_category(const std::string& category);
std::tuple<double, double> faa_aircraft_design_group_tail_height(const std::string& code_letter);
std::tuple<double, double> faa_aircraft_design_group_wing_span(const std::string& code_letter);
bool icao_faa_code_consistency_check(std::string icao, std::string faa);

void print_limits(const std::string& limit_type, const std::string& code, const std::string& unit,
                  std::tuple<double, double> limits);

std::map<std::string, std::tuple<double, double>> icao_easa_aerodrome_reference_code(const std::string& code) {
  myRuntimeInfo->out << "ICAO EASA Aerodrome reference code: " << code << "\n";
  std::string reference_field_length_code_number;
  std::string reference_wing_span_code_letter;
  std::string reference_tail_height_code_number;
  std::string reference_approach_speed_letter;

  // Set code numbers and letters
  reference_field_length_code_number.push_back(code.at(0));
  reference_wing_span_code_letter.push_back(code.at(1));
  for (size_t i = 0; i < code.size() - 3; ++i) {
    reference_tail_height_code_number.push_back(code.at(2 + i));
  }
  reference_approach_speed_letter.push_back(code.at(code.size() - 1));

  // Check if wing_span code letter and tail height code letter fit
  icao_faa_code_consistency_check(reference_wing_span_code_letter, reference_tail_height_code_number);

  // map limits
  std::map<std::string, std::tuple<double, double>> limits;
  limits["field_length"] = icao_easa_reference_field_length(reference_field_length_code_number);
  limits["wing_span"] = icao_easa_reference_wing_span(reference_wing_span_code_letter);
  limits["tail_height"] = faa_aircraft_design_group_tail_height(reference_tail_height_code_number);
  limits["approach_speed"] = icao_aircraft_approach_category(reference_approach_speed_letter);
  return limits;
}

std::tuple<double, double> icao_easa_reference_field_length(const std::string& code_number) {
  std::map<std::string, std::tuple<double, double>> reference_field_length = {
      {"1", {0., 800.}}, {"2", {800., 1200.}}, {"3", {1200, 1800.}}, {"4", {1800, std::numeric_limits<double>::max()}}};
  if ((code_number != "1" && code_number != "2" && code_number != "3" && code_number != "4")) {
    throw("Error - " + code_number + " not valid");
  } else {
    print_limits("ICAO field length", code_number, "m", reference_field_length[code_number]);
    return reference_field_length[code_number];
  }
}

std::tuple<double, double> icao_easa_reference_wing_span(const std::string& code_letter) {
  std::map<std::string, std::tuple<double, double>> wing_span = {
      {"A", {0., 15.}}, {"B", {15., 24.}}, {"C", {24., 36.}}, {"D", {36., 52.}}, {"E", {52., 65.}}, {"F", {65., 80.}}};
  if (code_letter != "A" && code_letter != "B" && code_letter != "C" && code_letter != "D" && code_letter != "E" &&
      code_letter != "F") {
    throw("Error - " + code_letter + " not valid");
  } else {
    print_limits("ICAO wing span", code_letter, "m", wing_span[code_letter]);
    return wing_span[code_letter];
  }
}

std::tuple<double, double> icao_aircraft_approach_category(const std::string& category) {
  std::map<std::string, std::tuple<double, double>> aircraft_approach_speed = {
      {"A", {0., 169.}}, {"B", {169., 224.}}, {"C", {224., 261.}}, {"D", {261., 307.}}};
  if (category != "A" && category != "B" && category != "C" && category != "D") {
    throw("Error - " + category + " not valid");
  } else {
    print_limits("ICAO aircraft approach speed", category, "m/s", aircraft_approach_speed[category]);
    return aircraft_approach_speed[category];
  }
}

std::tuple<double, double> faa_aircraft_design_group_tail_height(const std::string& code_letter) {
  std::map<std::string, std::tuple<double, double>> tail_height = {{"I", {0., 6.096}},       {"II", {6.096, 9.144}},
                                                                   {"III", {9.144, 13.716}}, {"IV", {13.716, 18.288}},
                                                                   {"V", {18.288, 20.1168}}, {"VI", {20.1168, 24.384}}};
  if (code_letter != "I" && code_letter != "II" && code_letter != "III" && code_letter != "IV" && code_letter != "V" &&
      code_letter != "VI") {
    throw("Error - " + code_letter + " not valid");
  } else {
    print_limits("FAA ADG tail height", code_letter, "m", tail_height[code_letter]);
    return tail_height[code_letter];
  }
}

std::tuple<double, double> faa_aircraft_design_group_wing_span(const std::string& code_letter) {
  std::map<std::string, std::tuple<double, double>> wing_span = {{"I", {0.000, 15.}}, {"II", {15., 24.}},
                                                                 {"III", {24., 36.}}, {"IV", {36., 52.}},
                                                                 {"V", {52., 65.}},   {"VI", {65., 80.}}};
  if (code_letter != "I" && code_letter != "II" && code_letter != "III" && code_letter != "IV" && code_letter != "V" &&
      code_letter != "VI") {
    throw("Error - " + code_letter + " not valid");
  } else {
    print_limits("FAA ADG wing span", code_letter, "m", wing_span[code_letter]);
    return wing_span[code_letter];
  }
}

void print_limits(const std::string& limit_type, const std::string& code, const std::string& unit,
                  std::tuple<double, double> limits) {
  const auto [lower, upper] = limits;
  myRuntimeInfo->out << "\tLimts for  : " << limit_type << "\n";
  myRuntimeInfo->out << "\tCode       : " << code << "\n";
  myRuntimeInfo->out << "\tLower limit: " << lower << "\n";
  myRuntimeInfo->out << "\tUpper limit: " << upper << "\n";
}

bool icao_faa_code_consistency_check(std::string icao, std::string faa) {
  std::map<std::string, std::string> to_check{{"A", "I"},  {"B", "II"}, {"C", "III"},
                                              {"D", "IV"}, {"E", "V"},  {"F", "VI"}};

  if (to_check[icao] != faa) {
    throw("Error - ICAO and FAA code not consistent");
  } else {
    return true;
  }
}

#endif  // ICAO_FAA_CODES
